@@ -1,25 +1,25 @@
-from __future__ import unicode_literals
-
 import datetime
 
 from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
-from django.contrib.admin.utils import (lookup_field, display_for_field,
-    display_for_value, label_for_field)
-from django.contrib.admin.views.main import (ALL_VAR, EMPTY_CHANGELIST_VALUE,
-    ORDER_VAR, PAGE_VAR, SEARCH_VAR)
-from django.contrib.admin.templatetags.admin_static import static
+from django.contrib.admin.utils import (
+    display_for_field, display_for_value, label_for_field, lookup_field,
+)
+from django.contrib.admin.views.main import (
+    ALL_VAR, ORDER_VAR, PAGE_VAR, SEARCH_VAR,
+)
 from django.core.exceptions import ObjectDoesNotExist
-from django.core.urlresolvers import NoReverseMatch
 from django.db import models
-from django.utils import formats
-from django.utils.html import escapejs, format_html
-from django.utils.safestring import mark_safe
-from django.utils.text import capfirst
-from django.utils.translation import ugettext as _
-from django.utils.encoding import force_text
 from django.template import Library
 from django.template.loader import get_template
-from django.template.context import Context
+from django.templatetags.static import static
+from django.urls import NoReverseMatch
+from django.utils import formats
+from django.utils.html import format_html
+from django.utils.safestring import mark_safe
+from django.utils.text import capfirst
+from django.utils.translation import gettext as _
+
+from .base import InclusionAdminNode
 
 register = Library()
 
@@ -29,23 +29,24 @@ DOT = '.'
 @register.simple_tag
 def paginator_number(cl, i):
     """
-    Generates an individual page index link in a paginated list.
+    Generate an individual page index link in a paginated list.
     """
     if i == DOT:
-        return '... '
+        return '… '
     elif i == cl.page_num:
-        return format_html('<span class="this-page">{0}</span> ', i + 1)
+        return format_html('<span class="this-page">{}</span> ', i + 1)
     else:
-        return format_html('<a href="{0}"{1}>{2}</a> ',
-                           cl.get_query_string({PAGE_VAR: i}),
-                           mark_safe(' class="end"' if i == cl.paginator.num_pages - 1 else ''),
-                           i + 1)
+        return format_html(
+            '<a href="{}"{}>{}</a> ',
+            cl.get_query_string({PAGE_VAR: i}),
+            mark_safe(' class="end"' if i == cl.paginator.num_pages - 1 else ''),
+            i + 1,
+        )
 
 
-@register.inclusion_tag('admin/pagination.html')
 def pagination(cl):
     """
-    Generates the series of links to the pages in a paginated list.
+    Generate the series of links to the pages in a paginated list.
     """
     paginator, page_num = cl.paginator, cl.page_num
 
@@ -66,15 +67,17 @@ def pagination(cl):
             # ON_EACH_SIDE links at either end of the "current page" link.
             page_range = []
             if page_num > (ON_EACH_SIDE + ON_ENDS):
-                page_range.extend(range(0, ON_ENDS))
-                page_range.append(DOT)
-                page_range.extend(range(page_num - ON_EACH_SIDE, page_num + 1))
+                page_range += [
+                    *range(0, ON_ENDS), DOT,
+                    *range(page_num - ON_EACH_SIDE, page_num + 1),
+                ]
             else:
                 page_range.extend(range(0, page_num + 1))
             if page_num < (paginator.num_pages - ON_EACH_SIDE - ON_ENDS - 1):
-                page_range.extend(range(page_num + 1, page_num + ON_EACH_SIDE + 1))
-                page_range.append(DOT)
-                page_range.extend(range(paginator.num_pages - ON_ENDS, paginator.num_pages))
+                page_range += [
+                    *range(page_num + 1, page_num + ON_EACH_SIDE + 1), DOT,
+                    *range(paginator.num_pages - ON_ENDS, paginator.num_pages)
+                ]
             else:
                 page_range.extend(range(page_num + 1, paginator.num_pages))
 
@@ -89,9 +92,19 @@ def pagination(cl):
     }
 
 
+@register.tag(name='pagination')
+def pagination_tag(parser, token):
+    return InclusionAdminNode(
+        parser, token,
+        func=pagination,
+        template_name='pagination.html',
+        takes_context=False,
+    )
+
+
 def result_headers(cl):
     """
-    Generates the list column headers.
+    Generate the list column headers.
     """
     ordering_field_columns = cl.get_ordering_field_columns()
     for i, field_name in enumerate(cl.list_display):
@@ -100,7 +113,9 @@ def result_headers(cl):
             model_admin=cl.model_admin,
             return_attr=True
         )
+        is_field_sortable = cl.sortable_by is None or field_name in cl.sortable_by
         if attr:
+            field_name = _coerce_field_name(field_name, i)
             # Potentially not sortable
 
             # if the field is the action checkbox: no sorting and special class
@@ -114,23 +129,25 @@ def result_headers(cl):
 
             admin_order_field = getattr(attr, "admin_order_field", None)
             if not admin_order_field:
-                # Not sortable
-                yield {
-                    "text": text,
-                    "class_attrib": format_html(' class="column-{0}"', field_name),
-                    "sortable": False,
-                }
-                continue
+                is_field_sortable = False
+
+        if not is_field_sortable:
+            # Not sortable
+            yield {
+                'text': text,
+                'class_attrib': format_html(' class="column-{}"', field_name),
+                'sortable': False,
+            }
+            continue
 
         # OK, it is sortable if we got this far
-        th_classes = ['sortable', 'column-{0}'.format(field_name)]
+        th_classes = ['sortable', 'column-{}'.format(field_name)]
         order_type = ''
         new_order_type = 'asc'
         sort_priority = 0
-        sorted = False
         # Is it currently being sorted on?
-        if i in ordering_field_columns:
-            sorted = True
+        is_sorted = i in ordering_field_columns
+        if is_sorted:
             order_type = ordering_field_columns.get(i).lower()
             sort_priority = list(ordering_field_columns).index(i) + 1
             th_classes.append('sorted %sending' % order_type)
@@ -140,7 +157,9 @@ def result_headers(cl):
         o_list_primary = []  # URL for making this field the primary sort
         o_list_remove = []  # URL for removing this field from sort
         o_list_toggle = []  # URL for toggling order type for this field
-        make_qs_param = lambda t, n: ('-' if t == 'desc' else '') + str(n)
+
+        def make_qs_param(t, n):
+            return ('-' if t == 'desc' else '') + str(n)
 
         for j, ot in ordering_field_columns.items():
             if j == i:  # Same column
@@ -162,25 +181,36 @@ def result_headers(cl):
         yield {
             "text": text,
             "sortable": True,
-            "sorted": sorted,
+            "sorted": is_sorted,
             "ascending": order_type == "asc",
             "sort_priority": sort_priority,
             "url_primary": cl.get_query_string({ORDER_VAR: '.'.join(o_list_primary)}),
             "url_remove": cl.get_query_string({ORDER_VAR: '.'.join(o_list_remove)}),
             "url_toggle": cl.get_query_string({ORDER_VAR: '.'.join(o_list_toggle)}),
-            "class_attrib": format_html(' class="{0}"', ' '.join(th_classes)) if th_classes else '',
+            "class_attrib": format_html(' class="{}"', ' '.join(th_classes)) if th_classes else '',
         }
 
 
 def _boolean_icon(field_val):
-    icon_url = static('admin/img/icon-%s.gif' %
-                      {True: 'yes', False: 'no', None: 'unknown'}[field_val])
-    return format_html('<img src="{0}" alt="{1}" />', icon_url, field_val)
+    icon_url = static('admin/img/icon-%s.svg' % {True: 'yes', False: 'no', None: 'unknown'}[field_val])
+    return format_html('<img src="{}" alt="{}">', icon_url, field_val)
+
+
+def _coerce_field_name(field_name, field_index):
+    """
+    Coerce a field_name (which may be a callable) to a string.
+    """
+    if callable(field_name):
+        if field_name.__name__ == '<lambda>':
+            return 'lambda' + str(field_index)
+        else:
+            return field_name.__name__
+    return field_name
 
 
 def items_for_result(cl, result, form):
     """
-    Generates the actual list of data.
+    Generate the actual list of data.
     """
 
     def link_in_col(is_first, field_name, cl):
@@ -192,39 +222,34 @@ def items_for_result(cl, result, form):
 
     first = True
     pk = cl.lookup_opts.pk.attname
-    for field_name in cl.list_display:
-        row_classes = ['field-%s' % field_name]
+    for field_index, field_name in enumerate(cl.list_display):
+        empty_value_display = cl.model_admin.get_empty_value_display()
+        row_classes = ['field-%s' % _coerce_field_name(field_name, field_index)]
         try:
             f, attr, value = lookup_field(field_name, result, cl.model_admin)
         except ObjectDoesNotExist:
-            result_repr = EMPTY_CHANGELIST_VALUE
+            result_repr = empty_value_display
         else:
-            if f is None:
+            empty_value_display = getattr(attr, 'empty_value_display', empty_value_display)
+            if f is None or f.auto_created:
                 if field_name == 'action_checkbox':
                     row_classes = ['action-checkbox']
-                allow_tags = getattr(attr, 'allow_tags', False)
                 boolean = getattr(attr, 'boolean', False)
-                if boolean:
-                    allow_tags = True
-                result_repr = display_for_value(value, boolean)
-                # Strip HTML tags in the resulting text, except if the
-                # function has an "allow_tags" attribute set to True.
-                if allow_tags:
-                    result_repr = mark_safe(result_repr)
+                result_repr = display_for_value(value, empty_value_display, boolean)
                 if isinstance(value, (datetime.date, datetime.time)):
                     row_classes.append('nowrap')
             else:
-                if isinstance(f.rel, models.ManyToOneRel):
+                if isinstance(f.remote_field, models.ManyToOneRel):
                     field_val = getattr(result, f.name)
                     if field_val is None:
-                        result_repr = EMPTY_CHANGELIST_VALUE
+                        result_repr = empty_value_display
                     else:
                         result_repr = field_val
                 else:
-                    result_repr = display_for_field(value, f)
+                    result_repr = display_for_field(value, f, empty_value_display)
                 if isinstance(f, (models.DateField, models.TimeField, models.ForeignKey)):
                     row_classes.append('nowrap')
-        if force_text(result_repr) == '':
+        if str(result_repr) == '':
             result_repr = mark_safe('&nbsp;')
         row_class = mark_safe(' class="%s"' % ' '.join(row_classes))
         # If list_display_links not defined, add the link tag to the first field
@@ -241,24 +266,21 @@ def items_for_result(cl, result, form):
             else:
                 url = add_preserved_filters({'preserved_filters': cl.preserved_filters, 'opts': cl.opts}, url)
                 # Convert the pk to something that can be used in Javascript.
-                # Problem cases are long ints (23L) and non-ASCII strings.
+                # Problem cases are non-ASCII strings.
                 if cl.to_field:
                     attr = str(cl.to_field)
                 else:
                     attr = pk
                 value = result.serializable_value(attr)
-                result_id = escapejs(value)
                 link_or_text = format_html(
-                    '<a href="{0}"{1}>{2}</a>',
+                    '<a href="{}"{}>{}</a>',
                     url,
-                    format_html(' onclick="opener.dismissRelatedLookupPopup(window, &#39;{0}&#39;); return false;"', result_id) if cl.is_popup else '',
+                    format_html(
+                        ' data-popup-opener="{}"', value
+                    ) if cl.is_popup else '',
                     result_repr)
 
-            yield format_html('<{0}{1}>{2}</{3}>',
-                              table_tag,
-                              row_class,
-                              link_or_text,
-                              table_tag)
+            yield format_html('<{}{}>{}</{}>', table_tag, row_class, link_or_text, table_tag)
         else:
             # By default the fields come from ModelAdmin.list_editable, but if we pull
             # the fields out of the form instead of list_editable custom admins
@@ -267,20 +289,21 @@ def items_for_result(cl, result, form):
                     field_name == cl.model._meta.pk.name and
                     form[cl.model._meta.pk.name].is_hidden)):
                 bf = form[field_name]
-                result_repr = mark_safe(force_text(bf.errors) + force_text(bf))
-            yield format_html('<td{0}>{1}</td>', row_class, result_repr)
+                result_repr = mark_safe(str(bf.errors) + str(bf))
+            yield format_html('<td{}>{}</td>', row_class, result_repr)
     if form and not form[cl.model._meta.pk.name].is_hidden:
-        yield format_html('<td>{0}</td>', force_text(form[cl.model._meta.pk.name]))
+        yield format_html('<td>{}</td>', form[cl.model._meta.pk.name])
 
 
 class ResultList(list):
-    # Wrapper class used to return items in a list_editable
-    # changelist, annotated with the form object for error
-    # reporting purposes. Needed to maintain backwards
-    # compatibility with existing admin templates.
+    """
+    Wrapper class used to return items in a list_editable changelist, annotated
+    with the form object for error reporting purposes. Needed to maintain
+    backwards compatibility with existing admin templates.
+    """
     def __init__(self, form, *items):
         self.form = form
-        super(ResultList, self).__init__(*items)
+        super().__init__(*items)
 
 
 def results(cl):
@@ -296,35 +319,43 @@ def result_hidden_fields(cl):
     if cl.formset:
         for res, form in zip(cl.result_list, cl.formset.forms):
             if form[cl.model._meta.pk.name].is_hidden:
-                yield mark_safe(force_text(form[cl.model._meta.pk.name]))
+                yield mark_safe(form[cl.model._meta.pk.name])
 
 
-@register.inclusion_tag("admin/change_list_results.html")
 def result_list(cl):
     """
-    Displays the headers and data list together
+    Display the headers and data list together.
     """
     headers = list(result_headers(cl))
     num_sorted_fields = 0
     for h in headers:
         if h['sortable'] and h['sorted']:
             num_sorted_fields += 1
-    return {'cl': cl,
-            'result_hidden_fields': list(result_hidden_fields(cl)),
-            'result_headers': headers,
-            'num_sorted_fields': num_sorted_fields,
-            'results': list(results(cl))}
+    return {
+        'cl': cl,
+        'result_hidden_fields': list(result_hidden_fields(cl)),
+        'result_headers': headers,
+        'num_sorted_fields': num_sorted_fields,
+        'results': list(results(cl)),
+    }
+
+
+@register.tag(name='result_list')
+def result_list_tag(parser, token):
+    return InclusionAdminNode(
+        parser, token,
+        func=result_list,
+        template_name='change_list_results.html',
+        takes_context=False,
+    )
 
 
-@register.inclusion_tag('admin/date_hierarchy.html')
 def date_hierarchy(cl):
     """
-    Displays the date hierarchy for date drill-down functionality.
+    Display the date hierarchy for date drill-down functionality.
     """
     if cl.date_hierarchy:
         field_name = cl.date_hierarchy
-        field = cl.opts.get_field_by_name(field_name)[0]
-        dates_or_datetimes = 'datetimes' if isinstance(field, models.DateTimeField) else 'dates'
         year_field = '%s__year' % field_name
         month_field = '%s__month' % field_name
         day_field = '%s__day' % field_name
@@ -333,7 +364,8 @@ def date_hierarchy(cl):
         month_lookup = cl.params.get(month_field)
         day_lookup = cl.params.get(day_field)
 
-        link = lambda filters: cl.get_query_string(filters, [field_generic])
+        def link(filters):
+            return cl.get_query_string(filters, [field_generic])
 
         if not (year_lookup or month_lookup or day_lookup):
             # select appropriate start level
@@ -356,8 +388,7 @@ def date_hierarchy(cl):
                 'choices': [{'title': capfirst(formats.date_format(day, 'MONTH_DAY_FORMAT'))}]
             }
         elif year_lookup and month_lookup:
-            days = cl.queryset.filter(**{year_field: year_lookup, month_field: month_lookup})
-            days = getattr(days, dates_or_datetimes)(field_name, 'day')
+            days = getattr(cl.queryset, 'dates')(field_name, 'day')
             return {
                 'show': True,
                 'back': {
@@ -370,8 +401,7 @@ def date_hierarchy(cl):
                 } for day in days]
             }
         elif year_lookup:
-            months = cl.queryset.filter(**{year_field: year_lookup})
-            months = getattr(months, dates_or_datetimes)(field_name, 'month')
+            months = getattr(cl.queryset, 'dates')(field_name, 'month')
             return {
                 'show': True,
                 'back': {
@@ -384,9 +414,10 @@ def date_hierarchy(cl):
                 } for month in months]
             }
         else:
-            years = getattr(cl.queryset, dates_or_datetimes)(field_name, 'year')
+            years = getattr(cl.queryset, 'dates')(field_name, 'year')
             return {
                 'show': True,
+                'back': None,
                 'choices': [{
                     'link': link({year_field: str(year.year)}),
                     'title': str(year.year),
@@ -394,10 +425,19 @@ def date_hierarchy(cl):
             }
 
 
-@register.inclusion_tag('admin/search_form.html')
+@register.tag(name='date_hierarchy')
+def date_hierarchy_tag(parser, token):
+    return InclusionAdminNode(
+        parser, token,
+        func=date_hierarchy,
+        template_name='date_hierarchy.html',
+        takes_context=False,
+    )
+
+
 def search_form(cl):
     """
-    Displays a search form for searching the list.
+    Display a search form for searching the list.
     """
     return {
         'cl': cl,
@@ -406,17 +446,21 @@ def search_form(cl):
     }
 
 
+@register.tag(name='search_form')
+def search_form_tag(parser, token):
+    return InclusionAdminNode(parser, token, func=search_form, template_name='search_form.html', takes_context=False)
+
+
 @register.simple_tag
 def admin_list_filter(cl, spec):
     tpl = get_template(spec.template)
-    return tpl.render(Context({
+    return tpl.render({
         'title': spec.title,
         'choices': list(spec.choices(cl)),
         'spec': spec,
-    }))
+    })
 
 
-@register.inclusion_tag('admin/actions.html', takes_context=True)
 def admin_actions(context):
     """
     Track the number of times the action field has been rendered on the page,
@@ -424,3 +468,18 @@ def admin_actions(context):
     """
     context['action_index'] = context.get('action_index', -1) + 1
     return context
+
+
+@register.tag(name='admin_actions')
+def admin_actions_tag(parser, token):
+    return InclusionAdminNode(parser, token, func=admin_actions, template_name='actions.html')
+
+
+@register.tag(name='change_list_object_tools')
+def change_list_object_tools_tag(parser, token):
+    """Display the row of change list object tools."""
+    return InclusionAdminNode(
+        parser, token,
+        func=lambda context: context,
+        template_name='change_list_object_tools.html',
+    )
