@@ -3,6 +3,7 @@ from MySQLdb.constants import FIELD_TYPE
 from django.contrib.gis.gdal import OGRGeomType
 from django.db.backends.mysql.introspection import DatabaseIntrospection
 
+
 class MySQLIntrospection(DatabaseIntrospection):
     # Updating the data_types_reverse dictionary with the appropriate
     # type for Geometry fields.
@@ -10,8 +11,7 @@ class MySQLIntrospection(DatabaseIntrospection):
     data_types_reverse[FIELD_TYPE.GEOMETRY] = 'GeometryField'
 
     def get_geometry_type(self, table_name, geo_col):
-        cursor = self.connection.cursor()
-        try:
+        with self.connection.cursor() as cursor:
             # In order to get the specific geometry type of the field,
             # we introspect on the table definition using `DESCRIBE`.
             cursor.execute('DESCRIBE %s' %
@@ -26,7 +26,13 @@ class MySQLIntrospection(DatabaseIntrospection):
                     field_type = OGRGeomType(typ).django
                     field_params = {}
                     break
-        finally:
-            cursor.close()
-
         return field_type, field_params
+
+    def supports_spatial_index(self, cursor, table_name):
+        # Supported with MyISAM/Aria, or InnoDB on MySQL 5.7.5+/MariaDB 10.2.2+
+        storage_engine = self.get_storage_engine(cursor, table_name)
+        if storage_engine == 'InnoDB':
+            return self.connection.mysql_version >= (
+                (10, 2, 2) if self.connection.mysql_is_mariadb else (5, 7, 5)
+            )
+        return storage_engine in ('MyISAM', 'Aria')
