@@ -1,9 +1,6 @@
-import sys
-
 import cx_Oracle
 
 from django.db.backends.oracle.introspection import DatabaseIntrospection
-from django.utils import six
 
 
 class OracleIntrospection(DatabaseIntrospection):
@@ -14,8 +11,7 @@ class OracleIntrospection(DatabaseIntrospection):
     data_types_reverse[cx_Oracle.OBJECT] = 'GeometryField'
 
     def get_geometry_type(self, table_name, geo_col):
-        cursor = self.connection.cursor()
-        try:
+        with self.connection.cursor() as cursor:
             # Querying USER_SDO_GEOM_METADATA to get the SRID and dimension information.
             try:
                 cursor.execute(
@@ -24,12 +20,11 @@ class OracleIntrospection(DatabaseIntrospection):
                     (table_name.upper(), geo_col.upper())
                 )
                 row = cursor.fetchone()
-            except Exception as msg:
-                new_msg = (
+            except Exception as exc:
+                raise Exception(
                     'Could not find entry in USER_SDO_GEOM_METADATA '
-                    'corresponding to "%s"."%s"\n'
-                    'Error message: %s.') % (table_name, geo_col, msg)
-                six.reraise(Exception, Exception(new_msg), sys.exc_info()[2])
+                    'corresponding to "%s"."%s"' % (table_name, geo_col)
+                ) from exc
 
             # TODO: Research way to find a more specific geometry field type for
             # the column's contents.
@@ -40,11 +35,8 @@ class OracleIntrospection(DatabaseIntrospection):
             dim, srid = row
             if srid != 4326:
                 field_params['srid'] = srid
-            # Length of object array ( SDO_DIM_ARRAY ) is number of dimensions.
-            dim = len(dim)
+            # Size of object array (SDO_DIM_ARRAY) is number of dimensions.
+            dim = dim.size()
             if dim != 2:
                 field_params['dim'] = dim
-        finally:
-            cursor.close()
-
         return field_type, field_params
