@@ -1,7 +1,8 @@
 from django.conf import settings
-from django.contrib.sessions.backends.base import CreateError, SessionBase
+from django.contrib.sessions.backends.base import (
+    CreateError, SessionBase, UpdateError,
+)
 from django.core.cache import caches
-from django.utils.six.moves import range
 
 KEY_PREFIX = "django.contrib.sessions.cache"
 
@@ -10,17 +11,19 @@ class SessionStore(SessionBase):
     """
     A cache-based session store.
     """
+    cache_key_prefix = KEY_PREFIX
+
     def __init__(self, session_key=None):
         self._cache = caches[settings.SESSION_CACHE_ALIAS]
-        super(SessionStore, self).__init__(session_key)
+        super().__init__(session_key)
 
     @property
     def cache_key(self):
-        return KEY_PREFIX + self._get_or_create_session_key()
+        return self.cache_key_prefix + self._get_or_create_session_key()
 
     def load(self):
         try:
-            session_data = self._cache.get(self.cache_key, None)
+            session_data = self._cache.get(self.cache_key)
         except Exception:
             # Some backends (e.g. memcache) raise an exception on invalid
             # cache keys. If this happens, reset the session. See #17810.
@@ -53,8 +56,10 @@ class SessionStore(SessionBase):
             return self.create()
         if must_create:
             func = self._cache.add
-        else:
+        elif self._cache.get(self.cache_key) is not None:
             func = self._cache.set
+        else:
+            raise UpdateError
         result = func(self.cache_key,
                       self._get_session(no_load=must_create),
                       self.get_expiry_age())
@@ -62,14 +67,14 @@ class SessionStore(SessionBase):
             raise CreateError
 
     def exists(self, session_key):
-        return session_key and (KEY_PREFIX + session_key) in self._cache
+        return bool(session_key) and (self.cache_key_prefix + session_key) in self._cache
 
     def delete(self, session_key=None):
         if session_key is None:
             if self.session_key is None:
                 return
             session_key = self.session_key
-        self._cache.delete(KEY_PREFIX + session_key)
+        self._cache.delete(self.cache_key_prefix + session_key)
 
     @classmethod
     def clear_expired(cls):
