@@ -7,12 +7,10 @@ import os
 import posixpath
 
 from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured
+from django.contrib.staticfiles import finders
 from django.http import Http404
-from django.utils.six.moves.urllib.parse import unquote
 from django.views import static
 
-from django.contrib.staticfiles import finders
 
 def serve(request, path, insecure=False, **kwargs):
     """
@@ -21,17 +19,17 @@ def serve(request, path, insecure=False, **kwargs):
 
     To use, put a URL pattern such as::
 
-        (r'^(?P<path>.*)$', 'django.contrib.staticfiles.views.serve')
+        from django.contrib.staticfiles import views
+
+        url(r'^(?P<path>.*)$', views.serve)
 
     in your URLconf.
 
-    It uses the django.views.static view to serve the found files.
+    It uses the django.views.static.serve() view to serve the found files.
     """
     if not settings.DEBUG and not insecure:
-        raise ImproperlyConfigured("The staticfiles view can only be used in "
-                                   "debug mode or if the --insecure "
-                                   "option of 'runserver' is used")
-    normalized_path = posixpath.normpath(unquote(path)).lstrip('/')
+        raise Http404
+    normalized_path = posixpath.normpath(path).lstrip('/')
     absolute_path = finders.find(normalized_path)
     if not absolute_path:
         if path.endswith('/') or path == '':
