@@ -10,36 +10,32 @@ Usage:
 7th October 2003 11:39
 >>>
 """
-from __future__ import unicode_literals
-
 import calendar
 import datetime
 import re
 import time
 
-from django.utils import six
 from django.utils.dates import (
     MONTHS, MONTHS_3, MONTHS_ALT, MONTHS_AP, WEEKDAYS, WEEKDAYS_ABBR,
 )
-from django.utils.encoding import force_text
 from django.utils.timezone import get_default_timezone, is_aware, is_naive
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 re_formatchars = re.compile(r'(?<!\\)([aAbBcdDeEfFgGhHiIjlLmMnNoOPrsStTUuwWyYzZ])')
 re_escaped = re.compile(r'\\(.)')
 
 
-class Formatter(object):
+class Formatter:
     def format(self, formatstr):
         pieces = []
-        for i, piece in enumerate(re_formatchars.split(force_text(formatstr))):
+        for i, piece in enumerate(re_formatchars.split(str(formatstr))):
             if i % 2:
                 if type(self.data) is datetime.date and hasattr(TimeFormat, piece):
                     raise TypeError(
                         "The format for date objects may not contain "
                         "time-related format specifiers (found '%s')." % piece
                     )
-                pieces.append(force_text(getattr(self, piece)()))
+                pieces.append(str(getattr(self, piece)()))
             elif piece:
                 pieces.append(re_escaped.sub(r'\1', piece))
         return ''.join(pieces)
@@ -80,17 +76,14 @@ class TimeFormat(Formatter):
         """
         Timezone name.
 
-        If timezone information is not available, this method returns
-        an empty string.
+        If timezone information is not available, return an empty string.
         """
         if not self.timezone:
             return ""
 
         try:
             if hasattr(self.data, 'tzinfo') and self.data.tzinfo:
-                # Have to use tzinfo.tzname and not datetime.tzname
-                # because datatime.tzname does not expect Unicode
-                return self.data.tzinfo.tzname(self.data) or ""
+                return self.data.tzname() or ''
         except NotImplementedError:
             pass
         return ""
@@ -130,12 +123,11 @@ class TimeFormat(Formatter):
         "Minutes; i.e. '00' to '59'"
         return '%02d' % self.data.minute
 
-    def O(self):
+    def O(self):  # NOQA: E743
         """
         Difference to Greenwich time in hours; e.g. '+0200', '-0430'.
 
-        If timezone information is not available, this method returns
-        an empty string.
+        If timezone information is not available, return an empty string.
         """
         if not self.timezone:
             return ""
@@ -168,8 +160,7 @@ class TimeFormat(Formatter):
         """
         Time zone of this machine; e.g. 'EST' or 'MDT'.
 
-        If timezone information is not available, this method returns
-        an empty string.
+        If timezone information is not available, return an empty string.
         """
         if not self.timezone:
             return ""
@@ -184,7 +175,7 @@ class TimeFormat(Formatter):
             pass
         if name is None:
             name = self.format('O')
-        return six.text_type(name)
+        return str(name)
 
     def u(self):
         "Microseconds; i.e. '000000' to '999999'"
@@ -196,8 +187,7 @@ class TimeFormat(Formatter):
         timezones west of UTC is always negative, and for those east of UTC is
         always positive.
 
-        If timezone information is not available, this method returns
-        an empty string.
+        If timezone information is not available, return an empty string.
         """
         if not self.timezone:
             return ""
@@ -247,7 +237,7 @@ class DateFormat(TimeFormat):
         "Month, textual, long; e.g. 'January'"
         return MONTHS[self.data.month]
 
-    def I(self):
+    def I(self):  # NOQA: E743
         "'1' if Daylight Savings Time, '0' otherwise."
         try:
             if self.timezone and self.timezone.dst(self.data):
@@ -264,7 +254,7 @@ class DateFormat(TimeFormat):
         "Day of the month without leading zeros; i.e. '1' to '31'"
         return self.data.day
 
-    def l(self):
+    def l(self):  # NOQA: E743
         "Day of the week, textual, long; e.g. 'Friday'"
         return WEEKDAYS[self.data.weekday()]
 
@@ -327,7 +317,6 @@ class DateFormat(TimeFormat):
     def W(self):
         "ISO-8601 week number of year, weeks starting on Monday"
         # Algorithm from http://www.personal.ecu.edu/mccartyr/ISOwdALG.txt
-        week_number = None
         jan1_weekday = self.data.replace(month=1, day=1).weekday() + 1
         weekday = self.data.weekday() + 1
         day_of_year = self.z()
@@ -352,14 +341,14 @@ class DateFormat(TimeFormat):
 
     def y(self):
         "Year, 2 digits; e.g. '99'"
-        return six.text_type(self.data.year)[2:]
+        return str(self.data.year)[2:]
 
     def Y(self):
         "Year, 4 digits; e.g. '1999'"
         return self.data.year
 
     def z(self):
-        "Day of the year; i.e. '0' to '365'"
+        """Day of the year, i.e. 1 to 366."""
         doy = self.year_days[self.data.month] + self.data.day
         if self.L() and self.data.month > 2:
             doy += 1
