@@ -2,14 +2,16 @@
 Timezone-related classes and functions.
 """
 
-from datetime import datetime, timedelta, tzinfo
+import functools
+import warnings
+from contextlib import ContextDecorator
+from datetime import datetime, timedelta, timezone, tzinfo
 from threading import local
 
 import pytz
 
 from django.conf import settings
-from django.utils import lru_cache, six
-from django.utils.decorators import ContextDecorator
+from django.utils.deprecation import RemovedInDjango31Warning
 
 __all__ = [
     'utc', 'get_fixed_timezone',
@@ -36,6 +38,10 @@ class FixedOffset(tzinfo):
     """
 
     def __init__(self, offset=None, name=None):
+        warnings.warn(
+            'FixedOffset is deprecated in favor of datetime.timezone',
+            RemovedInDjango31Warning, stacklevel=2,
+        )
         if offset is not None:
             self.__offset = timedelta(minutes=offset)
         if name is not None:
@@ -51,28 +57,26 @@ class FixedOffset(tzinfo):
         return ZERO
 
 
+# UTC time zone as a tzinfo instance.
 utc = pytz.utc
-"""UTC time zone as a tzinfo instance."""
 
 
 def get_fixed_timezone(offset):
-    """
-    Returns a tzinfo instance with a fixed offset from UTC.
-    """
+    """Return a tzinfo instance with a fixed offset from UTC."""
     if isinstance(offset, timedelta):
-        offset = offset.seconds // 60
+        offset = offset.total_seconds() // 60
     sign = '-' if offset < 0 else '+'
     hhmm = '%02d%02d' % divmod(abs(offset), 60)
     name = sign + hhmm
-    return FixedOffset(offset, name)
+    return timezone(timedelta(minutes=offset), name)
 
 
 # In order to avoid accessing settings at compile time,
 # wrap the logic in a function and cache the result.
-@lru_cache.lru_cache()
+@functools.lru_cache()
 def get_default_timezone():
     """
-    Returns the default time zone as a tzinfo instance.
+    Return the default time zone as a tzinfo instance.
 
     This is the time zone defined by settings.TIME_ZONE.
     """
@@ -81,9 +85,7 @@ def get_default_timezone():
 
 # This function exists for consistency with get_current_timezone_name
 def get_default_timezone_name():
-    """
-    Returns the name of the default time zone.
-    """
+    """Return the name of the default time zone."""
     return _get_timezone_name(get_default_timezone())
 
 
@@ -91,29 +93,18 @@ _active = local()
 
 
 def get_current_timezone():
-    """
-    Returns the currently active time zone as a tzinfo instance.
-    """
+    """Return the currently active time zone as a tzinfo instance."""
     return getattr(_active, "value", get_default_timezone())
 
 
 def get_current_timezone_name():
-    """
-    Returns the name of the currently active time zone.
-    """
+    """Return the name of the currently active time zone."""
     return _get_timezone_name(get_current_timezone())
 
 
 def _get_timezone_name(timezone):
-    """
-    Returns the name of ``timezone``.
-    """
-    try:
-        # for pytz timezones
-        return timezone.zone
-    except AttributeError:
-        # for regular tzinfo objects
-        return timezone.tzname(None)
+    """Return the name of ``timezone``."""
+    return timezone.tzname(None)
 
 # Timezone selection functions.
 
@@ -123,14 +114,14 @@ def _get_timezone_name(timezone):
 
 def activate(timezone):
     """
-    Sets the time zone for the current thread.
+    Set the time zone for the current thread.
 
     The ``timezone`` argument must be an instance of a tzinfo subclass or a
     time zone name.
     """
     if isinstance(timezone, tzinfo):
         _active.value = timezone
-    elif isinstance(timezone, six.string_types):
+    elif isinstance(timezone, str):
         _active.value = pytz.timezone(timezone)
     else:
         raise ValueError("Invalid timezone: %r" % timezone)
@@ -138,7 +129,7 @@ def activate(timezone):
 
 def deactivate():
     """
-    Unsets the time zone for the current thread.
+    Unset the time zone for the current thread.
 
     Django will then use the time zone defined by settings.TIME_ZONE.
     """
@@ -150,8 +141,8 @@ class override(ContextDecorator):
     """
     Temporarily set the time zone for the current thread.
 
-    This is a context manager that uses ``~django.utils.timezone.activate()``
-    to set the timezone on entry, and restores the previously active timezone
+    This is a context manager that uses django.utils.timezone.activate()
+    to set the timezone on entry and restores the previously active timezone
     on exit.
 
     The ``timezone`` argument must be an instance of a ``tzinfo`` subclass, a
@@ -179,7 +170,7 @@ class override(ContextDecorator):
 
 def template_localtime(value, use_tz=None):
     """
-    Checks if value is a datetime and converts it to local time if necessary.
+    Check if value is a datetime and converts it to local time if necessary.
 
     If use_tz is provided and is not None, that will force the value to
     be converted (or not), overriding the value of settings.USE_TZ.
@@ -199,7 +190,7 @@ def template_localtime(value, use_tz=None):
 
 def localtime(value=None, timezone=None):
     """
-    Converts an aware datetime.datetime to local time.
+    Convert an aware datetime.datetime to local time.
 
     Only aware datetimes are allowed. When value is omitted, it defaults to
     now().
@@ -214,11 +205,7 @@ def localtime(value=None, timezone=None):
     # Emulate the behavior of astimezone() on Python < 3.6.
     if is_naive(value):
         raise ValueError("localtime() cannot be applied to a naive datetime")
-    value = value.astimezone(timezone)
-    if hasattr(timezone, 'normalize'):
-        # This method is available for pytz time zones.
-        value = timezone.normalize(value)
-    return value
+    return value.astimezone(timezone)
 
 
 def localdate(value=None, timezone=None):
@@ -236,7 +223,7 @@ def localdate(value=None, timezone=None):
 
 def now():
     """
-    Returns an aware or naive datetime.datetime, depending on settings.USE_TZ.
+    Return an aware or naive datetime.datetime, depending on settings.USE_TZ.
     """
     if settings.USE_TZ:
         # timeit shows that datetime.now(tz=utc) is 24% slower
@@ -250,10 +237,10 @@ def now():
 
 def is_aware(value):
     """
-    Determines if a given datetime.datetime is aware.
+    Determine if a given datetime.datetime is aware.
 
     The concept is defined in Python's docs:
-    http://docs.python.org/library/datetime.html#datetime.tzinfo
+    https://docs.python.org/library/datetime.html#datetime.tzinfo
 
     Assuming value.tzinfo is either None or a proper datetime.tzinfo,
     value.utcoffset() implements the appropriate logic.
@@ -263,10 +250,10 @@ def is_aware(value):
 
 def is_naive(value):
     """
-    Determines if a given datetime.datetime is naive.
+    Determine if a given datetime.datetime is naive.
 
     The concept is defined in Python's docs:
-    http://docs.python.org/library/datetime.html#datetime.tzinfo
+    https://docs.python.org/library/datetime.html#datetime.tzinfo
 
     Assuming value.tzinfo is either None or a proper datetime.tzinfo,
     value.utcoffset() implements the appropriate logic.
@@ -275,9 +262,7 @@ def is_naive(value):
 
 
 def make_aware(value, timezone=None, is_dst=None):
-    """
-    Makes a naive datetime.datetime in a given time zone aware.
-    """
+    """Make a naive datetime.datetime in a given time zone aware."""
     if timezone is None:
         timezone = get_current_timezone()
     if hasattr(timezone, 'localize'):
@@ -293,16 +278,10 @@ def make_aware(value, timezone=None, is_dst=None):
 
 
 def make_naive(value, timezone=None):
-    """
-    Makes an aware datetime.datetime naive in a given time zone.
-    """
+    """Make an aware datetime.datetime naive in a given time zone."""
     if timezone is None:
         timezone = get_current_timezone()
     # Emulate the behavior of astimezone() on Python < 3.6.
     if is_naive(value):
         raise ValueError("make_naive() cannot be applied to a naive datetime")
-    value = value.astimezone(timezone)
-    if hasattr(timezone, 'normalize'):
-        # This method is available for pytz time zones.
-        value = timezone.normalize(value)
-    return value.replace(tzinfo=None)
+    return value.astimezone(timezone).replace(tzinfo=None)
