@@ -1,14 +1,16 @@
 from django import forms
 from django.contrib import admin
+from django.db import models
 
 from .models import (
-    Author, BinaryTree, CapoFamiglia, Chapter, ChildModel1, ChildModel2,
+    Author, BinaryTree, CapoFamiglia, Chapter, Child, ChildModel1, ChildModel2,
     Consigliere, EditablePKBook, ExtraTerrestrial, Fashionista, Holder,
     Holder2, Holder3, Holder4, Inner, Inner2, Inner3, Inner4Stacked,
     Inner4Tabular, NonAutoPKBook, NonAutoPKBookChild, Novel,
-    ParentModelWithCustomPk, Poll, Profile, ProfileCollection, Question,
-    ReadOnlyInline, ShoppingWeakness, Sighting, SomeChildModel,
-    SomeParentModel, SottoCapo, Title, TitleCollection,
+    NovelReadonlyChapter, ParentModelWithCustomPk, Poll, Profile,
+    ProfileCollection, Question, ReadOnlyInline, ShoppingWeakness, Sighting,
+    SomeChildModel, SomeParentModel, SottoCapo, Teacher, Title,
+    TitleCollection,
 )
 
 site = admin.AdminSite(name="admin")
@@ -73,8 +75,20 @@ class InnerInline2(admin.StackedInline):
         js = ('my_awesome_inline_scripts.js',)
 
 
+class InnerInline2Tabular(admin.TabularInline):
+    model = Inner2
+
+
+class CustomNumberWidget(forms.NumberInput):
+    class Media:
+        js = ('custom_number.js',)
+
+
 class InnerInline3(admin.StackedInline):
     model = Inner3
+    formfield_overrides = {
+        models.IntegerField: {'widget': CustomNumberWidget},
+    }
 
     class Media:
         js = ('my_awesome_inline_scripts.js',)
@@ -144,6 +158,17 @@ class NovelAdmin(admin.ModelAdmin):
     inlines = [ChapterInline]
 
 
+class ReadOnlyChapterInline(admin.TabularInline):
+    model = Chapter
+
+    def has_change_permission(self, request, obj=None):
+        return False
+
+
+class NovelReadonlyChapterAdmin(admin.ModelAdmin):
+    inlines = [ReadOnlyChapterInline]
+
+
 class ConsigliereInline(admin.TabularInline):
     model = Consigliere
 
@@ -197,15 +222,30 @@ class SomeChildModelForm(forms.ModelForm):
         widgets = {
             'position': forms.HiddenInput,
         }
+        labels = {'readonly_field': 'Label from ModelForm.Meta'}
+        help_texts = {'readonly_field': 'Help text from ModelForm.Meta'}
 
     def __init__(self, *args, **kwargs):
-        super(SomeChildModelForm, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.fields['name'].label = 'new label'
 
 
 class SomeChildModelInline(admin.TabularInline):
     model = SomeChildModel
     form = SomeChildModelForm
+    readonly_fields = ('readonly_field',)
+
+
+class StudentInline(admin.StackedInline):
+    model = Child
+    extra = 1
+    fieldsets = [
+        ('Name', {'fields': ('name',), 'classes': ('collapse',)}),
+    ]
+
+
+class TeacherAdmin(admin.ModelAdmin):
+    inlines = [StudentInline]
 
 
 site.register(TitleCollection, inlines=[TitleInline])
@@ -213,12 +253,13 @@ site.register(TitleCollection, inlines=[TitleInline])
 # only ModelAdmin media
 site.register(Holder, HolderAdmin, inlines=[InnerInline])
 # ModelAdmin and Inline media
-site.register(Holder2, HolderAdmin, inlines=[InnerInline2])
+site.register(Holder2, HolderAdmin, inlines=[InnerInline2, InnerInline2Tabular])
 # only Inline media
 site.register(Holder3, inlines=[InnerInline3])
 
 site.register(Poll, PollAdmin)
 site.register(Novel, NovelAdmin)
+site.register(NovelReadonlyChapter, NovelReadonlyChapterAdmin)
 site.register(Fashionista, inlines=[InlineWeakness])
 site.register(Holder4, Holder4Admin)
 site.register(Author, AuthorAdmin)
@@ -229,3 +270,4 @@ site.register(BinaryTree, inlines=[BinaryTreeAdmin])
 site.register(ExtraTerrestrial, inlines=[SightingInline])
 site.register(SomeParentModel, inlines=[SomeChildModelInline])
 site.register([Question, Inner4Stacked, Inner4Tabular])
+site.register(Teacher, TeacherAdmin)
