@@ -1,17 +1,13 @@
-from __future__ import unicode_literals
-
 import uuid
 
 from django.contrib.auth.models import User
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
 class MyFileField(models.FileField):
     pass
 
 
-@python_2_unicode_compatible
 class Member(models.Model):
     name = models.CharField(max_length=100)
     birthdate = models.DateTimeField(blank=True, null=True)
@@ -22,7 +18,6 @@ class Member(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Band(models.Model):
     name = models.CharField(max_length=100)
     style = models.CharField(max_length=20)
@@ -32,9 +27,9 @@ class Band(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Album(models.Model):
     band = models.ForeignKey(Band, models.CASCADE)
+    featuring = models.ManyToManyField(Band, related_name='featured')
     name = models.CharField(max_length=100)
     cover_art = models.FileField(upload_to='albums')
     backside_art = MyFileField(upload_to='albums_back', null=True)
@@ -45,10 +40,9 @@ class Album(models.Model):
 
 class HiddenInventoryManager(models.Manager):
     def get_queryset(self):
-        return super(HiddenInventoryManager, self).get_queryset().filter(hidden=False)
+        return super().get_queryset().filter(hidden=False)
 
 
-@python_2_unicode_compatible
 class Inventory(models.Model):
     barcode = models.PositiveIntegerField(unique=True)
     parent = models.ForeignKey('self', models.SET_NULL, to_field='barcode', blank=True, null=True)
@@ -83,7 +77,6 @@ class Event(models.Model):
     min_age = models.IntegerField(blank=True, null=True)
 
 
-@python_2_unicode_compatible
 class Car(models.Model):
     owner = models.ForeignKey(User, models.CASCADE)
     make = models.CharField(max_length=30)
@@ -139,18 +132,16 @@ class Advisor(models.Model):
     companies = models.ManyToManyField(Company)
 
 
-@python_2_unicode_compatible
 class Student(models.Model):
     name = models.CharField(max_length=255)
 
-    def __str__(self):
-        return self.name
-
     class Meta:
         ordering = ('name',)
 
+    def __str__(self):
+        return self.name
+
 
-@python_2_unicode_compatible
 class School(models.Model):
     name = models.CharField(max_length=255)
     students = models.ManyToManyField(Student, related_name='current_schools')
@@ -160,7 +151,6 @@ class School(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Profile(models.Model):
     user = models.ForeignKey('auth.User', models.CASCADE, to_field='username')
 
