@@ -1,9 +1,6 @@
-# -*- coding: utf-8 -*-
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Author(models.Model):
     name = models.CharField(max_length=100)
     age = models.IntegerField()
@@ -13,16 +10,15 @@ class Author(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Publisher(models.Model):
     name = models.CharField(max_length=255)
     num_awards = models.IntegerField()
+    duration = models.DurationField(blank=True, null=True)
 
     def __str__(self):
         return self.name
 
 
-@python_2_unicode_compatible
 class Book(models.Model):
     isbn = models.CharField(max_length=9)
     name = models.CharField(max_length=255)
@@ -30,15 +26,14 @@ class Book(models.Model):
     rating = models.FloatField()
     price = models.DecimalField(decimal_places=2, max_digits=6)
     authors = models.ManyToManyField(Author)
-    contact = models.ForeignKey(Author, related_name='book_contact_set')
-    publisher = models.ForeignKey(Publisher)
+    contact = models.ForeignKey(Author, models.CASCADE, related_name='book_contact_set')
+    publisher = models.ForeignKey(Publisher, models.CASCADE)
     pubdate = models.DateField()
 
     def __str__(self):
         return self.name
 
 
-@python_2_unicode_compatible
 class Store(models.Model):
     name = models.CharField(max_length=255)
     books = models.ManyToManyField(Book)
