@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.apps import AppConfig
 
 
@@ -18,7 +16,7 @@ class BadConfig(AppConfig):
     """This class doesn't supply the mandatory 'name' attribute."""
 
 
-class NotAConfig(object):
+class NotAConfig:
     name = 'apps'
 
 
