@@ -1,11 +1,15 @@
-from __future__ import unicode_literals
-
+import builtins
+import getpass
 import sys
 from datetime import date
+from io import StringIO
+from unittest import mock
 
 from django.apps import apps
-from django.contrib.auth import management
-from django.contrib.auth.management import create_permissions
+from django.contrib.auth import get_permission_codename, management
+from django.contrib.auth.management import (
+    create_permissions, get_default_username,
+)
 from django.contrib.auth.management.commands import (
     changepassword, createsuperuser,
 )
@@ -14,15 +18,23 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.management import call_command
 from django.core.management.base import CommandError
 from django.db import migrations
-from django.test import TestCase, mock, override_settings
-from django.utils import six
-from django.utils.encoding import force_str
-from django.utils.translation import ugettext_lazy as _
+from django.test import TestCase, override_settings
+from django.utils.translation import gettext_lazy as _
 
 from .models import (
     CustomUser, CustomUserNonUniqueUsername, CustomUserWithFK, Email,
+    UserProxy,
 )
 
+MOCK_INPUT_KEY_TO_PROMPTS = {
+    # @mock_inputs dict key: [expected prompt messages],
+    'bypass': ['Bypass password validation and create user anyway? [y/N]: '],
+    'email': ['Email address: '],
+    'date_of_birth': ['Date of birth: '],
+    'first_name': ['First name: '],
+    'username': ['Username: ', lambda: "Username (leave blank to use '%s'): " % get_default_username()],
+}
+
 
 def mock_inputs(inputs):
     """
@@ -34,39 +46,45 @@ def mock_inputs(inputs):
             class mock_getpass:
                 @staticmethod
                 def getpass(prompt=b'Password: ', stream=None):
-                    if six.PY2:
-                        # getpass on Windows only supports prompt as bytestring (#19807)
-                        assert isinstance(prompt, six.binary_type)
                     if callable(inputs['password']):
                         return inputs['password']()
                     return inputs['password']
 
             def mock_input(prompt):
-                # prompt should be encoded in Python 2. This line will raise an
-                # Exception if prompt contains unencoded non-ASCII on Python 2.
-                prompt = str(prompt)
-                assert str('__proxy__') not in prompt
-                response = ''
+                assert '__proxy__' not in prompt
+                response = None
                 for key, val in inputs.items():
-                    if force_str(key) in prompt.lower():
-                        response = val
+                    if val == 'KeyboardInterrupt':
+                        raise KeyboardInterrupt
+                    # get() fallback because sometimes 'key' is the actual
+                    # prompt rather than a shortcut name.
+                    prompt_msgs = MOCK_INPUT_KEY_TO_PROMPTS.get(key, key)
+                    if isinstance(prompt_msgs, list):
+                        prompt_msgs = [msg() if callable(msg) else msg for msg in prompt_msgs]
+                    if prompt in prompt_msgs:
+                        if callable(val):
+                            response = val()
+                        else:
+                            response = val
                         break
+                if response is None:
+                    raise ValueError('Mock input for %r not found.' % prompt)
                 return response
 
             old_getpass = createsuperuser.getpass
-            old_input = createsuperuser.input
+            old_input = builtins.input
             createsuperuser.getpass = mock_getpass
-            createsuperuser.input = mock_input
+            builtins.input = mock_input
             try:
                 test_func(*args)
             finally:
                 createsuperuser.getpass = old_getpass
-                createsuperuser.input = old_input
+                builtins.input = old_input
         return wrapped
     return inner
 
 
-class MockTTY(object):
+class MockTTY:
     """
     A fake stdin object that pretends to be a TTY to be used in conjunction
     with mock_inputs.
@@ -75,6 +93,13 @@ class MockTTY(object):
         return True
 
 
+class MockInputTests(TestCase):
+    @mock_inputs({'username': 'alice'})
+    def test_input_not_found(self):
+        with self.assertRaisesMessage(ValueError, "Mock input for 'Email address: ' not found."):
+            call_command('createsuperuser', stdin=MockTTY())
+
+
 class GetDefaultUsernameTestCase(TestCase):
 
     def setUp(self):
@@ -84,7 +109,7 @@ class GetDefaultUsernameTestCase(TestCase):
         management.get_system_username = self.old_get_system_username
 
     def test_actual_implementation(self):
-        self.assertIsInstance(management.get_system_username(), six.text_type)
+        self.assertIsInstance(management.get_system_username(), str)
 
     def test_simple(self):
         management.get_system_username = lambda: 'joe'
@@ -108,15 +133,40 @@ class GetDefaultUsernameTestCase(TestCase):
 ])
 class ChangepasswordManagementCommandTestCase(TestCase):
 
+    @classmethod
+    def setUpTestData(cls):
+        cls.user = User.objects.create_user(username='joe', password='qwerty')
+
     def setUp(self):
-        self.user = User.objects.create_user(username='joe', password='qwerty')
-        self.stdout = six.StringIO()
-        self.stderr = six.StringIO()
+        self.stdout = StringIO()
+        self.stderr = StringIO()
 
     def tearDown(self):
         self.stdout.close()
         self.stderr.close()
 
+    @mock.patch.object(getpass, 'getpass', return_value='password')
+    def test_get_pass(self, mock_get_pass):
+        call_command('changepassword', username='joe', stdout=self.stdout)
+        self.assertIs(User.objects.get(username='joe').check_password('password'), True)
+
+    @mock.patch.object(getpass, 'getpass', return_value='')
+    def test_get_pass_no_input(self, mock_get_pass):
+        with self.assertRaisesMessage(CommandError, 'aborted'):
+            call_command('changepassword', username='joe', stdout=self.stdout)
+
+    @mock.patch.object(changepassword.Command, '_get_pass', return_value='new_password')
+    def test_system_username(self, mock_get_pass):
+        """The system username is used if --username isn't provided."""
+        username = getpass.getuser()
+        User.objects.create_user(username=username, password='qwerty')
+        call_command('changepassword', stdout=self.stdout)
+        self.assertIs(User.objects.get(username=username).check_password('new_password'), True)
+
+    def test_nonexistent_username(self):
+        with self.assertRaisesMessage(CommandError, "user 'test' does not exist"):
+            call_command('changepassword', username='test', stdout=self.stdout)
+
     @mock.patch.object(changepassword.Command, '_get_pass', return_value='not qwerty')
     def test_that_changepassword_command_changes_joes_password(self, mock_get_pass):
         "Executing the changepassword management command should change joe's password"
@@ -137,7 +187,8 @@ class ChangepasswordManagementCommandTestCase(TestCase):
         A CommandError should be thrown by handle() if the user enters in
         mismatched passwords three times.
         """
-        with self.assertRaises(CommandError):
+        msg = "Aborting password change for user 'joe' after 3 attempts"
+        with self.assertRaisesMessage(CommandError, msg):
             call_command('changepassword', username='joe', stdout=self.stdout, stderr=self.stderr)
 
     @mock.patch.object(changepassword.Command, '_get_pass', return_value='1234567890')
@@ -163,7 +214,7 @@ class ChangepasswordManagementCommandTestCase(TestCase):
 
 
 class MultiDBChangepasswordManagementCommandTestCase(TestCase):
-    multi_db = True
+    databases = {'default', 'other'}
 
     @mock.patch.object(changepassword.Command, '_get_pass', return_value='not qwerty')
     def test_that_changepassword_command_with_database_option_uses_given_db(self, mock_get_pass):
@@ -173,7 +224,7 @@ class MultiDBChangepasswordManagementCommandTestCase(TestCase):
         user = User.objects.db_manager('other').create_user(username='joe', password='qwerty')
         self.assertTrue(user.check_password('qwerty'))
 
-        out = six.StringIO()
+        out = StringIO()
         call_command('changepassword', username='joe', database='other', stdout=out)
         command_output = out.getvalue().strip()
 
@@ -190,10 +241,15 @@ class MultiDBChangepasswordManagementCommandTestCase(TestCase):
 )
 class CreatesuperuserManagementCommandTestCase(TestCase):
 
+    def test_no_email_argument(self):
+        new_io = StringIO()
+        with self.assertRaisesMessage(CommandError, 'You must use --email with --noinput.'):
+            call_command('createsuperuser', interactive=False, username='joe', stdout=new_io)
+
     def test_basic_usage(self):
         "Check the operation of the createsuperuser management command"
         # We can use the management command to create a superuser
-        new_io = six.StringIO()
+        new_io = StringIO()
         call_command(
             "createsuperuser",
             interactive=False,
@@ -209,31 +265,35 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         # created password should be unusable
         self.assertFalse(u.has_usable_password())
 
-    @mock_inputs({
-        'password': "nopasswd",
-        'u\u017eivatel': 'foo',  # username (cz)
-        'email': 'nolocale@somewhere.org'})
     def test_non_ascii_verbose_name(self):
-        username_field = User._meta.get_field('username')
-        old_verbose_name = username_field.verbose_name
-        username_field.verbose_name = _('u\u017eivatel')
-        new_io = six.StringIO()
-        try:
-            call_command(
-                "createsuperuser",
-                interactive=True,
-                stdout=new_io,
-                stdin=MockTTY(),
-            )
-        finally:
-            username_field.verbose_name = old_verbose_name
+        @mock_inputs({
+            'password': "nopasswd",
+            "Uživatel (leave blank to use '%s'): " % get_default_username(): 'foo',  # username (cz)
+            'email': 'nolocale@somewhere.org',
+        })
+        def test(self):
+            username_field = User._meta.get_field('username')
+            old_verbose_name = username_field.verbose_name
+            username_field.verbose_name = _('u\u017eivatel')
+            new_io = StringIO()
+            try:
+                call_command(
+                    "createsuperuser",
+                    interactive=True,
+                    stdout=new_io,
+                    stdin=MockTTY(),
+                )
+            finally:
+                username_field.verbose_name = old_verbose_name
 
-        command_output = new_io.getvalue().strip()
-        self.assertEqual(command_output, 'Superuser created successfully.')
+            command_output = new_io.getvalue().strip()
+            self.assertEqual(command_output, 'Superuser created successfully.')
+
+        test(self)
 
     def test_verbosity_zero(self):
         # We can suppress output on the management command
-        new_io = six.StringIO()
+        new_io = StringIO()
         call_command(
             "createsuperuser",
             interactive=False,
@@ -249,7 +309,7 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         self.assertFalse(u.has_usable_password())
 
     def test_email_in_username(self):
-        new_io = six.StringIO()
+        new_io = StringIO()
         call_command(
             "createsuperuser",
             interactive=False,
@@ -267,12 +327,13 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         # We can use the management command to create a superuser
         # We skip validation because the temporary substitution of the
         # swappable User model messes with validation.
-        new_io = six.StringIO()
+        new_io = StringIO()
         call_command(
             "createsuperuser",
             interactive=False,
             email="joe@somewhere.org",
             date_of_birth="1976-04-01",
+            first_name='Joe',
             stdout=new_io,
         )
         command_output = new_io.getvalue().strip()
@@ -289,12 +350,11 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         # We can use the management command to create a superuser
         # We skip validation because the temporary substitution of the
         # swappable User model messes with validation.
-        new_io = six.StringIO()
-        with self.assertRaises(CommandError):
+        new_io = StringIO()
+        with self.assertRaisesMessage(CommandError, 'You must use --email with --noinput.'):
             call_command(
                 "createsuperuser",
                 interactive=False,
-                username="joe@somewhere.org",
                 stdout=new_io,
                 stderr=new_io,
             )
@@ -311,7 +371,7 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
             'password': 'nopasswd',
         })
         def createsuperuser():
-            new_io = six.StringIO()
+            new_io = StringIO()
             call_command(
                 "createsuperuser",
                 interactive=True,
@@ -333,12 +393,12 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         If the command is not called from a TTY, it should be skipped and a
         message should be displayed (#7423).
         """
-        class FakeStdin(object):
+        class FakeStdin:
             """A fake stdin object that has isatty() return False."""
             def isatty(self):
                 return False
 
-        out = six.StringIO()
+        out = StringIO()
         call_command(
             "createsuperuser",
             stdin=FakeStdin(),
@@ -360,8 +420,8 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         call_command(
             command,
             stdin=sentinel,
-            stdout=six.StringIO(),
-            stderr=six.StringIO(),
+            stdout=StringIO(),
+            stderr=StringIO(),
             interactive=False,
             verbosity=0,
             username='janet',
@@ -372,8 +432,8 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         command = createsuperuser.Command()
         call_command(
             command,
-            stdout=six.StringIO(),
-            stderr=six.StringIO(),
+            stdout=StringIO(),
+            stderr=StringIO(),
             interactive=False,
             verbosity=0,
             username='joe',
@@ -383,7 +443,7 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
 
     @override_settings(AUTH_USER_MODEL='auth_tests.CustomUserWithFK')
     def test_fields_with_fk(self):
-        new_io = six.StringIO()
+        new_io = StringIO()
         group = Group.objects.create(name='mygroup')
         email = Email.objects.create(email='mymail@gmail.com')
         call_command(
@@ -413,15 +473,15 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
 
     @override_settings(AUTH_USER_MODEL='auth_tests.CustomUserWithFK')
     def test_fields_with_fk_interactive(self):
-        new_io = six.StringIO()
+        new_io = StringIO()
         group = Group.objects.create(name='mygroup')
         email = Email.objects.create(email='mymail@gmail.com')
 
         @mock_inputs({
             'password': 'nopasswd',
-            'username (email.id)': email.pk,
-            'email (email.email)': email.email,
-            'group (group.id)': group.pk,
+            'Username (Email.id): ': email.pk,
+            'Email (Email.email): ': email.email,
+            'Group (Group.id): ': group.pk,
         })
         def test(self):
             call_command(
@@ -439,23 +499,45 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
 
         test(self)
 
+    def test_default_username(self):
+        """createsuperuser uses a default username when one isn't provided."""
+        # Get the default username before creating a user.
+        default_username = get_default_username()
+        new_io = StringIO()
+        entered_passwords = ['password', 'password']
+
+        def return_passwords():
+            return entered_passwords.pop(0)
+
+        @mock_inputs({'password': return_passwords, 'username': '', 'email': ''})
+        def test(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+            )
+            self.assertEqual(new_io.getvalue().strip(), 'Superuser created successfully.')
+            self.assertTrue(User.objects.filter(username=default_username).exists())
+
+        test(self)
+
     def test_password_validation(self):
         """
         Creation should fail if the password fails validation.
         """
-        new_io = six.StringIO()
+        new_io = StringIO()
+        entered_passwords = ['1234567890', '1234567890', 'password', 'password']
 
-        # Returns '1234567890' the first two times it is called, then
-        # 'password' subsequently.
-        def bad_then_good_password(index=[0]):
-            index[0] += 1
-            if index[0] <= 2:
-                return '1234567890'
-            return 'password'
+        def bad_then_good_password():
+            return entered_passwords.pop(0)
 
         @mock_inputs({
             'password': bad_then_good_password,
             'username': 'joe1234567890',
+            'email': '',
+            'bypass': 'n',
         })
         def test(self):
             call_command(
@@ -473,11 +555,259 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
 
         test(self)
 
+    @override_settings(AUTH_PASSWORD_VALIDATORS=[
+        {'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator'},
+    ])
+    def test_validate_password_against_username(self):
+        new_io = StringIO()
+        username = 'supremelycomplex'
+        entered_passwords = [username, username, 'superduperunguessablepassword', 'superduperunguessablepassword']
+
+        def bad_then_good_password():
+            return entered_passwords.pop(0)
+
+        @mock_inputs({
+            'password': bad_then_good_password,
+            'username': username,
+            'email': '',
+            'bypass': 'n',
+        })
+        def test(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+            )
+            self.assertEqual(
+                new_io.getvalue().strip(),
+                'The password is too similar to the username.\n'
+                'Superuser created successfully.'
+            )
+
+        test(self)
+
+    @override_settings(
+        AUTH_USER_MODEL='auth_tests.CustomUser',
+        AUTH_PASSWORD_VALIDATORS=[
+            {'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator'},
+        ]
+    )
+    def test_validate_password_against_required_fields(self):
+        new_io = StringIO()
+        first_name = 'josephine'
+        entered_passwords = [first_name, first_name, 'superduperunguessablepassword', 'superduperunguessablepassword']
+
+        def bad_then_good_password():
+            return entered_passwords.pop(0)
+
+        @mock_inputs({
+            'password': bad_then_good_password,
+            'username': 'whatever',
+            'first_name': first_name,
+            'date_of_birth': '1970-01-01',
+            'email': 'joey@example.com',
+            'bypass': 'n',
+        })
+        def test(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+            )
+            self.assertEqual(
+                new_io.getvalue().strip(),
+                "The password is too similar to the first name.\n"
+                "Superuser created successfully."
+            )
+
+        test(self)
+
+    def test_blank_username(self):
+        """Creation fails if --username is blank."""
+        new_io = StringIO()
+
+        def test(self):
+            with self.assertRaisesMessage(CommandError, 'Username cannot be blank.'):
+                call_command(
+                    'createsuperuser',
+                    username='',
+                    stdin=MockTTY(),
+                    stdout=new_io,
+                    stderr=new_io,
+                )
+
+        test(self)
+
+    def test_blank_username_non_interactive(self):
+        new_io = StringIO()
+
+        def test(self):
+            with self.assertRaisesMessage(CommandError, 'Username cannot be blank.'):
+                call_command(
+                    'createsuperuser',
+                    username='',
+                    interactive=False,
+                    stdin=MockTTY(),
+                    stdout=new_io,
+                    stderr=new_io,
+                )
+
+        test(self)
+
+    def test_password_validation_bypass(self):
+        """
+        Password validation can be bypassed by entering 'y' at the prompt.
+        """
+        new_io = StringIO()
+
+        @mock_inputs({
+            'password': '1234567890',
+            'username': 'joe1234567890',
+            'email': '',
+            'bypass': 'y',
+        })
+        def test(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+            )
+            self.assertEqual(
+                new_io.getvalue().strip(),
+                'This password is entirely numeric.\n'
+                'Superuser created successfully.'
+            )
+
+        test(self)
+
+    def test_invalid_username(self):
+        """Creation fails if the username fails validation."""
+        user_field = User._meta.get_field(User.USERNAME_FIELD)
+        new_io = StringIO()
+        entered_passwords = ['password', 'password']
+        # Enter an invalid (too long) username first and then a valid one.
+        invalid_username = ('x' * user_field.max_length) + 'y'
+        entered_usernames = [invalid_username, 'janet']
+
+        def return_passwords():
+            return entered_passwords.pop(0)
+
+        def return_usernames():
+            return entered_usernames.pop(0)
+
+        @mock_inputs({'password': return_passwords, 'username': return_usernames, 'email': ''})
+        def test(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+            )
+            self.assertEqual(
+                new_io.getvalue().strip(),
+                'Error: Ensure this value has at most %s characters (it has %s).\n'
+                'Superuser created successfully.' % (user_field.max_length, len(invalid_username))
+            )
+
+        test(self)
+
+    @mock_inputs({'username': 'KeyboardInterrupt'})
+    def test_keyboard_interrupt(self):
+        new_io = StringIO()
+        with self.assertRaises(SystemExit):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+            )
+        self.assertEqual(new_io.getvalue(), '\nOperation cancelled.\n')
+
+    def test_existing_username(self):
+        """Creation fails if the username already exists."""
+        user = User.objects.create(username='janet')
+        new_io = StringIO()
+        entered_passwords = ['password', 'password']
+        # Enter the existing username first and then a new one.
+        entered_usernames = [user.username, 'joe']
+
+        def return_passwords():
+            return entered_passwords.pop(0)
+
+        def return_usernames():
+            return entered_usernames.pop(0)
+
+        @mock_inputs({'password': return_passwords, 'username': return_usernames, 'email': ''})
+        def test(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+            )
+            self.assertEqual(
+                new_io.getvalue().strip(),
+                'Error: That username is already taken.\n'
+                'Superuser created successfully.'
+            )
+
+        test(self)
+
+    def test_existing_username_non_interactive(self):
+        """Creation fails if the username already exists."""
+        User.objects.create(username='janet')
+        new_io = StringIO()
+        with self.assertRaisesMessage(CommandError, "Error: That username is already taken."):
+            call_command(
+                'createsuperuser',
+                username='janet',
+                email='',
+                interactive=False,
+                stdout=new_io,
+            )
+
+    def test_existing_username_provided_via_option_and_interactive(self):
+        """call_command() gets username='janet' and interactive=True."""
+        new_io = StringIO()
+        entered_passwords = ['password', 'password']
+        User.objects.create(username='janet')
+
+        def return_passwords():
+            return entered_passwords.pop(0)
+
+        @mock_inputs({
+            'password': return_passwords,
+            'username': 'janet1',
+            'email': 'test@test.com'
+        })
+        def test(self):
+            call_command(
+                'createsuperuser',
+                username='janet',
+                interactive=True,
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+            )
+            msg = 'Error: That username is already taken.\nSuperuser created successfully.'
+            self.assertEqual(new_io.getvalue().strip(), msg)
+
+        test(self)
+
     def test_validation_mismatched_passwords(self):
         """
         Creation should fail if the user enters mismatched passwords.
         """
-        new_io = six.StringIO()
+        new_io = StringIO()
 
         # The first two passwords do not match, but the second two do match and
         # are valid.
@@ -489,6 +819,7 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         @mock_inputs({
             'password': mismatched_passwords_then_matched,
             'username': 'joe1234567890',
+            'email': '',
         })
         def test(self):
             call_command(
@@ -510,7 +841,7 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         """
         Creation should fail if the user enters blank passwords.
         """
-        new_io = six.StringIO()
+        new_io = StringIO()
 
         # The first two passwords are empty strings, but the second two are
         # valid.
@@ -522,6 +853,7 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         @mock_inputs({
             'password': blank_passwords_then_valid,
             'username': 'joe1234567890',
+            'email': '',
         })
         def test(self):
             call_command(
@@ -539,15 +871,49 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
 
         test(self)
 
+    @override_settings(AUTH_USER_MODEL='auth_tests.NoPasswordUser')
+    def test_usermodel_without_password(self):
+        new_io = StringIO()
+
+        def test(self):
+            call_command(
+                'createsuperuser',
+                interactive=False,
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+                username='username',
+            )
+            self.assertEqual(new_io.getvalue().strip(), 'Superuser created successfully.')
+
+        test(self)
+
+    @override_settings(AUTH_USER_MODEL='auth_tests.NoPasswordUser')
+    def test_usermodel_without_password_interactive(self):
+        new_io = StringIO()
+
+        @mock_inputs({'username': 'username'})
+        def test(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+            )
+            self.assertEqual(new_io.getvalue().strip(), 'Superuser created successfully.')
+
+        test(self)
+
 
 class MultiDBCreatesuperuserTestCase(TestCase):
-    multi_db = True
+    databases = {'default', 'other'}
 
     def test_createsuperuser_command_with_database_option(self):
         """
         changepassword --database should operate on the specified DB.
         """
-        new_io = six.StringIO()
+        new_io = StringIO()
         call_command(
             'createsuperuser',
             interactive=False,
@@ -581,10 +947,10 @@ class CreatePermissionsTests(TestCase):
         ]
         create_permissions(self.app_config, verbosity=0)
 
-        # add/change/delete permission by default + custom permission
+        # view/add/change/delete permission by default + custom permission
         self.assertEqual(Permission.objects.filter(
             content_type=permission_content_type,
-        ).count(), 4)
+        ).count(), 5)
 
         Permission.objects.filter(content_type=permission_content_type).delete()
         Permission._meta.default_permissions = []
@@ -608,3 +974,30 @@ class CreatePermissionsTests(TestCase):
         state = migrations.state.ProjectState(real_apps=['contenttypes'])
         with self.assertNumQueries(0):
             create_permissions(self.app_config, verbosity=0, apps=state.apps)
+
+    def test_create_permissions_checks_contenttypes_created(self):
+        """
+        `post_migrate` handler ordering isn't guaranteed. Simulate a case
+        where create_permissions() is called before create_contenttypes().
+        """
+        # Warm the manager cache.
+        ContentType.objects.get_for_model(Group)
+        # Apply a deletion as if e.g. a database 'flush' had been executed.
+        ContentType.objects.filter(app_label='auth', model='group').delete()
+        # This fails with a foreign key constraint without the fix.
+        create_permissions(apps.get_app_config('auth'), interactive=False, verbosity=0)
+
+    def test_permission_with_proxy_content_type_created(self):
+        """
+        A proxy model's permissions use its own content type rather than the
+        content type of the concrete model.
+        """
+        opts = UserProxy._meta
+        codename = get_permission_codename('add', opts)
+        self.assertTrue(
+            Permission.objects.filter(
+                content_type__model=opts.model_name,
+                content_type__app_label=opts.app_label,
+                codename=codename,
+            ).exists()
+        )
