@@ -1,16 +1,19 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
+from unittest import mock
 
 from django.conf.global_settings import PASSWORD_HASHERS
 from django.contrib.auth import get_user_model
+from django.contrib.auth.base_user import AbstractBaseUser
 from django.contrib.auth.hashers import get_hasher
 from django.contrib.auth.models import (
-    AbstractUser, Group, Permission, User, UserManager,
+    AbstractUser, AnonymousUser, Group, Permission, User, UserManager,
 )
 from django.contrib.contenttypes.models import ContentType
 from django.core import mail
 from django.db.models.signals import post_save
-from django.test import TestCase, mock, override_settings
+from django.test import SimpleTestCase, TestCase, override_settings
+
+from .models import IntegerUsernameUser
+from .models.with_custom_email_field import CustomEmailField
 
 
 class NaturalKeysTestCase(TestCase):
@@ -44,7 +47,7 @@ class LoadDataWithNaturalKeysTestCase(TestCase):
 
 
 class LoadDataWithNaturalKeysAndMultipleDatabasesTestCase(TestCase):
-    multi_db = True
+    databases = {'default', 'other'}
 
     def test_load_data_with_user_permissions(self):
         # Create test contenttypes for both databases
@@ -107,7 +110,7 @@ class UserManagerTestCase(TestCase):
         self.assertFalse(user.has_usable_password())
 
     def test_create_user_email_domain_normalize_rfc3696(self):
-        # According to  http://tools.ietf.org/html/rfc3696#section-3
+        # According to https://tools.ietf.org/html/rfc3696#section-3
         # the "@" symbol can be part of the local part of an email address
         returned = UserManager.normalize_email(r'Abc\@DEF@EXAMPLE.com')
         self.assertEqual(returned, r'Abc\@DEF@example.com')
@@ -117,8 +120,8 @@ class UserManagerTestCase(TestCase):
         self.assertEqual(returned, 'normal@domain.com')
 
     def test_create_user_email_domain_normalize_with_whitespace(self):
-        returned = UserManager.normalize_email('email\ with_whitespace@D.COM')
-        self.assertEqual(returned, 'email\ with_whitespace@d.com')
+        returned = UserManager.normalize_email(r'email\ with_whitespace@D.COM')
+        self.assertEqual(returned, r'email\ with_whitespace@d.com')
 
     def test_empty_username(self):
         with self.assertRaisesMessage(ValueError, 'The given username must be set'):
@@ -145,14 +148,31 @@ class UserManagerTestCase(TestCase):
                 password='test', is_staff=False,
             )
 
+    def test_make_random_password(self):
+        allowed_chars = 'abcdefg'
+        password = UserManager().make_random_password(5, allowed_chars)
+        self.assertEqual(len(password), 5)
+        for char in password:
+            self.assertIn(char, allowed_chars)
 
-class AbstractBaseUserTests(TestCase):
+
+class AbstractBaseUserTests(SimpleTestCase):
+
+    def test_has_usable_password(self):
+        """
+        Passwords are usable even if they don't correspond to a hasher in
+        settings.PASSWORD_HASHERS.
+        """
+        self.assertIs(User(password='some-gibbberish').has_usable_password(), True)
+
+    def test_normalize_username(self):
+        self.assertEqual(IntegerUsernameUser().normalize_username(123), 123)
 
     def test_clean_normalize_username(self):
         # The normalization happens in AbstractBaseUser.clean()
         ohm_username = 'iamtheΩ'  # U+2126 OHM SIGN
         for model in ('auth.User', 'auth_tests.CustomUser'):
-            with self.settings(AUTH_USER_MODEL=model):
+            with self.subTest(model=model), self.settings(AUTH_USER_MODEL=model):
                 User = get_user_model()
                 user = User(**{User.USERNAME_FIELD: ohm_username, 'password': 'foo'})
                 user.clean()
@@ -160,6 +180,14 @@ class AbstractBaseUserTests(TestCase):
                 self.assertNotEqual(username, ohm_username)
                 self.assertEqual(username, 'iamtheΩ')  # U+03A9 GREEK CAPITAL LETTER OMEGA
 
+    def test_default_email(self):
+        user = AbstractBaseUser()
+        self.assertEqual(user.get_email_field_name(), 'email')
+
+    def test_custom_email(self):
+        user = CustomEmailField()
+        self.assertEqual(user.get_email_field_name(), 'email_address')
+
 
 class AbstractUserTestCase(TestCase):
     def test_email_user(self):
@@ -178,9 +206,7 @@ class AbstractUserTestCase(TestCase):
             from_email="from@domain.com",
             **kwargs
         )
-        # Test that one message has been sent.
         self.assertEqual(len(mail.outbox), 1)
-        # Verify that test email contains the correct attributes:
         message = mail.outbox[0]
         self.assertEqual(message.subject, "Subject here")
         self.assertEqual(message.body, "This is a message")
@@ -194,6 +220,11 @@ class AbstractUserTestCase(TestCase):
         user2 = User.objects.create_user(username='user2')
         self.assertIsNone(user2.last_login)
 
+    def test_user_clean_normalize_email(self):
+        user = User(username='user', password='foo', email='foo@BAR.com')
+        user.clean()
+        self.assertEqual(user.email, 'foo@bar.com')
+
     def test_user_double_save(self):
         """
         Calling user.save() twice should trigger password_changed() once.
@@ -283,3 +314,69 @@ class TestCreateSuperUserSignals(TestCase):
     def test_create_superuser(self):
         User.objects.create_superuser("JohnDoe", "mail@example.com", "1")
         self.assertEqual(self.signals_count, 1)
+
+
+class AnonymousUserTests(SimpleTestCase):
+    no_repr_msg = "Django doesn't provide a DB representation for AnonymousUser."
+
+    def setUp(self):
+        self.user = AnonymousUser()
+
+    def test_properties(self):
+        self.assertIsNone(self.user.pk)
+        self.assertEqual(self.user.username, '')
+        self.assertEqual(self.user.get_username(), '')
+        self.assertIs(self.user.is_anonymous, True)
+        self.assertIs(self.user.is_authenticated, False)
+        self.assertIs(self.user.is_staff, False)
+        self.assertIs(self.user.is_active, False)
+        self.assertIs(self.user.is_superuser, False)
+        self.assertEqual(self.user.groups.all().count(), 0)
+        self.assertEqual(self.user.user_permissions.all().count(), 0)
+        self.assertEqual(self.user.get_group_permissions(), set())
+
+    def test_str(self):
+        self.assertEqual(str(self.user), 'AnonymousUser')
+
+    def test_eq(self):
+        self.assertEqual(self.user, AnonymousUser())
+        self.assertNotEqual(self.user, User('super', 'super@example.com', 'super'))
+
+    def test_hash(self):
+        self.assertEqual(hash(self.user), 1)
+
+    def test_int(self):
+        msg = (
+            'Cannot cast AnonymousUser to int. Are you trying to use it in '
+            'place of User?'
+        )
+        with self.assertRaisesMessage(TypeError, msg):
+            int(self.user)
+
+    def test_delete(self):
+        with self.assertRaisesMessage(NotImplementedError, self.no_repr_msg):
+            self.user.delete()
+
+    def test_save(self):
+        with self.assertRaisesMessage(NotImplementedError, self.no_repr_msg):
+            self.user.save()
+
+    def test_set_password(self):
+        with self.assertRaisesMessage(NotImplementedError, self.no_repr_msg):
+            self.user.set_password('password')
+
+    def test_check_password(self):
+        with self.assertRaisesMessage(NotImplementedError, self.no_repr_msg):
+            self.user.check_password('password')
+
+
+class GroupTests(SimpleTestCase):
+    def test_str(self):
+        g = Group(name='Users')
+        self.assertEqual(str(g), 'Users')
+
+
+class PermissionTests(TestCase):
+    def test_str(self):
+        p = Permission.objects.get(codename='view_customemailfield')
+        self.assertEqual(str(p), 'auth_tests | custom email field | Can view custom email field')
