@@ -1,11 +1,7 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
-import logging
 import re
-import warnings
 
 from django.conf import settings
+from django.contrib.sessions.backends.cache import SessionStore
 from django.core.exceptions import ImproperlyConfigured
 from django.http import HttpRequest
 from django.middleware.csrf import (
@@ -14,9 +10,6 @@ from django.middleware.csrf import (
     _compare_salted_tokens as equivalent_tokens, get_token,
 )
 from django.test import SimpleTestCase, override_settings
-from django.test.utils import patch_logger
-from django.utils.encoding import force_bytes
-from django.utils.six import text_type
 from django.views.decorators.csrf import csrf_exempt, requires_csrf_token
 
 from .views import (
@@ -31,15 +24,14 @@ class TestingHttpRequest(HttpRequest):
     more easily
     """
     def __init__(self):
-        super(TestingHttpRequest, self).__init__()
-        # A real session backend isn't needed.
-        self.session = {}
+        super().__init__()
+        self.session = SessionStore()
 
     def is_secure(self):
         return getattr(self, '_is_secure_override', False)
 
 
-class CsrfViewMiddlewareTestMixin(object):
+class CsrfViewMiddlewareTestMixin:
     """
     Shared methods and tests for session-based and cookie-based tokens.
     """
@@ -69,8 +61,8 @@ class CsrfViewMiddlewareTestMixin(object):
         return req
 
     def _check_token_present(self, response, csrf_id=None):
-        text = text_type(response.content, response.charset)
-        match = re.search("name='csrfmiddlewaretoken' value='(.*?)'", text)
+        text = str(response.content, response.charset)
+        match = re.search('name="csrfmiddlewaretoken" value="(.*?)"', text)
         csrf_token = csrf_id or self._csrf_id
         self.assertTrue(
             match and equivalent_tokens(csrf_token, match.group(1)),
@@ -104,24 +96,24 @@ class CsrfViewMiddlewareTestMixin(object):
         If no CSRF cookies is present, the middleware rejects the incoming
         request. This will stop login CSRF.
         """
-        with patch_logger('django.security.csrf', 'warning') as logger_calls:
-            req = self._get_POST_no_csrf_cookie_request()
-            self.mw.process_request(req)
+        req = self._get_POST_no_csrf_cookie_request()
+        self.mw.process_request(req)
+        with self.assertLogs('django.security.csrf', 'WARNING') as cm:
             req2 = self.mw.process_view(req, post_form_view, (), {})
-            self.assertEqual(403, req2.status_code)
-            self.assertEqual(logger_calls[0], 'Forbidden (%s): ' % REASON_NO_CSRF_COOKIE)
+        self.assertEqual(403, req2.status_code)
+        self.assertEqual(cm.records[0].getMessage(), 'Forbidden (%s): ' % REASON_NO_CSRF_COOKIE)
 
     def test_process_request_csrf_cookie_no_token(self):
         """
         If a CSRF cookie is present but no token, the middleware rejects
         the incoming request.
         """
-        with patch_logger('django.security.csrf', 'warning') as logger_calls:
-            req = self._get_POST_csrf_cookie_request()
-            self.mw.process_request(req)
+        req = self._get_POST_csrf_cookie_request()
+        self.mw.process_request(req)
+        with self.assertLogs('django.security.csrf', 'WARNING') as cm:
             req2 = self.mw.process_view(req, post_form_view, (), {})
-            self.assertEqual(403, req2.status_code)
-            self.assertEqual(logger_calls[0], 'Forbidden (%s): ' % REASON_BAD_TOKEN)
+        self.assertEqual(403, req2.status_code)
+        self.assertEqual(cm.records[0].getMessage(), 'Forbidden (%s): ' % REASON_BAD_TOKEN)
 
     def test_process_request_csrf_cookie_and_token(self):
         """
@@ -169,17 +161,17 @@ class CsrfViewMiddlewareTestMixin(object):
         """
         req = TestingHttpRequest()
         req.method = 'PUT'
-        with patch_logger('django.security.csrf', 'warning') as logger_calls:
+        with self.assertLogs('django.security.csrf', 'WARNING') as cm:
             req2 = self.mw.process_view(req, post_form_view, (), {})
-            self.assertEqual(403, req2.status_code)
-            self.assertEqual(logger_calls[0], 'Forbidden (%s): ' % REASON_NO_CSRF_COOKIE)
+        self.assertEqual(403, req2.status_code)
+        self.assertEqual(cm.records[0].getMessage(), 'Forbidden (%s): ' % REASON_NO_CSRF_COOKIE)
 
         req = TestingHttpRequest()
         req.method = 'DELETE'
-        with patch_logger('django.security.csrf', 'warning') as logger_calls:
+        with self.assertLogs('django.security.csrf', 'WARNING') as cm:
             req2 = self.mw.process_view(req, post_form_view, (), {})
-            self.assertEqual(403, req2.status_code)
-            self.assertEqual(logger_calls[0], 'Forbidden (%s): ' % REASON_NO_CSRF_COOKIE)
+        self.assertEqual(403, req2.status_code)
+        self.assertEqual(cm.records[0].getMessage(), 'Forbidden (%s): ' % REASON_NO_CSRF_COOKIE)
 
     def test_put_and_delete_allowed(self):
         """
@@ -216,7 +208,7 @@ class CsrfViewMiddlewareTestMixin(object):
         A new token is sent if the csrf_cookie is the empty string.
         """
         req = self._get_GET_no_csrf_cookie_request()
-        req.COOKIES[settings.CSRF_COOKIE_NAME] = b""
+        req.COOKIES[settings.CSRF_COOKIE_NAME] = ""
         self.mw.process_view(req, token_view, (), {})
         resp = token_view(req)
 
@@ -300,6 +292,19 @@ class CsrfViewMiddlewareTestMixin(object):
             status_code=403,
         )
 
+    def test_https_malformed_host(self):
+        """
+        CsrfViewMiddleware generates a 403 response if it receives an HTTPS
+        request with a bad host.
+        """
+        req = self._get_GET_no_csrf_cookie_request()
+        req._is_secure_override = True
+        req.META['HTTP_HOST'] = '@malformed'
+        req.META['HTTP_REFERER'] = 'https://www.evil.org/somepage'
+        req.META['SERVER_PORT'] = '443'
+        response = self.mw.process_view(req, token_view, (), {})
+        self.assertEqual(response.status_code, 403)
+
     @override_settings(DEBUG=True)
     def test_https_malformed_referer(self):
         """
@@ -320,7 +325,7 @@ class CsrfViewMiddlewareTestMixin(object):
         response = self.mw.process_view(req, post_form_view, (), {})
         self.assertContains(response, malformed_referer_msg, status_code=403)
         # Non-ASCII
-        req.META['HTTP_REFERER'] = b'\xd8B\xf6I\xdf'
+        req.META['HTTP_REFERER'] = 'ØBöIß'
         response = self.mw.process_view(req, post_form_view, (), {})
         self.assertContains(response, malformed_referer_msg, status_code=403)
         # missing scheme
@@ -429,22 +434,10 @@ class CsrfViewMiddlewareTestMixin(object):
         """
         ensure_csrf_cookie() doesn't log warnings (#19436).
         """
-        class TestHandler(logging.Handler):
-            def emit(self, record):
-                raise Exception("This shouldn't have happened!")
-
-        logger = logging.getLogger('django.request')
-        test_handler = TestHandler()
-        old_log_level = logger.level
-        try:
-            logger.addHandler(test_handler)
-            logger.setLevel(logging.WARNING)
-
-            req = self._get_GET_no_csrf_cookie_request()
-            ensure_csrf_cookie_view(req)
-        finally:
-            logger.removeHandler(test_handler)
-            logger.setLevel(old_log_level)
+        with self.assertRaisesMessage(AssertionError, 'no logs'):
+            with self.assertLogs('django.request', 'WARNING'):
+                req = self._get_GET_no_csrf_cookie_request()
+                ensure_csrf_cookie_view(req)
 
     def test_post_data_read_failure(self):
         """
@@ -456,7 +449,7 @@ class CsrfViewMiddlewareTestMixin(object):
             HttpRequest that can raise an IOError when accessing POST data
             """
             def __init__(self, token, raise_error):
-                super(CsrfPostRequest, self).__init__()
+                super().__init__()
                 self.method = 'POST'
 
                 self.raise_error = False
@@ -491,11 +484,11 @@ class CsrfViewMiddlewareTestMixin(object):
         self.assertIsNone(resp)
 
         req = CsrfPostRequest(token, raise_error=True)
-        with patch_logger('django.security.csrf', 'warning') as logger_calls:
-            self.mw.process_request(req)
+        self.mw.process_request(req)
+        with self.assertLogs('django.security.csrf', 'WARNING') as cm:
             resp = self.mw.process_view(req, post_form_view, (), {})
-            self.assertEqual(resp.status_code, 403)
-            self.assertEqual(logger_calls[0], 'Forbidden (%s): ' % REASON_BAD_TOKEN)
+        self.assertEqual(resp.status_code, 403)
+        self.assertEqual(cm.records[0].getMessage(), 'Forbidden (%s): ' % REASON_BAD_TOKEN)
 
 
 class CsrfViewMiddlewareTests(CsrfViewMiddlewareTestMixin, SimpleTestCase):
@@ -579,6 +572,14 @@ class CsrfViewMiddlewareTests(CsrfViewMiddlewareTestMixin, SimpleTestCase):
             max_age = resp2.cookies.get('csrfcookie').get('max-age')
             self.assertEqual(max_age, '')
 
+    def test_csrf_cookie_samesite(self):
+        req = self._get_GET_no_csrf_cookie_request()
+        with self.settings(CSRF_COOKIE_NAME='csrfcookie', CSRF_COOKIE_SAMESITE='Strict'):
+            self.mw.process_view(req, token_view, (), {})
+            resp = token_view(req)
+            resp2 = self.mw.process_response(req, resp)
+            self.assertEqual(resp2.cookies['csrfcookie']['samesite'], 'Strict')
+
     def test_process_view_token_too_long(self):
         """
         If the token is longer than expected, it is ignored and a new token is
@@ -592,24 +593,6 @@ class CsrfViewMiddlewareTests(CsrfViewMiddlewareTestMixin, SimpleTestCase):
         csrf_cookie = resp2.cookies.get(settings.CSRF_COOKIE_NAME, False)
         self.assertEqual(len(csrf_cookie.value), CSRF_TOKEN_LENGTH)
 
-    def test_process_view_token_invalid_bytes(self):
-        """
-        If the token contains improperly encoded unicode, it is ignored and a
-        new token is created.
-        """
-        token = (b"<1>\xc2\xa1" + force_bytes(self._csrf_id, 'ascii'))[:CSRF_TOKEN_LENGTH]
-        req = self._get_GET_no_csrf_cookie_request()
-        req.COOKIES[settings.CSRF_COOKIE_NAME] = token
-        # We expect a UnicodeWarning here, because we used broken utf-8 on purpose
-        with warnings.catch_warnings():
-            warnings.filterwarnings("ignore", category=UnicodeWarning)
-            CsrfViewMiddleware().process_view(req, token_view, (), {})
-        resp = token_view(req)
-        resp2 = CsrfViewMiddleware().process_response(req, resp)
-        csrf_cookie = resp2.cookies.get(settings.CSRF_COOKIE_NAME, False)
-        self.assertEqual(len(csrf_cookie.value), CSRF_TOKEN_LENGTH)
-        self.assertNotEqual(csrf_cookie.value, token)
-
     def test_process_view_token_invalid_chars(self):
         """
         If the token contains non-alphanumeric characters, it is ignored and a
@@ -710,6 +693,19 @@ class CsrfViewMiddlewareUseSessionsTests(CsrfViewMiddlewareTestMixin, SimpleTest
         ensure_csrf_cookie_view(req)
         self.assertTrue(req.session.get(CSRF_SESSION_KEY, False))
 
+    def test_session_modify(self):
+        """The session isn't saved if the CSRF cookie is unchanged."""
+        req = self._get_GET_no_csrf_cookie_request()
+        self.mw.process_view(req, ensure_csrf_cookie_view, (), {})
+        resp = ensure_csrf_cookie_view(req)
+        self.mw.process_response(req, resp)
+        self.assertIsNotNone(req.session.get(CSRF_SESSION_KEY))
+        req.session.modified = False
+        self.mw.process_view(req, ensure_csrf_cookie_view, (), {})
+        resp = ensure_csrf_cookie_view(req)
+        self.mw.process_response(req, resp)
+        self.assertFalse(req.session.modified)
+
     def test_ensures_csrf_cookie_with_middleware(self):
         """
         The ensure_csrf_cookie() decorator works with the CsrfViewMiddleware
