@@ -1,9 +1,5 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from django.db import IntegrityError, transaction
 from django.test import TestCase, skipIfDBFeature
-from django.utils import six
 
 from .models import Bar, Business, Employee, Foo
 
@@ -28,14 +24,14 @@ class BasicCustomPKTests(TestCase):
             Employee.objects.filter(pk=123), [
                 "Dan Jones",
             ],
-            six.text_type
+            str
         )
 
         self.assertQuerysetEqual(
             Employee.objects.filter(employee_code=123), [
                 "Dan Jones",
             ],
-            six.text_type
+            str
         )
 
         self.assertQuerysetEqual(
@@ -43,7 +39,7 @@ class BasicCustomPKTests(TestCase):
                 "Fran Bones",
                 "Dan Jones",
             ],
-            six.text_type
+            str
         )
 
         self.assertQuerysetEqual(
@@ -51,7 +47,7 @@ class BasicCustomPKTests(TestCase):
                 "Fran Bones",
                 "Dan Jones",
             ],
-            six.text_type
+            str
         )
 
         self.assertQuerysetEqual(
@@ -76,7 +72,7 @@ class BasicCustomPKTests(TestCase):
                 "Fran Bones",
                 "Dan Jones",
             ],
-            six.text_type
+            str
         )
         self.assertQuerysetEqual(
             self.fran.business_set.all(), [
@@ -94,14 +90,14 @@ class BasicCustomPKTests(TestCase):
                 "Fran Bones",
                 "Dan Jones",
             ],
-            six.text_type,
+            str,
         )
         self.assertQuerysetEqual(
             Employee.objects.filter(business__pk="Sears"), [
                 "Fran Bones",
                 "Dan Jones",
             ],
-            six.text_type,
+            str,
         )
 
         self.assertQuerysetEqual(
@@ -131,10 +127,8 @@ class BasicCustomPKTests(TestCase):
         self.assertEqual(Employee.objects.get(pk=123), self.dan)
         self.assertEqual(Employee.objects.get(pk=456), self.fran)
 
-        self.assertRaises(
-            Employee.DoesNotExist,
-            lambda: Employee.objects.get(pk=42)
-        )
+        with self.assertRaises(Employee.DoesNotExist):
+            Employee.objects.get(pk=42)
 
         # Use the name of the primary key, rather than pk.
         self.assertEqual(Employee.objects.get(employee_code=123), self.dan)
@@ -151,7 +145,8 @@ class BasicCustomPKTests(TestCase):
         # Or we can use the real attribute name for the primary key:
         self.assertEqual(e.employee_code, 123)
 
-        self.assertRaises(AttributeError, lambda: e.id)
+        with self.assertRaisesMessage(AttributeError, "'Employee' object has no attribute 'id'"):
+            e.id
 
     def test_in_bulk(self):
         """
@@ -177,7 +172,7 @@ class BasicCustomPKTests(TestCase):
                 "Dan Jones",
                 "Fran Jones",
             ],
-            six.text_type
+            str
         )
 
 
