@@ -3,10 +3,8 @@ Regression tests for defer() / only() behavior.
 """
 
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Item(models.Model):
     name = models.CharField(max_length=15)
     text = models.TextField(default="xyzzy")
@@ -31,7 +29,6 @@ class Child(models.Model):
     value = models.IntegerField()
 
 
-@python_2_unicode_compatible
 class Leaf(models.Model):
     name = models.CharField(max_length=10)
     child = models.ForeignKey(Child, models.CASCADE)
@@ -52,7 +49,6 @@ class Proxy(Item):
         proxy = True
 
 
-@python_2_unicode_compatible
 class SimpleItem(models.Model):
     name = models.CharField(max_length=15)
     value = models.IntegerField()
@@ -80,11 +76,11 @@ class ItemAndSimpleItem(models.Model):
 
 
 class Profile(models.Model):
-    profile1 = models.CharField(max_length=1000, default='profile1')
+    profile1 = models.CharField(max_length=255, default='profile1')
 
 
 class Location(models.Model):
-    location1 = models.CharField(max_length=1000, default='location1')
+    location1 = models.CharField(max_length=255, default='location1')
 
 
 class Request(models.Model):
@@ -92,10 +88,10 @@ class Request(models.Model):
     location = models.ForeignKey(Location, models.CASCADE)
     items = models.ManyToManyField(Item)
 
-    request1 = models.CharField(default='request1', max_length=1000)
-    request2 = models.CharField(default='request2', max_length=1000)
-    request3 = models.CharField(default='request3', max_length=1000)
-    request4 = models.CharField(default='request4', max_length=1000)
+    request1 = models.CharField(default='request1', max_length=255)
+    request2 = models.CharField(default='request2', max_length=255)
+    request3 = models.CharField(default='request3', max_length=255)
+    request4 = models.CharField(default='request4', max_length=255)
 
 
 class Base(models.Model):
