@@ -1,24 +1,19 @@
-#! -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import base64
-import errno
 import hashlib
-import json
 import os
 import shutil
 import sys
 import tempfile as sys_tempfile
 import unittest
-from io import BytesIO
+from io import BytesIO, StringIO
+from urllib.parse import quote
 
 from django.core.files import temp as tempfile
 from django.core.files.uploadedfile import SimpleUploadedFile
-from django.http.multipartparser import MultiPartParser, parse_header
+from django.http.multipartparser import (
+    MultiPartParser, MultiPartParserError, parse_header,
+)
 from django.test import SimpleTestCase, TestCase, client, override_settings
-from django.utils.encoding import force_bytes
-from django.utils.http import urlquote
-from django.utils.six import PY2, StringIO
 
 from . import uploadhandler
 from .models import FileModel
@@ -33,14 +28,14 @@ class FileUploadTests(TestCase):
 
     @classmethod
     def setUpClass(cls):
-        super(FileUploadTests, cls).setUpClass()
+        super().setUpClass()
         if not os.path.isdir(MEDIA_ROOT):
             os.makedirs(MEDIA_ROOT)
 
     @classmethod
     def tearDownClass(cls):
         shutil.rmtree(MEDIA_ROOT)
-        super(FileUploadTests, cls).tearDownClass()
+        super().tearDownClass()
 
     def test_simple_upload(self):
         with open(__file__, 'rb') as fp:
@@ -71,7 +66,7 @@ class FileUploadTests(TestCase):
                     post_data[key + '_hash'] = hashlib.sha1(post_data[key].read()).hexdigest()
                     post_data[key].seek(0)
                 except AttributeError:
-                    post_data[key + '_hash'] = hashlib.sha1(force_bytes(post_data[key])).hexdigest()
+                    post_data[key + '_hash'] = hashlib.sha1(post_data[key].encode()).hexdigest()
 
             response = self.client.post('/verify/', post_data)
 
@@ -84,7 +79,7 @@ class FileUploadTests(TestCase):
             'Content-Type: application/octet-stream',
             'Content-Transfer-Encoding: base64',
             '']))
-        payload.write(b"\r\n" + encode(force_bytes(content)) + b"\r\n")
+        payload.write(b'\r\n' + encode(content.encode()) + b'\r\n')
         payload.write('--' + client.BOUNDARY + '--\r\n')
         r = {
             'CONTENT_LENGTH': len(payload),
@@ -94,9 +89,7 @@ class FileUploadTests(TestCase):
             'wsgi.input': payload,
         }
         response = self.client.request(**r)
-        received = json.loads(response.content.decode('utf-8'))
-
-        self.assertEqual(received['file'], content)
+        self.assertEqual(response.json()['file'], content)
 
     def test_base64_upload(self):
         self._test_base64_upload("This data will be transmitted base64-encoded.")
@@ -105,26 +98,16 @@ class FileUploadTests(TestCase):
         self._test_base64_upload("Big data" * 68000)  # > 512Kb
 
     def test_big_base64_newlines_upload(self):
-        self._test_base64_upload(
-            # encodestring is a deprecated alias on Python 3
-            "Big data" * 68000, encode=base64.encodestring if PY2 else base64.encodebytes)
+        self._test_base64_upload("Big data" * 68000, encode=base64.encodebytes)
 
     def test_unicode_file_name(self):
-        tdir = sys_tempfile.mkdtemp()
-        self.addCleanup(shutil.rmtree, tdir, True)
-
-        # This file contains Chinese symbols and an accented char in the name.
-        with open(os.path.join(tdir, UNICODE_FILENAME), 'w+b') as file1:
-            file1.write(b'b' * (2 ** 10))
-            file1.seek(0)
-
-            post_data = {
-                'file_unicode': file1,
-            }
-
-            response = self.client.post('/unicode_name/', post_data)
-
-        self.assertEqual(response.status_code, 200)
+        with sys_tempfile.TemporaryDirectory() as temp_dir:
+            # This file contains Chinese symbols and an accented char in the name.
+            with open(os.path.join(temp_dir, UNICODE_FILENAME), 'w+b') as file1:
+                file1.write(b'b' * (2 ** 10))
+                file1.seek(0)
+                response = self.client.post('/unicode_name/', {'file_unicode': file1})
+            self.assertEqual(response.status_code, 200)
 
     def test_unicode_file_name_rfc2231(self):
         """
@@ -134,7 +117,7 @@ class FileUploadTests(TestCase):
         payload = client.FakePayload()
         payload.write('\r\n'.join([
             '--' + client.BOUNDARY,
-            'Content-Disposition: form-data; name="file_unicode"; filename*=UTF-8\'\'%s' % urlquote(UNICODE_FILENAME),
+            'Content-Disposition: form-data; name="file_unicode"; filename*=UTF-8\'\'%s' % quote(UNICODE_FILENAME),
             'Content-Type: application/octet-stream',
             '',
             'You got pwnd.\r\n',
@@ -160,7 +143,7 @@ class FileUploadTests(TestCase):
         payload.write(
             '\r\n'.join([
                 '--' + client.BOUNDARY,
-                'Content-Disposition: form-data; name*=UTF-8\'\'file_unicode; filename*=UTF-8\'\'%s' % urlquote(
+                'Content-Disposition: form-data; name*=UTF-8\'\'file_unicode; filename*=UTF-8\'\'%s' % quote(
                     UNICODE_FILENAME
                 ),
                 'Content-Type: application/octet-stream',
@@ -211,7 +194,7 @@ class FileUploadTests(TestCase):
         self.assertEqual(response.status_code, 200)
 
         # Empty filenames should be ignored
-        received = json.loads(response.content.decode('utf-8'))
+        received = response.json()
         for i, name in enumerate(filenames):
             self.assertIsNone(received.get('file%s' % i))
 
@@ -255,9 +238,8 @@ class FileUploadTests(TestCase):
             'wsgi.input': payload,
         }
         response = self.client.request(**r)
-
         # The filenames should have been sanitized by the time it got to the view.
-        received = json.loads(response.content.decode('utf-8'))
+        received = response.json()
         for i, name in enumerate(scary_file_names):
             got = received["file%s" % i]
             self.assertEqual(got, "hax0rd.txt")
@@ -292,8 +274,7 @@ class FileUploadTests(TestCase):
             'wsgi.input': payload,
         }
         response = self.client.request(**r)
-
-        result = json.loads(response.content.decode('utf-8'))
+        result = response.json()
         for name, _, expected in cases:
             got = result[name]
             self.assertEqual(expected, got, 'Mismatch for {}'.format(name))
@@ -319,7 +300,7 @@ class FileUploadTests(TestCase):
                 'string': string_io,
                 'binary': bytes_io,
             })
-            received = json.loads(response.content.decode('utf-8'))
+            received = response.json()
             self.assertEqual(received['no_content_type'], 'no content')
             self.assertEqual(received['simple_file'], 'text content')
             self.assertEqual(received['string'], 'string content')
@@ -340,7 +321,7 @@ class FileUploadTests(TestCase):
                 'no_content_type': no_content_type,
                 'simple_file': simple_file,
             })
-            received = json.loads(response.content.decode('utf-8'))
+            received = response.json()
             self.assertEqual(received['no_content_type'], {})
             self.assertEqual(received['simple_file'], {'test-key': 'test_value'})
 
@@ -367,8 +348,7 @@ class FileUploadTests(TestCase):
             'REQUEST_METHOD': 'POST',
             'wsgi.input': payload,
         }
-        got = json.loads(self.client.request(**r).content.decode('utf-8'))
-        self.assertEqual(got, {})
+        self.assertEqual(self.client.request(**r).json(), {})
 
     def test_empty_multipart_handled_gracefully(self):
         """
@@ -382,8 +362,7 @@ class FileUploadTests(TestCase):
             'REQUEST_METHOD': 'POST',
             'wsgi.input': client.FakePayload(b''),
         }
-        got = json.loads(self.client.request(**r).content.decode('utf-8'))
-        self.assertEqual(got, {})
+        self.assertEqual(self.client.request(**r).json(), {})
 
     def test_custom_upload_handler(self):
         file = tempfile.NamedTemporaryFile
@@ -397,22 +376,18 @@ class FileUploadTests(TestCase):
             bigfile.seek(0)
 
             # Small file posting should work.
-            response = self.client.post('/quota/', {'f': smallfile})
-            got = json.loads(response.content.decode('utf-8'))
-            self.assertIn('f', got)
+            self.assertIn('f', self.client.post('/quota/', {'f': smallfile}).json())
 
             # Large files don't go through.
-            response = self.client.post("/quota/", {'f': bigfile})
-            got = json.loads(response.content.decode('utf-8'))
-            self.assertNotIn('f', got)
+            self.assertNotIn('f', self.client.post("/quota/", {'f': bigfile}).json())
 
     def test_broken_custom_upload_handler(self):
         with tempfile.NamedTemporaryFile() as file:
             file.write(b'a' * (2 ** 21))
             file.seek(0)
 
-            # AttributeError: You cannot alter upload handlers after the upload has been processed.
-            with self.assertRaises(AttributeError):
+            msg = 'You cannot alter upload handlers after the upload has been processed.'
+            with self.assertRaisesMessage(AttributeError, msg):
                 self.client.post('/quota/broken/', {'f': file})
 
     def test_fileupload_getlist(self):
@@ -436,8 +411,7 @@ class FileUploadTests(TestCase):
                 'field5': 'test7',
                 'file2': (file2, file2a)
             })
-            got = json.loads(response.content.decode('utf-8'))
-
+            got = response.json()
             self.assertEqual(got.get('file1'), 1)
             self.assertEqual(got.get('file2'), 2)
 
@@ -482,7 +456,7 @@ class FileUploadTests(TestCase):
         class POSTAccessingHandler(client.ClientHandler):
             """A handler that'll access POST during an exception."""
             def handle_uncaught_exception(self, request, resolver, exc_info):
-                ret = super(POSTAccessingHandler, self).handle_uncaught_exception(request, resolver, exc_info)
+                ret = super().handle_uncaught_exception(request, resolver, exc_info)
                 request.POST  # evaluate
                 return ret
 
@@ -553,14 +527,14 @@ class DirectoryCreationTests(SimpleTestCase):
     """
     @classmethod
     def setUpClass(cls):
-        super(DirectoryCreationTests, cls).setUpClass()
+        super().setUpClass()
         if not os.path.isdir(MEDIA_ROOT):
             os.makedirs(MEDIA_ROOT)
 
     @classmethod
     def tearDownClass(cls):
         shutil.rmtree(MEDIA_ROOT)
-        super(DirectoryCreationTests, cls).tearDownClass()
+        super().tearDownClass()
 
     def setUp(self):
         self.obj = FileModel()
@@ -570,9 +544,8 @@ class DirectoryCreationTests(SimpleTestCase):
         """Permission errors are not swallowed"""
         os.chmod(MEDIA_ROOT, 0o500)
         self.addCleanup(os.chmod, MEDIA_ROOT, 0o700)
-        with self.assertRaises(OSError) as cm:
+        with self.assertRaises(PermissionError):
             self.obj.testfile.save('foo.txt', SimpleUploadedFile('foo.txt', b'x'), save=False)
-        self.assertEqual(cm.exception.errno, errno.EACCES)
 
     def test_not_a_directory(self):
         """The correct IOError is raised when the upload directory name exists but isn't a directory"""
@@ -587,7 +560,7 @@ class DirectoryCreationTests(SimpleTestCase):
         self.assertEqual(exc_info.exception.args[0], "%s exists and is not a directory." % UPLOAD_TO)
 
 
-class MultiParserTests(unittest.TestCase):
+class MultiParserTests(SimpleTestCase):
 
     def test_empty_upload_handlers(self):
         # We're not actually parsing here; just checking if the parser properly
@@ -597,6 +570,27 @@ class MultiParserTests(unittest.TestCase):
             'CONTENT_LENGTH': '1'
         }, StringIO('x'), [], 'utf-8')
 
+    def test_invalid_content_type(self):
+        with self.assertRaisesMessage(MultiPartParserError, 'Invalid Content-Type: text/plain'):
+            MultiPartParser({
+                'CONTENT_TYPE': 'text/plain',
+                'CONTENT_LENGTH': '1',
+            }, StringIO('x'), [], 'utf-8')
+
+    def test_negative_content_length(self):
+        with self.assertRaisesMessage(MultiPartParserError, 'Invalid content length: -1'):
+            MultiPartParser({
+                'CONTENT_TYPE': 'multipart/form-data; boundary=_foo',
+                'CONTENT_LENGTH': -1,
+            }, StringIO('x'), [], 'utf-8')
+
+    def test_bad_type_content_length(self):
+        multipart_parser = MultiPartParser({
+            'CONTENT_TYPE': 'multipart/form-data; boundary=_foo',
+            'CONTENT_LENGTH': 'a',
+        }, StringIO('x'), [], 'utf-8')
+        self.assertEqual(multipart_parser._content_length, 0)
+
     def test_rfc2231_parsing(self):
         test_data = (
             (b"Content-Type: application/x-stuff; title*=us-ascii'en-us'This%20is%20%2A%2A%2Afun%2A%2A%2A",
