@@ -1,10 +1,10 @@
-from __future__ import unicode_literals
-
 import os
 import sys
 import tempfile
 import unittest
 import warnings
+from io import StringIO
+from unittest import mock
 
 from django.apps import apps
 from django.contrib.sites.models import Site
@@ -14,20 +14,19 @@ from django.core.management import CommandError
 from django.core.management.commands.dumpdata import ProxyModelWarning
 from django.core.serializers.base import ProgressBar
 from django.db import IntegrityError, connection
-from django.test import (
-    TestCase, TransactionTestCase, mock, skipUnlessDBFeature,
-)
-from django.utils import six
-from django.utils.encoding import force_text
+from django.test import TestCase, TransactionTestCase, skipUnlessDBFeature
 
-from .models import Article, ProxySpy, Spy, Tag, Visa
+from .models import (
+    Article, Category, NaturalKeyThing, PrimaryKeyUUIDModel, ProxySpy, Spy,
+    Tag, Visa,
+)
 
 
 class TestCaseFixtureLoadingTests(TestCase):
     fixtures = ['fixture1.json', 'fixture2.json']
 
     def testClassFixtures(self):
-        "Check that test case has installed 3 fixture objects"
+        "Test case has installed 3 fixture objects"
         self.assertEqual(Article.objects.count(), 3)
         self.assertQuerysetEqual(Article.objects.all(), [
             '<Article: Django conquers world!>',
@@ -43,27 +42,30 @@ class SubclassTestCaseFixtureLoadingTests(TestCaseFixtureLoadingTests):
     fixtures = []
 
     def testClassFixtures(self):
-        "Check that there were no fixture objects installed"
+        "There were no fixture objects installed"
         self.assertEqual(Article.objects.count(), 0)
 
 
-class DumpDataAssertMixin(object):
+class DumpDataAssertMixin:
 
     def _dumpdata_assert(self, args, output, format='json', filename=None,
                          natural_foreign_keys=False, natural_primary_keys=False,
                          use_base_manager=False, exclude_list=[], primary_keys=''):
-        new_io = six.StringIO()
-        if filename:
-            filename = os.path.join(tempfile.gettempdir(), filename)
-        management.call_command('dumpdata', *args, **{'format': format,
-                                                      'stdout': new_io,
-                                                      'stderr': new_io,
-                                                      'output': filename,
-                                                      'use_natural_foreign_keys': natural_foreign_keys,
-                                                      'use_natural_primary_keys': natural_primary_keys,
-                                                      'use_base_manager': use_base_manager,
-                                                      'exclude': exclude_list,
-                                                      'primary_keys': primary_keys})
+        new_io = StringIO()
+        filename = filename and os.path.join(tempfile.gettempdir(), filename)
+        management.call_command(
+            'dumpdata',
+            *args,
+            format=format,
+            stdout=new_io,
+            stderr=new_io,
+            output=filename,
+            use_natural_foreign_keys=natural_foreign_keys,
+            use_natural_primary_keys=natural_primary_keys,
+            use_base_manager=use_base_manager,
+            exclude=exclude_list,
+            primary_keys=primary_keys,
+        )
         if filename:
             with open(filename, "r") as f:
                 command_output = f.read()
@@ -337,7 +339,8 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         self._dumpdata_assert(
             ['sites', 'fixtures'],
             '[{"pk": 1, "model": "sites.site", "fields": {"domain": "example.com", "name": "example.com"}}]',
-            exclude_list=['fixtures'])
+            exclude_list=['fixtures'],
+        )
 
         # Excluding fixtures.Article/Book should leave fixtures.Category
         self._dumpdata_assert(
@@ -370,7 +373,7 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             self._dumpdata_assert(['fixtures', 'sites'], '', exclude_list=['foo_app'])
 
         # Excluding a bogus model should throw an error
-        with self.assertRaisesMessage(management.CommandError, "Unknown model in excludes: fixtures.FooModel"):
+        with self.assertRaisesMessage(management.CommandError, "Unknown model: fixtures.FooModel"):
             self._dumpdata_assert(['fixtures', 'sites'], '', exclude_list=['fixtures.FooModel'])
 
     @unittest.skipIf(sys.platform.startswith('win'), "Windows doesn't support '?' in filenames.")
@@ -442,6 +445,18 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
                 primary_keys='2,3'
             )
 
+    def test_dumpdata_with_uuid_pks(self):
+        m1 = PrimaryKeyUUIDModel.objects.create()
+        m2 = PrimaryKeyUUIDModel.objects.create()
+        output = StringIO()
+        management.call_command(
+            'dumpdata', 'fixtures.PrimaryKeyUUIDModel', '--pks', ', '.join([str(m1.id), str(m2.id)]),
+            stdout=output,
+        )
+        result = output.getvalue()
+        self.assertIn('"pk": "%s"' % m1.id, result)
+        self.assertIn('"pk": "%s"' % m2.id, result)
+
     def test_dumpdata_with_file_output(self):
         management.call_command('loaddata', 'fixture1.json', verbosity=0)
         self._dumpdata_assert(
@@ -459,7 +474,7 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         stdout is a tty, and verbosity > 0.
         """
         management.call_command('loaddata', 'fixture1.json', verbosity=0)
-        new_io = six.StringIO()
+        new_io = StringIO()
         new_io.isatty = lambda: True
         with NamedTemporaryFile() as file:
             options = {
@@ -473,7 +488,7 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
 
             # Test no progress bar when verbosity = 0
             options['verbosity'] = 0
-            new_io = six.StringIO()
+            new_io = StringIO()
             new_io.isatty = lambda: True
             options.update({'stdout': new_io, 'stderr': new_io})
             management.call_command('dumpdata', 'fixtures', **options)
@@ -485,16 +500,9 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         parent.
         """
         ProxySpy.objects.create(name='Paul')
-
-        with warnings.catch_warnings(record=True) as warning_list:
-            warnings.simplefilter('always')
+        msg = "fixtures.ProxySpy is a proxy model and won't be serialized."
+        with self.assertWarnsMessage(ProxyModelWarning, msg):
             self._dumpdata_assert(['fixtures.ProxySpy'], '[]')
-        warning = warning_list.pop()
-        self.assertEqual(warning.category, ProxyModelWarning)
-        self.assertEqual(
-            str(warning.message),
-            "fixtures.ProxySpy is a proxy model and won't be serialized."
-        )
 
     def test_dumpdata_proxy_with_concrete(self):
         """
@@ -549,9 +557,8 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
 
     def test_loaddata_error_message(self):
         """
-        Verifies that loading a fixture which contains an invalid object
-        outputs an error message which contains the pk of the object
-        that triggered the error.
+        Loading a fixture which contains an invalid object outputs an error
+        message which contains the pk of the object that triggered the error.
         """
         # MySQL needs a little prodding to reject invalid data.
         # This won't affect other tests because the database connection
@@ -562,10 +569,16 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             management.call_command('loaddata', 'invalid.json', verbosity=0)
             self.assertIn("Could not load fixtures.Article(pk=1):", cm.exception.args[0])
 
+    @unittest.skipUnless(connection.vendor == 'postgresql', 'psycopg2 prohibits null characters in data.')
+    def test_loaddata_null_characters_on_postgresql(self):
+        msg = (
+            'Could not load fixtures.Article(pk=2): '
+            'A string literal cannot contain NUL (0x00) characters.'
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            management.call_command('loaddata', 'null_character_in_field_value.json')
+
     def test_loaddata_app_option(self):
-        """
-        Verifies that the --app option works.
-        """
         with self.assertRaisesMessage(CommandError, "No fixture named 'db_fixture_1' found."):
             management.call_command('loaddata', 'db_fixture_1', verbosity=0, app_label="someotherapp")
         self.assertQuerysetEqual(Article.objects.all(), [])
@@ -575,9 +588,9 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         ])
 
     def test_loaddata_verbosity_three(self):
-        output = six.StringIO()
+        output = StringIO()
         management.call_command('loaddata', 'fixture1.json', verbosity=3, stdout=output, stderr=output)
-        command_output = force_text(output.getvalue())
+        command_output = output.getvalue()
         self.assertIn(
             "\rProcessed 1 object(s).\rProcessed 2 object(s)."
             "\rProcessed 3 object(s).\rProcessed 4 object(s).\n",
@@ -586,8 +599,8 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
 
     def test_loading_using(self):
         # Load db fixtures 1 and 2. These will load using the 'default' database identifier explicitly
-        management.call_command('loaddata', 'db_fixture_1', verbosity=0, using='default')
-        management.call_command('loaddata', 'db_fixture_2', verbosity=0, using='default')
+        management.call_command('loaddata', 'db_fixture_1', verbosity=0, database='default')
+        management.call_command('loaddata', 'db_fixture_2', verbosity=0, database='default')
         self.assertQuerysetEqual(Article.objects.all(), [
             '<Article: Who needs more than one database?>',
             '<Article: Who needs to use compressed data?>',
@@ -598,7 +611,7 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         with self.assertRaisesMessage(CommandError, "No fixture named 'db_fixture_3' found."):
             management.call_command('loaddata', 'db_fixture_3', verbosity=0)
         with self.assertRaisesMessage(CommandError, "No fixture named 'db_fixture_3' found."):
-            management.call_command('loaddata', 'db_fixture_3', verbosity=0, using='default')
+            management.call_command('loaddata', 'db_fixture_3', verbosity=0, database='default')
         self.assertQuerysetEqual(Article.objects.all(), [])
 
     def test_output_formats(self):
@@ -650,15 +663,67 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             format='xml', natural_foreign_keys=True
         )
 
+    def test_loading_with_exclude_app(self):
+        Site.objects.all().delete()
+        management.call_command('loaddata', 'fixture1', exclude=['fixtures'], verbosity=0)
+        self.assertFalse(Article.objects.exists())
+        self.assertFalse(Category.objects.exists())
+        self.assertQuerysetEqual(Site.objects.all(), ['<Site: example.com>'])
 
-class NonExistentFixtureTests(TestCase):
+    def test_loading_with_exclude_model(self):
+        Site.objects.all().delete()
+        management.call_command('loaddata', 'fixture1', exclude=['fixtures.Article'], verbosity=0)
+        self.assertFalse(Article.objects.exists())
+        self.assertQuerysetEqual(Category.objects.all(), ['<Category: News Stories>'])
+        self.assertQuerysetEqual(Site.objects.all(), ['<Site: example.com>'])
+
+    def test_exclude_option_errors(self):
+        """Excluding a bogus app or model should raise an error."""
+        msg = "No installed app with label 'foo_app'."
+        with self.assertRaisesMessage(management.CommandError, msg):
+            management.call_command('loaddata', 'fixture1', exclude=['foo_app'], verbosity=0)
+
+        msg = "Unknown model: fixtures.FooModel"
+        with self.assertRaisesMessage(management.CommandError, msg):
+            management.call_command('loaddata', 'fixture1', exclude=['fixtures.FooModel'], verbosity=0)
+
+    def test_stdin_without_format(self):
+        """Reading from stdin raises an error if format isn't specified."""
+        msg = '--format must be specified when reading from stdin.'
+        with self.assertRaisesMessage(management.CommandError, msg):
+            management.call_command('loaddata', '-', verbosity=0)
+
+    def test_loading_stdin(self):
+        """Loading fixtures from stdin with json and xml."""
+        tests_dir = os.path.dirname(__file__)
+        fixture_json = os.path.join(tests_dir, 'fixtures', 'fixture1.json')
+        fixture_xml = os.path.join(tests_dir, 'fixtures', 'fixture3.xml')
+
+        with mock.patch('django.core.management.commands.loaddata.sys.stdin', open(fixture_json, 'r')):
+            management.call_command('loaddata', '--format=json', '-', verbosity=0)
+            self.assertEqual(Article.objects.count(), 2)
+            self.assertQuerysetEqual(Article.objects.all(), [
+                '<Article: Time to reform copyright>',
+                '<Article: Poker has no place on ESPN>',
+            ])
+
+        with mock.patch('django.core.management.commands.loaddata.sys.stdin', open(fixture_xml, 'r')):
+            management.call_command('loaddata', '--format=xml', '-', verbosity=0)
+            self.assertEqual(Article.objects.count(), 3)
+            self.assertQuerysetEqual(Article.objects.all(), [
+                '<Article: XML identified as leading cause of cancer>',
+                '<Article: Time to reform copyright>',
+                '<Article: Poker on TV is great!>',
+            ])
+
+
+class NonexistentFixtureTests(TestCase):
     """
     Custom class to limit fixture dirs.
     """
-    available_apps = ['django.contrib.auth', 'django.contrib.contenttypes']
 
     def test_loaddata_not_existent_fixture_file(self):
-        stdout_output = six.StringIO()
+        stdout_output = StringIO()
         with self.assertRaisesMessage(CommandError, "No fixture named 'this_fixture_doesnt_exist' found."):
             management.call_command('loaddata', 'this_fixture_doesnt_exist', stdout=stdout_output)
 
@@ -680,8 +745,6 @@ class FixtureTransactionTests(DumpDataAssertMixin, TransactionTestCase):
 
     available_apps = [
         'fixtures',
-        'django.contrib.contenttypes',
-        'django.contrib.auth',
         'django.contrib.sites',
     ]
 
@@ -722,3 +785,21 @@ class FixtureTransactionTests(DumpDataAssertMixin, TransactionTestCase):
             '<Article: Time to reform copyright>',
             '<Article: Poker has no place on ESPN>',
         ])
+
+
+class ForwardReferenceTests(TestCase):
+    def test_forward_reference_fk(self):
+        management.call_command('loaddata', 'forward_reference_fk.json', verbosity=0)
+        self.assertEqual(NaturalKeyThing.objects.count(), 2)
+        t1, t2 = NaturalKeyThing.objects.all()
+        self.assertEqual(t1.other_thing, t2)
+        self.assertEqual(t2.other_thing, t1)
+
+    def test_forward_reference_m2m(self):
+        management.call_command('loaddata', 'forward_reference_m2m.json', verbosity=0)
+        self.assertEqual(NaturalKeyThing.objects.count(), 3)
+        t1 = NaturalKeyThing.objects.get_by_natural_key('t1')
+        self.assertQuerysetEqual(
+            t1.other_things.order_by('key'),
+            ['<NaturalKeyThing: t2>', '<NaturalKeyThing: t3>']
+        )
