@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import re
 
 from django.forms import RegexField, ValidationError
@@ -45,10 +42,10 @@ class RegexFieldTest(SimpleTestCase):
         f = RegexField('^[0-9]+$', min_length=5, max_length=10)
         with self.assertRaisesMessage(ValidationError, "'Ensure this value has at least 5 characters (it has 3).'"):
             f.clean('123')
-        with self.assertRaisesRegex(
+        with self.assertRaisesMessage(
             ValidationError,
-            r"'Ensure this value has at least 5 characters \(it has 3\)\.',"
-            r" u?'Enter a valid value\.'",
+            "'Ensure this value has at least 5 characters (it has 3).', "
+            "'Enter a valid value.'",
         ):
             f.clean('abc')
         self.assertEqual('12345', f.clean('12345'))
