@@ -15,7 +15,7 @@ class WidgetTest(SimpleTestCase):
         cls.django_renderer = DjangoTemplates()
         cls.jinja2_renderer = Jinja2() if jinja2 else None
         cls.renderers = [cls.django_renderer] + ([cls.jinja2_renderer] if cls.jinja2_renderer else [])
-        super(WidgetTest, cls).setUpClass()
+        super().setUpClass()
 
     def check_html(self, widget, name, value, html='', attrs=None, strict=False, **kwargs):
         assertEqual = self.assertEqual if strict else self.assertHTMLEqual
