@@ -11,9 +11,9 @@ class FileInputTest(WidgetTest):
         FileInput widgets never render the value attribute. The old value
         isn't useful if a form is updated or an error occurred.
         """
-        self.check_html(self.widget, 'email', 'test@example.com', html='<input type="file" name="email" />')
-        self.check_html(self.widget, 'email', '', html='<input type="file" name="email" />')
-        self.check_html(self.widget, 'email', None, html='<input type="file" name="email" />')
+        self.check_html(self.widget, 'email', 'test@example.com', html='<input type="file" name="email">')
+        self.check_html(self.widget, 'email', '', html='<input type="file" name="email">')
+        self.check_html(self.widget, 'email', None, html='<input type="file" name="email">')
 
     def test_value_omitted_from_data(self):
         self.assertIs(self.widget.value_omitted_from_data({}, {}, 'field'), True)
