@@ -131,10 +131,12 @@ class GenericRelationTests(TestCase):
         note = Note(note='Deserve a bonus', content_object=team1)
         note.save()
 
-    def test_target_model_nonzero_false(self):
-        """Test related to #13085"""
-        # __nonzero__() returns False -- This actually doesn't currently fail.
-        # This test validates that
+    def test_target_model_bool_false(self):
+        """
+        Saving a model with a GenericForeignKey to a model instance whose
+        __bool__ method returns False (Guild.__bool__() here) shouldn't fail
+        (#13085).
+        """
         g1 = Guild.objects.create(name='First guild')
         note = Note(note='Note for guild', content_object=g1)
         note.save()
@@ -261,3 +263,22 @@ class GenericRelationTests(TestCase):
         place = Place.objects.create()
         Link.objects.create(content_object=place)
         self.assertEqual(Place.objects.get(link_proxy__object_id=place.id), place)
+
+    def test_generic_reverse_relation_with_mti(self):
+        """
+        Filtering with a reverse generic relation, where the GenericRelation
+        comes from multi-table inheritance.
+        """
+        place = Place.objects.create(name='Test Place')
+        link = Link.objects.create(content_object=place)
+        result = Link.objects.filter(places=place)
+        self.assertCountEqual(result, [link])
+
+    def test_generic_reverse_relation_with_abc(self):
+        """
+        The reverse generic relation accessor (targets) is created if the
+        GenericRelation comes from an abstract base model (HasLinks).
+        """
+        thing = HasLinkThing.objects.create()
+        link = Link.objects.create(content_object=thing)
+        self.assertCountEqual(link.targets.all(), [thing])
