@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.contrib.auth.decorators import login_required
 from django.core.paginator import Paginator
 from django.urls import reverse, reverse_lazy
@@ -14,7 +12,7 @@ class CustomTemplateView(generic.TemplateView):
     template_name = 'generic_views/about.html'
 
     def get_context_data(self, **kwargs):
-        context = super(CustomTemplateView, self).get_context_data(**kwargs)
+        context = super().get_context_data(**kwargs)
         context.update({'key': 'value'})
         return context
 
@@ -68,28 +66,25 @@ class AuthorList(generic.ListView):
     queryset = Author.objects.all()
 
 
+class AuthorListGetQuerysetReturnsNone(AuthorList):
+    def get_queryset(self):
+        return None
+
+
 class BookList(generic.ListView):
     model = Book
 
 
 class CustomPaginator(Paginator):
     def __init__(self, queryset, page_size, orphans=0, allow_empty_first_page=True):
-        super(CustomPaginator, self).__init__(
-            queryset,
-            page_size,
-            orphans=2,
-            allow_empty_first_page=allow_empty_first_page)
+        super().__init__(queryset, page_size, orphans=2, allow_empty_first_page=allow_empty_first_page)
 
 
 class AuthorListCustomPaginator(AuthorList):
     paginate_by = 5
 
     def get_paginator(self, queryset, page_size, orphans=0, allow_empty_first_page=True):
-        return super(AuthorListCustomPaginator, self).get_paginator(
-            queryset,
-            page_size,
-            orphans=2,
-            allow_empty_first_page=allow_empty_first_page)
+        return super().get_paginator(queryset, page_size, orphans=2, allow_empty_first_page=allow_empty_first_page)
 
 
 class ContactView(generic.FormView):
@@ -154,7 +149,7 @@ class AuthorUpdate(generic.UpdateView):
 
     def get_form(self, *args, **kwargs):
         self.get_form_called_count += 1
-        return super(AuthorUpdate, self).get_form(*args, **kwargs)
+        return super().get_form(*args, **kwargs)
 
 
 class OneAuthorUpdate(generic.UpdateView):
@@ -191,7 +186,7 @@ class SpecializedAuthorDelete(generic.DeleteView):
     success_url = reverse_lazy('authors_list')
 
 
-class BookConfig(object):
+class BookConfig:
     queryset = Book.objects.all()
     date_field = 'pubdate'
 
@@ -233,8 +228,7 @@ class AuthorGetQuerySetFormView(generic.edit.ModelFormMixin):
 
 class BookDetailGetObjectCustomQueryset(BookDetail):
     def get_object(self, queryset=None):
-        return super(BookDetailGetObjectCustomQueryset, self).get_object(
-            queryset=Book.objects.filter(pk=self.kwargs['pk']))
+        return super().get_object(queryset=Book.objects.filter(pk=self.kwargs['pk']))
 
 
 class CustomMultipleObjectMixinView(generic.list.MultipleObjectMixin, generic.View):
@@ -257,7 +251,7 @@ class CustomContextView(generic.detail.SingleObjectMixin, generic.View):
     def get_context_data(self, **kwargs):
         context = {'custom_key': 'custom_value'}
         context.update(kwargs)
-        return super(CustomContextView, self).get_context_data(**context)
+        return super().get_context_data(**context)
 
     def get_context_object_name(self, obj):
         return "test_name"
@@ -268,7 +262,7 @@ class CustomSingleObjectView(generic.detail.SingleObjectMixin, generic.View):
     object = Book(name="dummy")
 
 
-class BookSigningConfig(object):
+class BookSigningConfig:
     model = BookSigning
     date_field = 'event_date'
     # use the same templates as for books
@@ -301,11 +295,15 @@ class BookSigningTodayArchive(BookSigningConfig, generic.TodayArchiveView):
     pass
 
 
+class BookArchiveWithoutDateField(generic.ArchiveIndexView):
+    queryset = Book.objects.all()
+
+
 class BookSigningDetail(BookSigningConfig, generic.DateDetailView):
     context_object_name = 'book'
 
 
-class NonModel(object):
+class NonModel:
     id = "non_model_1"
 
     _meta = None
