@@ -1,10 +1,6 @@
-from __future__ import unicode_literals
-
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Person(models.Model):
     first_name = models.CharField(max_length=100, unique=True)
     last_name = models.CharField(max_length=100)
@@ -33,7 +29,7 @@ class Tag(models.Model):
 
 
 class Thing(models.Model):
-    name = models.CharField(max_length=256)
+    name = models.CharField(max_length=255)
     tags = models.ManyToManyField(Tag)
 
     @property
