@@ -1,18 +1,13 @@
-from django.utils.encoding import python_2_unicode_compatible
+from django.contrib.gis.db import models
 
-from ..models import models
 from ..utils import gisfield_may_be_null
 
 
-@python_2_unicode_compatible
 class NamedModel(models.Model):
     name = models.CharField(max_length=30)
 
-    objects = models.GeoManager()
-
     class Meta:
         abstract = True
-        required_db_features = ['gis_enabled']
 
     def __str__(self):
         return self.name
