@@ -1,67 +1,65 @@
 import os
-import unittest
-from unittest import skipUnless
+import re
 
-from django.contrib.gis.gdal import HAS_GDAL
+from django.contrib.gis.gdal import (
+    GDAL_VERSION, DataSource, Envelope, GDALException, OGRGeometry,
+)
+from django.contrib.gis.gdal.field import OFTInteger, OFTReal, OFTString
+from django.test import SimpleTestCase
 
 from ..test_data import TEST_DATA, TestDS, get_ds_file
 
-if HAS_GDAL:
-    from django.contrib.gis.gdal import DataSource, Envelope, OGRGeometry, GDALException, OGRIndexError, GDAL_VERSION
-    from django.contrib.gis.gdal.field import OFTReal, OFTInteger, OFTString
+wgs_84_wkt = (
+    'GEOGCS["GCS_WGS_1984",DATUM["WGS_1984",SPHEROID["WGS_1984",'
+    '6378137,298.257223563]],PRIMEM["Greenwich",0],UNIT["Degree",'
+    '0.017453292519943295]]'
+)
+# Using a regex because of small differences depending on GDAL versions.
+wgs_84_wkt_regex = r'^GEOGCS\["(GCS_)?WGS[ _](19)?84".*$'
 
-    # List of acceptable data sources.
-    ds_list = (
-        TestDS(
-            'test_point', nfeat=5, nfld=3, geom='POINT', gtype=1, driver='ESRI Shapefile',
-            fields={'dbl': OFTReal, 'int': OFTInteger, 'str': OFTString},
-            extent=(-1.35011, 0.166623, -0.524093, 0.824508),  # Got extent from QGIS
-            srs_wkt=(
-                'GEOGCS["GCS_WGS_1984",DATUM["WGS_1984",SPHEROID["WGS_1984",'
-                '6378137,298.257223563]],PRIMEM["Greenwich",0],UNIT["Degree",'
-                '0.017453292519943295]]'
-            ),
-            field_values={
-                'dbl': [float(i) for i in range(1, 6)],
-                'int': list(range(1, 6)),
-                'str': [str(i) for i in range(1, 6)],
-            },
-            fids=range(5)
-        ),
-        TestDS(
-            'test_vrt', ext='vrt', nfeat=3, nfld=3, geom='POINT', gtype='Point25D',
-            driver='OGR_VRT' if GDAL_VERSION >= (2, 0) else 'VRT',
-            fields={
-                'POINT_X': OFTString,
-                'POINT_Y': OFTString,
-                'NUM': OFTString,
-            },  # VRT uses CSV, which all types are OFTString.
-            extent=(1.0, 2.0, 100.0, 523.5),  # Min/Max from CSV
-            field_values={
-                'POINT_X': ['1.0', '5.0', '100.0'],
-                'POINT_Y': ['2.0', '23.0', '523.5'],
-                'NUM': ['5', '17', '23'],
-            },
-            fids=range(1, 4)
-        ),
-        TestDS(
-            'test_poly', nfeat=3, nfld=3, geom='POLYGON', gtype=3,
-            driver='ESRI Shapefile',
-            fields={'float': OFTReal, 'int': OFTInteger, 'str': OFTString},
-            extent=(-1.01513, -0.558245, 0.161876, 0.839637),  # Got extent from QGIS
-            srs_wkt=(
-                'GEOGCS["GCS_WGS_1984",DATUM["WGS_1984",SPHEROID["WGS_1984",'
-                '6378137,298.257223563]],PRIMEM["Greenwich",0],UNIT["Degree",'
-                '0.017453292519943295]]'
-            ),
-        )
+# List of acceptable data sources.
+ds_list = (
+    TestDS(
+        'test_point', nfeat=5, nfld=3, geom='POINT', gtype=1, driver='ESRI Shapefile',
+        fields={'dbl': OFTReal, 'int': OFTInteger, 'str': OFTString},
+        extent=(-1.35011, 0.166623, -0.524093, 0.824508),  # Got extent from QGIS
+        srs_wkt=wgs_84_wkt,
+        field_values={
+            'dbl': [float(i) for i in range(1, 6)],
+            'int': list(range(1, 6)),
+            'str': [str(i) for i in range(1, 6)],
+        },
+        fids=range(5)
+    ),
+    TestDS(
+        'test_vrt', ext='vrt', nfeat=3, nfld=3, geom='POINT', gtype='Point25D',
+        driver='OGR_VRT' if GDAL_VERSION >= (2, 0) else 'VRT',
+        fields={
+            'POINT_X': OFTString,
+            'POINT_Y': OFTString,
+            'NUM': OFTString,
+        },  # VRT uses CSV, which all types are OFTString.
+        extent=(1.0, 2.0, 100.0, 523.5),  # Min/Max from CSV
+        field_values={
+            'POINT_X': ['1.0', '5.0', '100.0'],
+            'POINT_Y': ['2.0', '23.0', '523.5'],
+            'NUM': ['5', '17', '23'],
+        },
+        fids=range(1, 4)
+    ),
+    TestDS(
+        'test_poly', nfeat=3, nfld=3, geom='POLYGON', gtype=3,
+        driver='ESRI Shapefile',
+        fields={'float': OFTReal, 'int': OFTInteger, 'str': OFTString},
+        extent=(-1.01513, -0.558245, 0.161876, 0.839637),  # Got extent from QGIS
+        srs_wkt=wgs_84_wkt,
     )
+)
 
 bad_ds = (TestDS('foo'),)
 
 
-@skipUnless(HAS_GDAL, "GDAL is required")
-class DataSourceTest(unittest.TestCase):
+class DataSourceTest(SimpleTestCase):
 
     def test01_valid_shp(self):
         "Testing valid SHP Data Source files."
@@ -80,8 +78,12 @@ class DataSourceTest(unittest.TestCase):
             self.assertEqual(source.driver, str(ds.driver))
 
             # Making sure indexing works
-            with self.assertRaises(OGRIndexError):
-                ds[len(ds)]
+            msg = 'Index out of range when accessing layers in a datasource: %s.'
+            with self.assertRaisesMessage(IndexError, msg % len(ds)):
+                ds.__getitem__(len(ds))
+
+            with self.assertRaisesMessage(IndexError, 'Invalid OGR layer name given: invalid.'):
+                ds.__getitem__('invalid')
 
     def test02_invalid_shp(self):
         "Testing invalid SHP files for the Data Source."
@@ -104,17 +106,11 @@ class DataSourceTest(unittest.TestCase):
                 self.assertEqual(source.nfld, len(layer.fields))
 
                 # Testing the layer's extent (an Envelope), and its properties
-                if source.driver == 'VRT' and (GDAL_VERSION >= (1, 7, 0) and GDAL_VERSION < (1, 7, 3)):
-                    # There's a known GDAL regression with retrieving the extent
-                    # of a VRT layer in versions 1.7.0-1.7.2:
-                    #  http://trac.osgeo.org/gdal/ticket/3783
-                    pass
-                else:
-                    self.assertIsInstance(layer.extent, Envelope)
-                    self.assertAlmostEqual(source.extent[0], layer.extent.min_x, 5)
-                    self.assertAlmostEqual(source.extent[1], layer.extent.min_y, 5)
-                    self.assertAlmostEqual(source.extent[2], layer.extent.max_x, 5)
-                    self.assertAlmostEqual(source.extent[3], layer.extent.max_y, 5)
+                self.assertIsInstance(layer.extent, Envelope)
+                self.assertAlmostEqual(source.extent[0], layer.extent.min_x, 5)
+                self.assertAlmostEqual(source.extent[1], layer.extent.min_y, 5)
+                self.assertAlmostEqual(source.extent[2], layer.extent.max_x, 5)
+                self.assertAlmostEqual(source.extent[3], layer.extent.max_y, 5)
 
                 # Now checking the field names.
                 flds = layer.fields
@@ -122,17 +118,15 @@ class DataSourceTest(unittest.TestCase):
                     self.assertIn(f, source.fields)
 
                 # Negative FIDs are not allowed.
-                with self.assertRaises(OGRIndexError):
+                with self.assertRaisesMessage(IndexError, 'Negative indices are not allowed on OGR Layers.'):
                     layer.__getitem__(-1)
-                with self.assertRaises(OGRIndexError):
+                with self.assertRaisesMessage(IndexError, 'Invalid feature id: 50000.'):
                     layer.__getitem__(50000)
 
                 if hasattr(source, 'field_values'):
-                    fld_names = source.field_values.keys()
-
                     # Testing `Layer.get_fields` (which uses Layer.__iter__)
-                    for fld_name in fld_names:
-                        self.assertEqual(source.field_values[fld_name], layer.get_fields(fld_name))
+                    for fld_name, fld_value in source.field_values.items():
+                        self.assertEqual(fld_value, layer.get_fields(fld_name))
 
                     # Testing `Layer.__getitem__`.
                     for i, fid in enumerate(source.fids):
@@ -140,8 +134,15 @@ class DataSourceTest(unittest.TestCase):
                         self.assertEqual(fid, feat.fid)
                         # Maybe this should be in the test below, but we might as well test
                         # the feature values here while in this loop.
-                        for fld_name in fld_names:
-                            self.assertEqual(source.field_values[fld_name][i], feat.get(fld_name))
+                        for fld_name, fld_value in source.field_values.items():
+                            self.assertEqual(fld_value[i], feat.get(fld_name))
+
+                        msg = 'Index out of range when accessing field in a feature: %s.'
+                        with self.assertRaisesMessage(IndexError, msg % len(feat)):
+                            feat.__getitem__(len(feat))
+
+                        with self.assertRaisesMessage(IndexError, 'Invalid OFT field name given: invalid.'):
+                            feat.__getitem__('invalid')
 
     def test03b_layer_slice(self):
         "Test indexing and slicing on Layers."
@@ -199,10 +200,11 @@ class DataSourceTest(unittest.TestCase):
                         # Making sure we get the proper OGR Field instance, using
                         # a string value index for the feature.
                         self.assertIsInstance(feat[k], v)
+                    self.assertIsInstance(feat.fields[0], str)
 
                     # Testing Feature.__iter__
                     for fld in feat:
-                        self.assertIn(fld.name, source.fields.keys())
+                        self.assertIn(fld.name, source.fields)
 
     def test05_geometries(self):
         "Testing Geometries from Data Source Features."
@@ -220,11 +222,7 @@ class DataSourceTest(unittest.TestCase):
 
                     # Making sure the SpatialReference is as expected.
                     if hasattr(source, 'srs_wkt'):
-                        self.assertEqual(
-                            source.srs_wkt,
-                            # Depending on lib versions, WGS_84 might be WGS_1984
-                            g.srs.wkt.replace('SPHEROID["WGS_84"', 'SPHEROID["WGS_1984"')
-                        )
+                        self.assertIsNotNone(re.match(wgs_84_wkt_regex, g.srs.wkt))
 
     def test06_spatial_filter(self):
         "Testing the Layer.spatial_filter property."
