@@ -1,25 +1,25 @@
-from __future__ import unicode_literals
-
+import json
+import math
 import re
 from decimal import Decimal
 
 from django.contrib.gis.db.models import functions
-from django.contrib.gis.geos import LineString, Point, Polygon, fromstr
+from django.contrib.gis.geos import (
+    GEOSGeometry, LineString, Point, Polygon, fromstr,
+)
 from django.contrib.gis.measure import Area
-from django.db import connection
+from django.db import NotSupportedError, connection
 from django.db.models import Sum
-from django.test import TestCase, skipUnlessDBFeature
-from django.utils import six
+from django.test import TestCase, ignore_warnings, skipUnlessDBFeature
+from django.utils.deprecation import RemovedInDjango30Warning
 
-from ..utils import mysql, oracle, postgis, spatialite
+from ..utils import FuncTestMixin, mysql, oracle, postgis, spatialite
 from .models import City, Country, CountryWebMercator, State, Track
 
 
-@skipUnlessDBFeature("gis_enabled")
-class GISFunctionsTests(TestCase):
+class GISFunctionsTests(FuncTestMixin, TestCase):
     """
     Testing functions from django/contrib/gis/db/models/functions.py.
-    Several tests are taken and adapted from GeoQuerySetTest.
     Area/Distance/Length/Perimeter are tested in distapp/tests.
 
     Please keep the tests in function's alphabetic order.
@@ -28,8 +28,8 @@ class GISFunctionsTests(TestCase):
 
     def test_asgeojson(self):
         # Only PostGIS and SpatiaLite support GeoJSON.
-        if not connection.ops.geojson:
-            with self.assertRaises(NotImplementedError):
+        if not connection.features.has_AsGeoJSON_function:
+            with self.assertRaises(NotSupportedError):
                 list(Country.objects.annotate(json=functions.AsGeoJSON('mpoly')))
             return
 
@@ -46,11 +46,12 @@ class GISFunctionsTests(TestCase):
             '{"type":"Point","crs":{"type":"name","properties":{"name":"EPSG:4326"}},'
             '"bbox":[-87.65018,41.85039,-87.65018,41.85039],"coordinates":[-87.65018,41.85039]}'
         )
-        if spatialite:
-            victoria_json = (
-                '{"type":"Point","bbox":[-123.305196,48.462611,-123.305196,48.462611],'
-                '"coordinates":[-123.305196,48.462611]}'
-            )
+        # MySQL ignores the crs option.
+        if mysql:
+            houston_json = json.loads(houston_json)
+            del houston_json['crs']
+            chicago_json = json.loads(chicago_json)
+            del chicago_json['crs']
 
         # Precision argument should only be an integer
         with self.assertRaises(TypeError):
@@ -59,7 +60,7 @@ class GISFunctionsTests(TestCase):
         # Reference queries and values.
         # SELECT ST_AsGeoJson("geoapp_city"."point", 8, 0)
         # FROM "geoapp_city" WHERE "geoapp_city"."name" = 'Pueblo';
-        self.assertEqual(
+        self.assertJSONEqual(
             pueblo_json,
             City.objects.annotate(geojson=functions.AsGeoJSON('point')).get(name='Pueblo').geojson
         )
@@ -67,15 +68,15 @@ class GISFunctionsTests(TestCase):
         # SELECT ST_AsGeoJson("geoapp_city"."point", 8, 2) FROM "geoapp_city"
         # WHERE "geoapp_city"."name" = 'Houston';
         # This time we want to include the CRS by using the `crs` keyword.
-        self.assertEqual(
+        self.assertJSONEqual(
+            City.objects.annotate(json=functions.AsGeoJSON('point', crs=True)).get(name='Houston').json,
             houston_json,
-            City.objects.annotate(json=functions.AsGeoJSON('point', crs=True)).get(name='Houston').json
         )
 
         # SELECT ST_AsGeoJson("geoapp_city"."point", 8, 1) FROM "geoapp_city"
         # WHERE "geoapp_city"."name" = 'Houston';
         # This time we include the bounding box by using the `bbox` keyword.
-        self.assertEqual(
+        self.assertJSONEqual(
             victoria_json,
             City.objects.annotate(
                 geojson=functions.AsGeoJSON('point', bbox=True)
@@ -85,11 +86,11 @@ class GISFunctionsTests(TestCase):
         # SELECT ST_AsGeoJson("geoapp_city"."point", 5, 3) FROM "geoapp_city"
         # WHERE "geoapp_city"."name" = 'Chicago';
         # Finally, we set every available keyword.
-        self.assertEqual(
-            chicago_json,
+        self.assertJSONEqual(
             City.objects.annotate(
                 geojson=functions.AsGeoJSON('point', bbox=True, crs=True, precision=5)
-            ).get(name='Chicago').geojson
+            ).get(name='Chicago').geojson,
+            chicago_json,
         )
 
     @skipUnlessDBFeature("has_AsGML_function")
@@ -104,7 +105,7 @@ class GISFunctionsTests(TestCase):
         if oracle:
             # No precision parameter for Oracle :-/
             gml_regex = re.compile(
-                r'^<gml:Point srsName="SDO:4326" xmlns:gml="http://www.opengis.net/gml">'
+                r'^<gml:Point srsName="EPSG:4326" xmlns:gml="http://www.opengis.net/gml">'
                 r'<gml:coordinates decimal="\." cs="," ts=" ">-104.60925\d+,38.25500\d+ '
                 r'</gml:coordinates></gml:Point>'
             )
@@ -113,14 +114,11 @@ class GISFunctionsTests(TestCase):
                 r'^<gml:Point srsName="EPSG:4326"><gml:coordinates>'
                 r'-104\.60925\d+,38\.255001</gml:coordinates></gml:Point>'
             )
-
         self.assertTrue(gml_regex.match(ptown.gml))
-
-        if postgis:
-            self.assertIn(
-                '<gml:pos srsDimension="2">',
-                City.objects.annotate(gml=functions.AsGML('point', version=3)).get(name='Pueblo').gml
-            )
+        self.assertIn(
+            '<gml:pos srsDimension="2">',
+            City.objects.annotate(gml=functions.AsGML('point', version=3)).get(name='Pueblo').gml
+        )
 
     @skipUnlessDBFeature("has_AsKML_function")
     def test_askml(self):
@@ -145,18 +143,40 @@ class GISFunctionsTests(TestCase):
         self.assertEqual(svg1, City.objects.annotate(svg=functions.AsSVG('point')).get(name='Pueblo').svg)
         self.assertEqual(svg2, City.objects.annotate(svg=functions.AsSVG('point', relative=5)).get(name='Pueblo').svg)
 
+    @skipUnlessDBFeature("has_Azimuth_function")
+    def test_azimuth(self):
+        # Returns the azimuth in radians.
+        azimuth_expr = functions.Azimuth(Point(0, 0, srid=4326), Point(1, 1, srid=4326))
+        self.assertAlmostEqual(City.objects.annotate(azimuth=azimuth_expr).first().azimuth, math.pi / 4)
+        # Returns None if the two points are coincident.
+        azimuth_expr = functions.Azimuth(Point(0, 0, srid=4326), Point(0, 0, srid=4326))
+        self.assertIsNone(City.objects.annotate(azimuth=azimuth_expr).first().azimuth)
+
     @skipUnlessDBFeature("has_BoundingCircle_function")
     def test_bounding_circle(self):
-        # The weak precision in the assertions is because the BoundingCircle
-        # calculation changed on PostGIS 2.3.
+        def circle_num_points(num_seg):
+            # num_seg is the number of segments per quarter circle.
+            return (4 * num_seg) + 1
+
+        expected_areas = (169, 136) if postgis else (171, 126)
         qs = Country.objects.annotate(circle=functions.BoundingCircle('mpoly')).order_by('name')
-        self.assertAlmostEqual(qs[0].circle.area, 169, 0)
-        self.assertAlmostEqual(qs[1].circle.area, 136, 0)
+        self.assertAlmostEqual(qs[0].circle.area, expected_areas[0], 0)
+        self.assertAlmostEqual(qs[1].circle.area, expected_areas[1], 0)
+        if postgis:
+            # By default num_seg=48.
+            self.assertEqual(qs[0].circle.num_points, circle_num_points(48))
+            self.assertEqual(qs[1].circle.num_points, circle_num_points(48))
 
         qs = Country.objects.annotate(circle=functions.BoundingCircle('mpoly', num_seg=12)).order_by('name')
-        self.assertGreater(qs[0].circle.area, 168.4, 0)
-        self.assertLess(qs[0].circle.area, 169.5, 0)
-        self.assertAlmostEqual(qs[1].circle.area, 136, 0)
+        if postgis:
+            self.assertGreater(qs[0].circle.area, 168.4, 0)
+            self.assertLess(qs[0].circle.area, 169.5, 0)
+            self.assertAlmostEqual(qs[1].circle.area, 136, 0)
+            self.assertEqual(qs[0].circle.num_points, circle_num_points(12))
+            self.assertEqual(qs[1].circle.num_points, circle_num_points(12))
+        else:
+            self.assertAlmostEqual(qs[0].circle.area, expected_areas[0], 0)
+            self.assertAlmostEqual(qs[1].circle.area, expected_areas[1], 0)
 
     @skipUnlessDBFeature("has_Centroid_function")
     def test_centroid(self):
@@ -172,8 +192,8 @@ class GISFunctionsTests(TestCase):
     def test_difference(self):
         geom = Point(5, 23, srid=4326)
         qs = Country.objects.annotate(diff=functions.Difference('mpoly', geom))
-        # SpatiaLite and Oracle do something screwy with the Texas geometry.
-        if spatialite or oracle:
+        # Oracle does something screwy with the Texas geometry.
+        if oracle:
             qs = qs.exclude(name='Texas')
 
         for c in qs:
@@ -182,10 +202,10 @@ class GISFunctionsTests(TestCase):
     @skipUnlessDBFeature("has_Difference_function", "has_Transform_function")
     def test_difference_mixed_srid(self):
         """Testing with mixed SRID (Country has default 4326)."""
-        geom = Point(556597.4, 2632018.6, srid=3857)  # Spherical mercator
+        geom = Point(556597.4, 2632018.6, srid=3857)  # Spherical Mercator
         qs = Country.objects.annotate(difference=functions.Difference('mpoly', geom))
-        # SpatiaLite and Oracle do something screwy with the Texas geometry.
-        if spatialite or oracle:
+        # Oracle does something screwy with the Texas geometry.
+        if oracle:
             qs = qs.exclude(name='Texas')
         for c in qs:
             self.assertTrue(c.mpoly.difference(geom).equals(c.difference))
@@ -194,9 +214,24 @@ class GISFunctionsTests(TestCase):
     def test_envelope(self):
         countries = Country.objects.annotate(envelope=functions.Envelope('mpoly'))
         for country in countries:
-            self.assertIsInstance(country.envelope, Polygon)
+            self.assertTrue(country.envelope.equals(country.mpoly.envelope))
+
+    @skipUnlessDBFeature("has_ForcePolygonCW_function")
+    def test_force_polygon_cw(self):
+        rings = (
+            ((0, 0), (5, 0), (0, 5), (0, 0)),
+            ((1, 1), (1, 3), (3, 1), (1, 1)),
+        )
+        rhr_rings = (
+            ((0, 0), (0, 5), (5, 0), (0, 0)),
+            ((1, 1), (3, 1), (1, 3), (1, 1)),
+        )
+        State.objects.create(name='Foo', poly=Polygon(*rings))
+        st = State.objects.annotate(force_polygon_cw=functions.ForcePolygonCW('poly')).get(name='Foo')
+        self.assertEqual(rhr_rings, st.force_polygon_cw.coords)
 
     @skipUnlessDBFeature("has_ForceRHR_function")
+    @ignore_warnings(category=RemovedInDjango30Warning)
     def test_force_rhr(self):
         rings = (
             ((0, 0), (5, 0), (0, 5), (0, 0)),
@@ -218,7 +253,7 @@ class GISFunctionsTests(TestCase):
         ref_hash = '9vk1mfq8jx0c8e0386z6'
         h1 = City.objects.annotate(geohash=functions.GeoHash('point')).get(name='Houston')
         h2 = City.objects.annotate(geohash=functions.GeoHash('point', precision=5)).get(name='Houston')
-        self.assertEqual(ref_hash, h1.geohash)
+        self.assertEqual(ref_hash, h1.geohash[:len(ref_hash)])
         self.assertEqual(ref_hash[:5], h2.geohash)
 
     @skipUnlessDBFeature("has_Intersection_function")
@@ -226,7 +261,7 @@ class GISFunctionsTests(TestCase):
         geom = Point(5, 23, srid=4326)
         qs = Country.objects.annotate(inter=functions.Intersection('mpoly', geom))
         for c in qs:
-            if spatialite or (mysql and not connection.ops.uses_invalid_empty_geometry_collection) or oracle:
+            if spatialite or (mysql and not connection.features.supports_empty_geometry_collection) or oracle:
                 # When the intersection is empty, some databases return None.
                 expected = None
             else:
@@ -248,7 +283,7 @@ class GISFunctionsTests(TestCase):
     def test_area_with_regular_aggregate(self):
         # Create projected country objects, for this test to work on all backends.
         for c in Country.objects.all():
-            CountryWebMercator.objects.create(name=c.name, mpoly=c.mpoly)
+            CountryWebMercator.objects.create(name=c.name, mpoly=c.mpoly.transform(3857, clone=True))
         # Test in projected coordinate system
         qs = CountryWebMercator.objects.annotate(area_sum=Sum(functions.Area('mpoly')))
         # Some backends (e.g. Oracle) cannot group by multipolygon values, so
@@ -260,13 +295,31 @@ class GISFunctionsTests(TestCase):
                 result = result.sq_m
             self.assertAlmostEqual((result - c.mpoly.area) / c.mpoly.area, 0)
 
+    @skipUnlessDBFeature("has_Area_function")
+    def test_area_lookups(self):
+        # Create projected countries so the test works on all backends.
+        CountryWebMercator.objects.bulk_create(
+            CountryWebMercator(name=c.name, mpoly=c.mpoly.transform(3857, clone=True))
+            for c in Country.objects.all()
+        )
+        qs = CountryWebMercator.objects.annotate(area=functions.Area('mpoly'))
+        self.assertEqual(qs.get(area__lt=Area(sq_km=500000)), CountryWebMercator.objects.get(name='New Zealand'))
+
+        with self.assertRaisesMessage(ValueError, 'AreaField only accepts Area measurement objects.'):
+            qs.get(area__lt=500000)
+
+    @skipUnlessDBFeature("has_LineLocatePoint_function")
+    def test_line_locate_point(self):
+        pos_expr = functions.LineLocatePoint(LineString((0, 0), (0, 3), srid=4326), Point(0, 1, srid=4326))
+        self.assertAlmostEqual(State.objects.annotate(pos=pos_expr).first().pos, 0.3333333)
+
     @skipUnlessDBFeature("has_MakeValid_function")
     def test_make_valid(self):
         invalid_geom = fromstr('POLYGON((0 0, 0 1, 1 1, 1 0, 1 1, 1 0, 0 0))')
         State.objects.create(name='invalid', poly=invalid_geom)
         invalid = State.objects.filter(name='invalid').annotate(repaired=functions.MakeValid('poly')).first()
         self.assertIs(invalid.repaired.valid, True)
-        self.assertEqual(invalid.repaired, fromstr('POLYGON((0 0, 0 1, 1 1, 1 0, 0 0))'))
+        self.assertEqual(invalid.repaired, fromstr('POLYGON((0 0, 0 1, 1 1, 1 0, 0 0))', srid=invalid.poly.srid))
 
     @skipUnlessDBFeature("has_MemSize_function")
     def test_memsize(self):
@@ -294,17 +347,17 @@ class GISFunctionsTests(TestCase):
         Track.objects.create(name='Foo', line=LineString(coords))
         qs = Track.objects.annotate(num_points=functions.NumPoints('line'))
         self.assertEqual(qs.first().num_points, 2)
-        if spatialite or mysql:
-            # SpatiaLite and MySQL can only count points on LineStrings
+        mpoly_qs = Country.objects.annotate(num_points=functions.NumPoints('mpoly'))
+        if not connection.features.supports_num_points_poly:
+            for c in mpoly_qs:
+                self.assertIsNone(c.num_points)
             return
 
-        for c in Country.objects.annotate(num_points=functions.NumPoints('mpoly')):
+        for c in mpoly_qs:
             self.assertEqual(c.mpoly.num_points, c.num_points)
 
-        if not oracle:
-            # Oracle cannot count vertices in Point geometries.
-            for c in City.objects.annotate(num_points=functions.NumPoints('point')):
-                self.assertEqual(1, c.num_points)
+        for c in City.objects.annotate(num_points=functions.NumPoints('point')):
+            self.assertEqual(c.num_points, 1)
 
     @skipUnlessDBFeature("has_PointOnSurface_function")
     def test_point_on_surface(self):
@@ -312,15 +365,17 @@ class GISFunctionsTests(TestCase):
         if oracle:
             # SELECT SDO_UTIL.TO_WKTGEOMETRY(SDO_GEOM.SDO_POINTONSURFACE(GEOAPP_COUNTRY.MPOLY, 0.05))
             # FROM GEOAPP_COUNTRY;
-            ref = {'New Zealand': fromstr('POINT (174.616364 -36.100861)', srid=4326),
-                   'Texas': fromstr('POINT (-103.002434 36.500397)', srid=4326),
-                   }
+            ref = {
+                'New Zealand': fromstr('POINT (174.616364 -36.100861)', srid=4326),
+                'Texas': fromstr('POINT (-103.002434 36.500397)', srid=4326),
+            }
         else:
             # Using GEOSGeometry to compute the reference point on surface values
             # -- since PostGIS also uses GEOS these should be the same.
-            ref = {'New Zealand': Country.objects.get(name='New Zealand').mpoly.point_on_surface,
-                   'Texas': Country.objects.get(name='Texas').mpoly.point_on_surface
-                   }
+            ref = {
+                'New Zealand': Country.objects.get(name='New Zealand').mpoly.point_on_surface,
+                'Texas': Country.objects.get(name='Texas').mpoly.point_on_surface
+            }
 
         qs = Country.objects.annotate(point_on_surface=functions.PointOnSurface('mpoly'))
         for country in qs:
@@ -356,7 +411,7 @@ class GISFunctionsTests(TestCase):
         for bad_args in ((), range(3), range(5)):
             with self.assertRaises(ValueError):
                 Country.objects.annotate(snap=functions.SnapToGrid('mpoly', *bad_args))
-        for bad_args in (('1.0',), (1.0, None), tuple(map(six.text_type, range(4)))):
+        for bad_args in (('1.0',), (1.0, None), tuple(map(str, range(4)))):
             with self.assertRaises(TypeError):
                 Country.objects.annotate(snap=functions.SnapToGrid('mpoly', *bad_args))
 
@@ -454,21 +509,14 @@ class GISFunctionsTests(TestCase):
         "has_Difference_function", "has_Intersection_function",
         "has_SymDifference_function", "has_Union_function")
     def test_diff_intersection_union(self):
-        "Testing the `difference`, `intersection`, `sym_difference`, and `union` GeoQuerySet methods."
         geom = Point(5, 23, srid=4326)
         qs = Country.objects.all().annotate(
             difference=functions.Difference('mpoly', geom),
             sym_difference=functions.SymDifference('mpoly', geom),
             union=functions.Union('mpoly', geom),
+            intersection=functions.Intersection('mpoly', geom),
         )
 
-        # For some reason SpatiaLite does something screwy with the Texas geometry here.
-        # Also, it doesn't like the null intersection.
-        if spatialite:
-            qs = qs.exclude(name='Texas')
-        else:
-            qs = qs.annotate(intersection=functions.Intersection('mpoly', geom))
-
         if oracle:
             # Should be able to execute the queries; however, they won't be the same
             # as GEOS (because Oracle doesn't use GEOS internally like PostGIS or
@@ -483,10 +531,48 @@ class GISFunctionsTests(TestCase):
 
     @skipUnlessDBFeature("has_Union_function")
     def test_union(self):
+        """Union with all combinations of geometries/geometry fields."""
         geom = Point(-95.363151, 29.763374, srid=4326)
-        ptown = City.objects.annotate(union=functions.Union('point', geom)).get(name='Dallas')
-        tol = 0.00001
-        # Undefined ordering
-        expected1 = fromstr('MULTIPOINT(-96.801611 32.782057,-95.363151 29.763374)', srid=4326)
-        expected2 = fromstr('MULTIPOINT(-95.363151 29.763374,-96.801611 32.782057)', srid=4326)
-        self.assertTrue(expected1.equals_exact(ptown.union, tol) or expected2.equals_exact(ptown.union, tol))
+
+        union = City.objects.annotate(union=functions.Union('point', geom)).get(name='Dallas').union
+        expected = fromstr('MULTIPOINT(-96.801611 32.782057,-95.363151 29.763374)', srid=4326)
+        self.assertTrue(expected.equals(union))
+
+        union = City.objects.annotate(union=functions.Union(geom, 'point')).get(name='Dallas').union
+        self.assertTrue(expected.equals(union))
+
+        union = City.objects.annotate(union=functions.Union('point', 'point')).get(name='Dallas').union
+        expected = GEOSGeometry('POINT(-96.801611 32.782057)', srid=4326)
+        self.assertTrue(expected.equals(union))
+
+        union = City.objects.annotate(union=functions.Union(geom, geom)).get(name='Dallas').union
+        self.assertTrue(geom.equals(union))
+
+    @skipUnlessDBFeature("has_Union_function", "has_Transform_function")
+    def test_union_mixed_srid(self):
+        """The result SRID depends on the order of parameters."""
+        geom = Point(61.42915, 55.15402, srid=4326)
+        geom_3857 = geom.transform(3857, clone=True)
+        tol = 0.001
+
+        for city in City.objects.annotate(union=functions.Union('point', geom_3857)):
+            expected = city.point | geom
+            self.assertTrue(city.union.equals_exact(expected, tol))
+            self.assertEqual(city.union.srid, 4326)
+
+        for city in City.objects.annotate(union=functions.Union(geom_3857, 'point')):
+            expected = geom_3857 | city.point.transform(3857, clone=True)
+            self.assertTrue(expected.equals_exact(city.union, tol))
+            self.assertEqual(city.union.srid, 3857)
+
+    def test_argument_validation(self):
+        with self.assertRaisesMessage(ValueError, 'SRID is required for all geometries.'):
+            City.objects.annotate(geo=functions.GeoFunc(Point(1, 1)))
+
+        msg = 'GeoFunc function requires a GeometryField in position 1, got CharField.'
+        with self.assertRaisesMessage(TypeError, msg):
+            City.objects.annotate(geo=functions.GeoFunc('name'))
+
+        msg = 'GeoFunc function requires a geometric argument in position 1.'
+        with self.assertRaisesMessage(TypeError, msg):
+            City.objects.annotate(union=functions.GeoFunc(1, 'point')).get(name='Dallas')
