@@ -1,4 +1,4 @@
-from __future__ import unicode_literals
+from http import HTTPStatus
 
 from django.core.exceptions import SuspiciousOperation
 from django.db import connection, transaction
@@ -10,6 +10,15 @@ def regular(request):
     return HttpResponse(b"regular content")
 
 
+def no_response(request):
+    pass
+
+
+class NoResponse:
+    def __call__(self, request):
+        pass
+
+
 def streaming(request):
     return StreamingHttpResponse([b"streaming", b" ", b"content"])
 
@@ -31,3 +40,7 @@ def suspicious(request):
 def malformed_post(request):
     request.POST
     return HttpResponse()
+
+
+def httpstatus_enum(request):
+    return HttpResponse(status=HTTPStatus.OK)
