@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import datetime
 from decimal import Decimal
 
@@ -9,7 +7,7 @@ from django.test import SimpleTestCase, modify_settings, override_settings
 from django.utils import translation
 from django.utils.html import escape
 from django.utils.timezone import get_fixed_timezone, utc
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 # Mock out datetime in some tests so they don't fail occasionally when they
 # run too slow. Use a fixed datetime for datetime.now(). DST change in
@@ -100,12 +98,13 @@ class HumanizeTests(SimpleTestCase):
         test_list = (
             '100', '1000000', '1200000', '1290000', '1000000000', '2000000000',
             '6000000000000', '1300000000000000', '3500000000000000000000',
-            '8100000000000000000000000000000000', None,
+            '8100000000000000000000000000000000', None, ('1' + '0' * 100),
+            ('1' + '0' * 104),
         )
         result_list = (
             '100', '1.0 million', '1.2 million', '1.3 million', '1.0 billion',
             '2.0 billion', '6.0 trillion', '1.3 quadrillion', '3.5 sextillion',
-            '8.1 decillion', None,
+            '8.1 decillion', None, '1.0 googol', ('1' + '0' * 104),
         )
         with translation.override('en'):
             self.humanize_tester(test_list, result_list, 'intword')
@@ -185,7 +184,9 @@ class HumanizeTests(SimpleTestCase):
             def utcoffset(self, dt):
                 return None
         test_list = [
+            'test',
             now,
+            now - datetime.timedelta(microseconds=1),
             now - datetime.timedelta(seconds=1),
             now - datetime.timedelta(seconds=30),
             now - datetime.timedelta(minutes=1, seconds=30),
@@ -207,6 +208,8 @@ class HumanizeTests(SimpleTestCase):
             now.replace(tzinfo=utc),
         ]
         result_list = [
+            'test',
+            'now',
             'now',
             'a second ago',
             '30\xa0seconds ago',
@@ -291,3 +294,47 @@ class HumanizeTests(SimpleTestCase):
                 self.assertEqual(expected_natural_time, natural_time)
         finally:
             humanize.datetime = orig_humanize_datetime
+
+    def test_inflection_for_timedelta(self):
+        """
+        Translation of '%d day'/'%d month'/… may differ depending on the context
+        of the string it is inserted in.
+        """
+        test_list = [
+            # "%(delta)s ago" translations
+            now - datetime.timedelta(days=1),
+            now - datetime.timedelta(days=2),
+            now - datetime.timedelta(days=30),
+            now - datetime.timedelta(days=60),
+            now - datetime.timedelta(days=500),
+            now - datetime.timedelta(days=865),
+            # "%(delta)s from now" translations
+            now + datetime.timedelta(days=1),
+            now + datetime.timedelta(days=2),
+            now + datetime.timedelta(days=30),
+            now + datetime.timedelta(days=60),
+            now + datetime.timedelta(days=500),
+            now + datetime.timedelta(days=865),
+        ]
+        result_list = [
+            'před 1\xa0dnem',
+            'před 2\xa0dny',
+            'před 1\xa0měsícem',
+            'před 2\xa0měsíci',
+            'před 1\xa0rokem, 4\xa0měsíci',
+            'před 2\xa0lety, 4\xa0měsíci',
+            'za 1\xa0den',
+            'za 2\xa0dny',
+            'za 1\xa0měsíc',
+            'za 2\xa0měsíce',
+            'za 1\xa0rok, 4\xa0měsíce',
+            'za 2\xa0roky, 4\xa0měsíce',
+        ]
+
+        orig_humanize_datetime, humanize.datetime = humanize.datetime, MockDateTime
+        try:
+            # Choose a language with different naturaltime-past/naturaltime-future translations
+            with translation.override('cs'), self.settings(USE_L10N=True):
+                self.humanize_tester(test_list, result_list, 'naturaltime')
+        finally:
+            humanize.datetime = orig_humanize_datetime
