@@ -1,4 +1,4 @@
 # Sample project used by test_extraction.CustomLayoutExtractionTests
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 string = _("This is a project-level string")
