@@ -1,15 +1,11 @@
-from __future__ import unicode_literals
-
-from unittest import skipUnless
+from unittest import mock, skipUnless
 
 from django.db import connection
 from django.db.models import Index
 from django.db.utils import DatabaseError
-from django.test import TransactionTestCase, mock, skipUnlessDBFeature
-from django.test.utils import ignore_warnings
-from django.utils.deprecation import RemovedInDjango21Warning
+from django.test import TransactionTestCase, skipUnlessDBFeature
 
-from .models import Article, ArticleReporter, City, District, Reporter
+from .models import Article, ArticleReporter, City, Comment, District, Reporter
 
 
 class IntrospectionTests(TransactionTestCase):
@@ -48,9 +44,12 @@ class IntrospectionTests(TransactionTestCase):
                     self.fail("The test user has no CREATE VIEW privileges")
                 else:
                     raise
-
-        self.assertIn('introspection_article_view', connection.introspection.table_names(include_views=True))
-        self.assertNotIn('introspection_article_view', connection.introspection.table_names())
+        try:
+            self.assertIn('introspection_article_view', connection.introspection.table_names(include_views=True))
+            self.assertNotIn('introspection_article_view', connection.introspection.table_names())
+        finally:
+            with connection.cursor() as cursor:
+                cursor.execute('DROP VIEW introspection_article_view')
 
     def test_unmanaged_through_model(self):
         tables = connection.introspection.django_table_names()
@@ -63,8 +62,9 @@ class IntrospectionTests(TransactionTestCase):
 
     def test_sequence_list(self):
         sequences = connection.introspection.sequence_list()
-        expected = {'table': Reporter._meta.db_table, 'column': 'id'}
-        self.assertIn(expected, sequences, 'Reporter sequence not found in sequence_list()')
+        reporter_seqs = [seq for seq in sequences if seq['table'] == Reporter._meta.db_table]
+        self.assertEqual(len(reporter_seqs), 1, 'Reporter sequence not found in sequence_list()')
+        self.assertEqual(reporter_seqs[0]['column'], 'id')
 
     def test_get_table_description_names(self):
         with connection.cursor() as cursor:
@@ -77,11 +77,16 @@ class IntrospectionTests(TransactionTestCase):
             desc = connection.introspection.get_table_description(cursor, Reporter._meta.db_table)
         self.assertEqual(
             [datatype(r[1], r) for r in desc],
-            ['AutoField' if connection.features.can_introspect_autofield else 'IntegerField',
-             'CharField', 'CharField', 'CharField',
-             'BigIntegerField' if connection.features.can_introspect_big_integer_field else 'IntegerField',
-             'BinaryField' if connection.features.can_introspect_binary_field else 'TextField',
-             'SmallIntegerField' if connection.features.can_introspect_small_integer_field else 'IntegerField']
+            [
+                'AutoField' if connection.features.can_introspect_autofield else 'IntegerField',
+                'CharField',
+                'CharField',
+                'CharField',
+                'BigIntegerField' if connection.features.can_introspect_big_integer_field else 'IntegerField',
+                'BinaryField' if connection.features.can_introspect_binary_field else 'TextField',
+                'SmallIntegerField' if connection.features.can_introspect_small_integer_field else 'IntegerField',
+                'DurationField' if connection.features.can_introspect_duration_field else 'BigIntegerField',
+            ]
         )
 
     def test_get_table_description_col_lengths(self):
@@ -92,21 +97,20 @@ class IntrospectionTests(TransactionTestCase):
             [30, 30, 254]
         )
 
-    @skipUnlessDBFeature('can_introspect_null')
     def test_get_table_description_nullable(self):
         with connection.cursor() as cursor:
             desc = connection.introspection.get_table_description(cursor, Reporter._meta.db_table)
         nullable_by_backend = connection.features.interprets_empty_strings_as_nulls
         self.assertEqual(
             [r[6] for r in desc],
-            [False, nullable_by_backend, nullable_by_backend, nullable_by_backend, True, True, False]
+            [False, nullable_by_backend, nullable_by_backend, nullable_by_backend, True, True, False, False]
         )
 
     @skipUnlessDBFeature('can_introspect_autofield')
     def test_bigautofield(self):
         with connection.cursor() as cursor:
             desc = connection.introspection.get_table_description(cursor, City._meta.db_table)
-        self.assertIn('BigAutoField', [datatype(r[1], r) for r in desc])
+        self.assertIn(connection.features.introspected_big_auto_field_type, [datatype(r[1], r) for r in desc])
 
     # Regression test for #9991 - 'real' types in postgres
     @skipUnlessDBFeature('has_real_datatype')
@@ -151,7 +155,7 @@ class IntrospectionTests(TransactionTestCase):
         ]
         for statement in create_table_statements:
             with connection.cursor() as cursor:
-                cursor.fetchone = mock.Mock(return_value=[statement.format(Article._meta.db_table)])
+                cursor.fetchone = mock.Mock(return_value=[statement.format(Article._meta.db_table), 'table'])
                 relations = connection.introspection.get_relations(cursor, 'mocked_table')
             self.assertEqual(relations, {'art_id': ('id', Article._meta.db_table)})
 
@@ -159,10 +163,10 @@ class IntrospectionTests(TransactionTestCase):
     def test_get_key_columns(self):
         with connection.cursor() as cursor:
             key_columns = connection.introspection.get_key_columns(cursor, Article._meta.db_table)
-        self.assertEqual(
-            set(key_columns),
-            {('reporter_id', Reporter._meta.db_table, 'id'),
-             ('response_to_id', Article._meta.db_table, 'id')})
+        self.assertEqual(set(key_columns), {
+            ('reporter_id', Reporter._meta.db_table, 'id'),
+            ('response_to_id', Article._meta.db_table, 'id'),
+        })
 
     def test_get_primary_key_column(self):
         with connection.cursor() as cursor:
@@ -171,28 +175,12 @@ class IntrospectionTests(TransactionTestCase):
         self.assertEqual(primary_key_column, 'id')
         self.assertEqual(pk_fk_column, 'city_id')
 
-    @ignore_warnings(category=RemovedInDjango21Warning)
-    def test_get_indexes(self):
-        with connection.cursor() as cursor:
-            indexes = connection.introspection.get_indexes(cursor, Article._meta.db_table)
-        self.assertEqual(indexes['reporter_id'], {'unique': False, 'primary_key': False})
-
-    @ignore_warnings(category=RemovedInDjango21Warning)
-    def test_get_indexes_multicol(self):
-        """
-        Multicolumn indexes are not included in the introspection results.
-        """
-        with connection.cursor() as cursor:
-            indexes = connection.introspection.get_indexes(cursor, Reporter._meta.db_table)
-        self.assertNotIn('first_name', indexes)
-        self.assertIn('id', indexes)
-
     def test_get_constraints_index_types(self):
         with connection.cursor() as cursor:
             constraints = connection.introspection.get_constraints(cursor, Article._meta.db_table)
         index = {}
         index2 = {}
-        for key, val in constraints.items():
+        for val in constraints.values():
             if val['columns'] == ['headline', 'pub_date']:
                 index = val
             if val['columns'] == ['headline', 'response_to_id', 'pub_date', 'reporter_id']:
@@ -214,13 +202,70 @@ class IntrospectionTests(TransactionTestCase):
             ['response_to_id'],
             ['headline', 'response_to_id', 'pub_date', 'reporter_id'],
         ]
-        for key, val in constraints.items():
+        for val in constraints.values():
             if val['index'] and not (val['primary_key'] or val['unique']):
                 self.assertIn(val['columns'], expected_columns)
                 self.assertEqual(val['orders'], ['ASC'] * len(val['columns']))
                 indexes_verified += 1
         self.assertEqual(indexes_verified, 4)
 
+    def test_get_constraints(self):
+        def assertDetails(details, cols, primary_key=False, unique=False, index=False, check=False, foreign_key=None):
+            # Different backends have different values for same constraints:
+            #               PRIMARY KEY     UNIQUE CONSTRAINT    UNIQUE INDEX
+            # MySQL      pk=1 uniq=1 idx=1  pk=0 uniq=1 idx=1  pk=0 uniq=1 idx=1
+            # PostgreSQL pk=1 uniq=1 idx=0  pk=0 uniq=1 idx=0  pk=0 uniq=1 idx=1
+            # SQLite     pk=1 uniq=0 idx=0  pk=0 uniq=1 idx=0  pk=0 uniq=1 idx=1
+            if details['primary_key']:
+                details['unique'] = True
+            if details['unique']:
+                details['index'] = False
+            self.assertEqual(details['columns'], cols)
+            self.assertEqual(details['primary_key'], primary_key)
+            self.assertEqual(details['unique'], unique)
+            self.assertEqual(details['index'], index)
+            self.assertEqual(details['check'], check)
+            self.assertEqual(details['foreign_key'], foreign_key)
+
+        with connection.cursor() as cursor:
+            constraints = connection.introspection.get_constraints(cursor, Comment._meta.db_table)
+        # Test custom constraints
+        custom_constraints = {
+            'article_email_pub_date_uniq',
+            'email_pub_date_idx',
+        }
+        if connection.features.supports_column_check_constraints:
+            custom_constraints.add('up_votes_gte_0_check')
+            assertDetails(constraints['up_votes_gte_0_check'], ['up_votes'], check=True)
+        assertDetails(constraints['article_email_pub_date_uniq'], ['article_id', 'email', 'pub_date'], unique=True)
+        assertDetails(constraints['email_pub_date_idx'], ['email', 'pub_date'], index=True)
+        # Test field constraints
+        field_constraints = set()
+        for name, details in constraints.items():
+            if name in custom_constraints:
+                continue
+            elif details['columns'] == ['up_votes'] and details['check']:
+                assertDetails(details, ['up_votes'], check=True)
+                field_constraints.add(name)
+            elif details['columns'] == ['ref'] and details['unique']:
+                assertDetails(details, ['ref'], unique=True)
+                field_constraints.add(name)
+            elif details['columns'] == ['article_id'] and details['index']:
+                assertDetails(details, ['article_id'], index=True)
+                field_constraints.add(name)
+            elif details['columns'] == ['id'] and details['primary_key']:
+                assertDetails(details, ['id'], primary_key=True, unique=True)
+                field_constraints.add(name)
+            elif details['columns'] == ['article_id'] and details['foreign_key']:
+                assertDetails(details, ['article_id'], foreign_key=('introspection_article', 'id'))
+                field_constraints.add(name)
+            elif details['check']:
+                # Some databases (e.g. Oracle) include additional check
+                # constraints.
+                field_constraints.add(name)
+        # All constraints are accounted for.
+        self.assertEqual(constraints.keys() ^ (custom_constraints | field_constraints), set())
+
 
 def datatype(dbtype, description):
     """Helper to convert a data type into a string."""
