@@ -1,9 +1,8 @@
-# -*- encoding: utf-8 -*-
-from __future__ import unicode_literals
+from unittest import mock
 
 from django.core.checks import Error
 from django.db import connections, models
-from django.test import SimpleTestCase, mock
+from django.test import SimpleTestCase
 from django.test.utils import isolate_apps
 
 
@@ -26,6 +25,4 @@ class BackendSpecificChecksTests(SimpleTestCase):
 
         field = Model._meta.get_field('field')
         with mock.patch.object(connections['default'].validation, 'check_field', return_value=[error]):
-            errors = field.check()
-
-        self.assertEqual(errors, [error])
+            self.assertEqual(field.check(), [error])
