@@ -1,5 +1,5 @@
 """
-20. Multiple many-to-many relationships between the same two tables
+Multiple many-to-many relationships between the same two tables
 
 In this example, an ``Article`` can have many "primary" ``Category`` objects
 and many "secondary" ``Category`` objects.
@@ -8,10 +8,8 @@ Set ``related_name`` to designate what the reverse relationship is called.
 """
 
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Category(models.Model):
     name = models.CharField(max_length=20)
 
@@ -22,7 +20,6 @@ class Category(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Article(models.Model):
     headline = models.CharField(max_length=50)
     pub_date = models.DateTimeField()
