@@ -1,11 +1,9 @@
 from django.contrib.auth import models as auth
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
 # No related name is needed here, since symmetrical relations are not
 # explicitly reversible.
-@python_2_unicode_compatible
 class SelfRefer(models.Model):
     name = models.CharField(max_length=10)
     references = models.ManyToManyField('self')
@@ -15,7 +13,6 @@ class SelfRefer(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Tag(models.Model):
     name = models.CharField(max_length=10)
 
@@ -24,7 +21,6 @@ class Tag(models.Model):
 
 
 # Regression for #11956 -- a many to many to the base class
-@python_2_unicode_compatible
 class TagCollection(Tag):
     tags = models.ManyToManyField(Tag, related_name='tag_collections')
 
@@ -34,7 +30,6 @@ class TagCollection(Tag):
 
 # A related_name is required on one of the ManyToManyField entries here because
 # they are both addressable as reverse relations from Tag.
-@python_2_unicode_compatible
 class Entry(models.Model):
     name = models.CharField(max_length=10)
     topics = models.ManyToManyField(Tag)
@@ -54,7 +49,6 @@ class SelfReferChildSibling(SelfRefer):
 
 
 # Many-to-Many relation between models, where one of the PK's isn't an Autofield
-@python_2_unicode_compatible
 class Line(models.Model):
     name = models.CharField(max_length=100)
 
@@ -79,12 +73,12 @@ class User(models.Model):
 class BadModelWithSplit(models.Model):
     name = models.CharField(max_length=1)
 
-    def split(self):
-        raise RuntimeError('split should not be called')
-
     class Meta:
         abstract = True
 
+    def split(self):
+        raise RuntimeError('split should not be called')
+
 
 class RegressionModelSplit(BadModelWithSplit):
     """
