@@ -1,8 +1,6 @@
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Part(models.Model):
     name = models.CharField(max_length=20)
 
@@ -13,7 +11,6 @@ class Part(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Car(models.Model):
     name = models.CharField(max_length=20)
     default_parts = models.ManyToManyField(Part)
@@ -30,7 +27,6 @@ class SportsCar(Car):
     price = models.IntegerField()
 
 
-@python_2_unicode_compatible
 class Person(models.Model):
     name = models.CharField(max_length=20)
     fans = models.ManyToManyField('self', related_name='idols', symmetrical=False)
