@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.test import TestCase
 
 from .models import Article, Car, Driver, Reporter
@@ -87,6 +85,11 @@ class ManyToOneNullTests(TestCase):
             ['<Article: Fourth>', '<Article: Second>', '<Article: Third>']
         )
 
+    def test_set_clear_non_bulk(self):
+        # 2 queries for clear(), 1 for add(), and 1 to select objects.
+        with self.assertNumQueries(4):
+            self.r.article_set.set([self.a], bulk=False, clear=True)
+
     def test_assign_clear_related_set(self):
         # Use descriptor assignment to allocate ForeignKey. Null is legal, so
         # existing members of the set that are not in the assignment set are
@@ -102,7 +105,7 @@ class ManyToOneNullTests(TestCase):
         )
 
     def test_assign_with_queryset(self):
-        # Ensure that querysets used in reverse FK assignments are pre-evaluated
+        # Querysets used in reverse FK assignments are pre-evaluated
         # so their value isn't affected by the clearing operation in
         # RelatedManager.set() (#19816).
         self.r2.article_set.set([self.a2, self.a3])
