@@ -6,7 +6,7 @@ class C1(models.Model):
 
 
 class C2(models.Model):
-    a1 = models.ForeignKey('lookuperror_a.A1')
+    a1 = models.ForeignKey('lookuperror_a.A1', models.CASCADE)
 
 
 class C3(models.Model):
