@@ -11,17 +11,13 @@ Model inheritance exists in two varieties:
 
 Both styles are demonstrated here.
 """
-from __future__ import unicode_literals
-
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 #
 # Abstract base classes
 #
 
 
-@python_2_unicode_compatible
 class CommonInfo(models.Model):
     name = models.CharField(max_length=50)
     age = models.PositiveIntegerField()
@@ -53,7 +49,6 @@ class Post(models.Model):
     title = models.CharField(max_length=50)
 
 
-@python_2_unicode_compatible
 class Attachment(models.Model):
     post = models.ForeignKey(
         Post,
@@ -82,7 +77,6 @@ class Link(Attachment):
 # Multi-table inheritance
 #
 
-@python_2_unicode_compatible
 class Chef(models.Model):
     name = models.CharField(max_length=50)
 
@@ -90,7 +84,6 @@ class Chef(models.Model):
         return "%s the chef" % self.name
 
 
-@python_2_unicode_compatible
 class Place(models.Model):
     name = models.CharField(max_length=50)
     address = models.CharField(max_length=80)
@@ -107,7 +100,6 @@ class Rating(models.Model):
         ordering = ['-rating']
 
 
-@python_2_unicode_compatible
 class Restaurant(Place, Rating):
     serves_hot_dogs = models.BooleanField(default=False)
     serves_pizza = models.BooleanField(default=False)
@@ -120,7 +112,6 @@ class Restaurant(Place, Rating):
         return "%s the restaurant" % self.name
 
 
-@python_2_unicode_compatible
 class ItalianRestaurant(Restaurant):
     serves_gnocchi = models.BooleanField(default=False)
 
@@ -128,7 +119,6 @@ class ItalianRestaurant(Restaurant):
         return "%s the italian restaurant" % self.name
 
 
-@python_2_unicode_compatible
 class Supplier(Place):
     customers = models.ManyToManyField(Restaurant, related_name='provider')
 
@@ -136,7 +126,6 @@ class Supplier(Place):
         return "%s the supplier" % self.name
 
 
-@python_2_unicode_compatible
 class ParkingLot(Place):
     # An explicit link to the parent (we can control the attribute name).
     parent = models.OneToOneField(Place, models.CASCADE, primary_key=True, parent_link=True)
@@ -167,10 +156,10 @@ class NamedURL(models.Model):
         abstract = True
 
 
-class Mixin(object):
+class Mixin:
     def __init__(self):
         self.other_attr = 1
-        super(Mixin, self).__init__()
+        super().__init__()
 
 
 class MixinModel(models.Model, Mixin):
@@ -189,6 +178,7 @@ class GrandParent(models.Model):
     first_name = models.CharField(max_length=80)
     last_name = models.CharField(max_length=80)
     email = models.EmailField(unique=True)
+    place = models.ForeignKey(Place, models.CASCADE, null=True, related_name='+')
 
     class Meta:
         unique_together = ('first_name', 'last_name')
