@@ -3,7 +3,6 @@ Regression tests for proper working of ForeignKey(null=True).
 """
 
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
 class SystemDetails(models.Model):
@@ -20,7 +19,6 @@ class Forum(models.Model):
     forum_name = models.CharField(max_length=32)
 
 
-@python_2_unicode_compatible
 class Post(models.Model):
     forum = models.ForeignKey(Forum, models.SET_NULL, null=True)
     title = models.CharField(max_length=32)
@@ -29,7 +27,6 @@ class Post(models.Model):
         return self.title
 
 
-@python_2_unicode_compatible
 class Comment(models.Model):
     post = models.ForeignKey(Post, models.SET_NULL, null=True)
     comment_text = models.CharField(max_length=250)
