@@ -14,7 +14,7 @@ undefined -- not random, just undefined.
 """
 
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
+from django.db.models.expressions import OrderBy
 
 
 class Author(models.Model):
@@ -24,7 +24,6 @@ class Author(models.Model):
         ordering = ('-pk',)
 
 
-@python_2_unicode_compatible
 class Article(models.Model):
     author = models.ForeignKey(Author, models.SET_NULL, null=True)
     second_author = models.ForeignKey(Author, models.SET_NULL, null=True, related_name='+')
@@ -32,7 +31,12 @@ class Article(models.Model):
     pub_date = models.DateTimeField()
 
     class Meta:
-        ordering = ('-pub_date', 'headline')
+        ordering = (
+            '-pub_date',
+            'headline',
+            models.F('author__name').asc(),
+            OrderBy(models.F('second_author__name')),
+        )
 
     def __str__(self):
         return self.headline
@@ -44,6 +48,12 @@ class OrderedByAuthorArticle(Article):
         ordering = ('author', 'second_author')
 
 
+class OrderedByFArticle(Article):
+    class Meta:
+        proxy = True
+        ordering = (models.F('author').asc(nulls_first=True), 'id')
+
+
 class Reference(models.Model):
     article = models.ForeignKey(OrderedByAuthorArticle, models.CASCADE)
 
