@@ -1,13 +1,12 @@
-from __future__ import unicode_literals
-
 from datetime import datetime
 from operator import attrgetter
 
-from django.db.models import DateTimeField, F, Max, OuterRef, Subquery
+from django.db.models import Count, DateTimeField, F, Max, OuterRef, Subquery
 from django.db.models.functions import Upper
 from django.test import TestCase
+from django.utils.deprecation import RemovedInDjango31Warning
 
-from .models import Article, Author, Reference
+from .models import Article, Author, OrderedByFArticle, Reference
 
 
 class OrderingTests(TestCase):
@@ -230,6 +229,14 @@ class OrderingTests(TestCase):
             attrgetter("headline")
         )
 
+    def test_no_reordering_after_slicing(self):
+        msg = 'Cannot reverse a query once a slice has been taken.'
+        qs = Article.objects.all()[0:2]
+        with self.assertRaisesMessage(TypeError, msg):
+            qs.reverse()
+        with self.assertRaisesMessage(TypeError, msg):
+            qs.last()
+
     def test_extra_ordering(self):
         """
         Ordering can be based on fields included from an 'extra' clause
@@ -387,3 +394,23 @@ class OrderingTests(TestCase):
         r1 = Reference.objects.create(article_id=self.a1.pk)
         r2 = Reference.objects.create(article_id=self.a2.pk)
         self.assertSequenceEqual(Reference.objects.all(), [r2, r1])
+
+    def test_default_ordering_by_f_expression(self):
+        """F expressions can be used in Meta.ordering."""
+        articles = OrderedByFArticle.objects.all()
+        articles.filter(headline='Article 2').update(author=self.author_2)
+        articles.filter(headline='Article 3').update(author=self.author_1)
+        self.assertQuerysetEqual(
+            articles, ['Article 1', 'Article 4', 'Article 3', 'Article 2'],
+            attrgetter('headline')
+        )
+
+    def test_deprecated_values_annotate(self):
+        msg = (
+            "Article QuerySet won't use Meta.ordering in Django 3.1. Add "
+            ".order_by('-pub_date', 'headline', OrderBy(F(author__name), "
+            "descending=False), OrderBy(F(second_author__name), "
+            "descending=False)) to retain the current query."
+        )
+        with self.assertRaisesMessage(RemovedInDjango31Warning, msg):
+            list(Article.objects.values('author').annotate(Count('headline')))
