@@ -5,10 +5,8 @@ This demonstrates the reverse lookup features of the database API.
 """
 
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class User(models.Model):
     name = models.CharField(max_length=200)
 
@@ -16,7 +14,6 @@ class User(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Poll(models.Model):
     question = models.CharField(max_length=200)
     creator = models.ForeignKey(User, models.CASCADE)
@@ -25,7 +22,6 @@ class Poll(models.Model):
         return self.question
 
 
-@python_2_unicode_compatible
 class Choice(models.Model):
     name = models.CharField(max_length=100)
     poll = models.ForeignKey(Poll, models.CASCADE, related_name="poll_choice")
