@@ -1,27 +1,26 @@
-from __future__ import unicode_literals
-
 from django.core.exceptions import FieldError
+from django.db.models import FilteredRelation
 from django.test import SimpleTestCase, TestCase
 
 from .models import (
-    AdvancedUserStat, Child1, Child2, Child3, Child4, Image, Parent1, Parent2,
-    Product, StatDetails, User, UserProfile, UserStat, UserStatResult,
+    AdvancedUserStat, Child1, Child2, Child3, Child4, Image, LinkedList,
+    Parent1, Parent2, Product, StatDetails, User, UserProfile, UserStat,
+    UserStatResult,
 )
 
 
 class ReverseSelectRelatedTestCase(TestCase):
-    def setUp(self):
+    @classmethod
+    def setUpTestData(cls):
         user = User.objects.create(username="test")
         UserProfile.objects.create(user=user, state="KS", city="Lawrence")
         results = UserStatResult.objects.create(results='first results')
-        userstat = UserStat.objects.create(user=user, posts=150,
-                                           results=results)
+        userstat = UserStat.objects.create(user=user, posts=150, results=results)
         StatDetails.objects.create(base_stats=userstat, comments=259)
 
         user2 = User.objects.create(username="bob")
         results2 = UserStatResult.objects.create(results='moar results')
-        advstat = AdvancedUserStat.objects.create(user=user2, posts=200, karma=5,
-                                                  results=results2)
+        advstat = AdvancedUserStat.objects.create(user=user2, posts=200, karma=5, results=results2)
         StatDetails.objects.create(base_stats=advstat, comments=250)
         p1 = Parent1(name1="Only Parent1")
         p1.save()
@@ -82,7 +81,7 @@ class ReverseSelectRelatedTestCase(TestCase):
             stat = UserStat.objects.select_related('user', 'advanceduserstat').get(posts=200)
             self.assertEqual(stat.advanceduserstat.posts, 200)
             self.assertEqual(stat.user.username, 'bob')
-        with self.assertNumQueries(1):
+        with self.assertNumQueries(0):
             self.assertEqual(stat.advanceduserstat.user.username, 'bob')
 
     def test_nullable_relation(self):
@@ -207,6 +206,13 @@ class ReverseSelectRelatedTestCase(TestCase):
             self.assertEqual(p.child1.name1, 'n1')
             self.assertEqual(p.child1.child4.name1, 'n1')
 
+    def test_self_relation(self):
+        item1 = LinkedList.objects.create(name='item1')
+        LinkedList.objects.create(name='item2', previous_item=item1)
+        with self.assertNumQueries(1):
+            item1_db = LinkedList.objects.select_related('next_item').get(name='item1')
+            self.assertEqual(item1_db.next_item.name, 'item2')
+
 
 class ReverseSelectRelatedValidationTests(SimpleTestCase):
     """
@@ -224,3 +230,8 @@ class ReverseSelectRelatedValidationTests(SimpleTestCase):
 
         with self.assertRaisesMessage(FieldError, self.non_relational_error % ('username', fields)):
             list(User.objects.select_related('username'))
+
+    def test_reverse_related_validation_with_filtered_relation(self):
+        fields = 'userprofile, userstat, relation'
+        with self.assertRaisesMessage(FieldError, self.invalid_error % ('foobar', fields)):
+            list(User.objects.annotate(relation=FilteredRelation('userprofile')).select_related('foobar'))
