@@ -1,9 +1,8 @@
-import errno
 import os
 from datetime import datetime, timedelta
 
 from django.conf import settings
-from django.contrib.staticfiles.storage import CachedStaticFilesStorage
+from django.contrib.staticfiles.storage import ManifestStaticFilesStorage
 from django.core.files import storage
 from django.utils import timezone
 
@@ -40,20 +39,19 @@ class PathNotImplementedStorage(storage.Storage):
     def listdir(self, path):
         path = self._path(path)
         directories, files = [], []
-        for entry in os.listdir(path):
-            if os.path.isdir(os.path.join(path, entry)):
-                directories.append(entry)
+        for entry in os.scandir(path):
+            if entry.is_dir():
+                directories.append(entry.name)
             else:
-                files.append(entry)
+                files.append(entry.name)
         return directories, files
 
     def delete(self, name):
         name = self._path(name)
         try:
             os.remove(name)
-        except OSError as e:
-            if e.errno != errno.ENOENT:
-                raise
+        except FileNotFoundError:
+            pass
 
     def path(self, name):
         raise NotImplementedError
@@ -72,18 +70,18 @@ class QueryStringStorage(storage.Storage):
         return path + '?a=b&c=d'
 
 
-class SimpleCachedStaticFilesStorage(CachedStaticFilesStorage):
+class SimpleStorage(ManifestStaticFilesStorage):
 
     def file_hash(self, name, content=None):
         return 'deploy12345'
 
 
-class ExtraPatternsCachedStaticFilesStorage(CachedStaticFilesStorage):
+class ExtraPatternsStorage(ManifestStaticFilesStorage):
     """
     A storage class to test pattern substitutions with more than one pattern
     entry. The added pattern rewrites strings like "url(...)" to JS_URL("...").
     """
-    patterns = tuple(CachedStaticFilesStorage.patterns) + (
+    patterns = tuple(ManifestStaticFilesStorage.patterns) + (
         (
             "*.js", (
                 (r"""(url\(['"]{0,1}\s*(.*?)["']{0,1}\))""", 'JS_URL("%s")'),
