@@ -1,8 +1,6 @@
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Entry(models.Model):
     title = models.CharField(max_length=200)
     updated = models.DateTimeField()
@@ -18,7 +16,6 @@ class Entry(models.Model):
         return "/blog/%s/" % self.pk
 
 
-@python_2_unicode_compatible
 class Article(models.Model):
     title = models.CharField(max_length=200)
     entry = models.ForeignKey(Entry, models.CASCADE)
