@@ -1,4 +1,5 @@
 from django.template import TemplateSyntaxError
+from django.template.defaulttags import IfNode
 from django.test import SimpleTestCase
 
 from ..utils import TestObj, setup
@@ -508,6 +509,12 @@ class IfTagTests(SimpleTestCase):
         with self.assertRaises(TemplateSyntaxError):
             self.engine.get_template('if-tag-error12')
 
+    @setup({'else-if-tag-error01': '{% if foo is bar %} yes {% else if foo is not bar %} no {% endif %}'})
+    def test_else_if_tag_error01(self):
+        error_message = 'Malformed template tag at line 1: "else if foo is not bar"'
+        with self.assertRaisesMessage(TemplateSyntaxError, error_message):
+            self.engine.get_template('else-if-tag-error01')
+
     @setup({'if-tag-shortcircuit01': '{% if x.is_true or x.is_bad %}yes{% else %}no{% endif %}'})
     def test_if_tag_shortcircuit01(self):
         """
@@ -527,9 +534,7 @@ class IfTagTests(SimpleTestCase):
 
     @setup({'if-tag-badarg01': '{% if x|default_if_none:y %}yes{% endif %}'})
     def test_if_tag_badarg01(self):
-        """
-        Non-existent args
-        """
+        """Nonexistent args"""
         output = self.engine.render_to_string('if-tag-badarg01')
         self.assertEqual(output, '')
 
@@ -595,3 +600,9 @@ class IfTagTests(SimpleTestCase):
     def test_if_is_not_both_variables_missing(self):
         output = self.engine.render_to_string('template', {})
         self.assertEqual(output, 'no')
+
+
+class IfNodeTests(SimpleTestCase):
+    def test_repr(self):
+        node = IfNode(conditions_nodelists=[])
+        self.assertEqual(repr(node), '<IfNode>')
