@@ -1,17 +1,11 @@
-# coding: utf-8
-
-from __future__ import unicode_literals
-
 import functools
 import os
 
 from django.template.engine import Engine
 from django.test.utils import override_settings
-from django.utils._os import upath
-from django.utils.encoding import python_2_unicode_compatible
 from django.utils.safestring import mark_safe
 
-ROOT = os.path.dirname(os.path.abspath(upath(__file__)))
+ROOT = os.path.dirname(os.path.abspath(__file__))
 TEMPLATE_DIR = os.path.join(ROOT, 'templates')
 
 
@@ -137,13 +131,17 @@ class SomeClass:
     def attribute_error_attribute(self):
         raise AttributeError
 
+    @property
+    def type_error_attribute(self):
+        raise TypeError
+
 
 class OtherClass:
     def method(self):
         return 'OtherClass.method'
 
 
-class TestObj(object):
+class TestObj:
     def is_true(self):
         return True
 
@@ -154,32 +152,29 @@ class TestObj(object):
         raise ShouldNotExecuteException()
 
 
-class SilentGetItemClass(object):
+class SilentGetItemClass:
     def __getitem__(self, key):
         raise SomeException
 
 
-class SilentAttrClass(object):
+class SilentAttrClass:
     def b(self):
         raise SomeException
     b = property(b)
 
 
-@python_2_unicode_compatible
 class UTF8Class:
-    "Class whose __str__ returns non-ASCII data on Python 2"
+    "Class whose __str__ returns non-ASCII data"
     def __str__(self):
         return 'ŠĐĆŽćžšđ'
 
 
-# These two classes are used to test auto-escaping of unicode output.
-@python_2_unicode_compatible
+# These two classes are used to test auto-escaping of string output.
 class UnsafeClass:
     def __str__(self):
         return 'you & me'
 
 
-@python_2_unicode_compatible
 class SafeClass:
     def __str__(self):
         return mark_safe('you &gt; me')
