@@ -1,9 +1,8 @@
 """
 Tests for django test runner
 """
-from __future__ import unicode_literals
-
 import unittest
+from unittest import mock
 
 from admin_scripts.tests import AdminScriptTestCase
 
@@ -11,13 +10,13 @@ from django import db
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.core.management import call_command
-from django.test import (
-    TestCase, TransactionTestCase, mock, skipUnlessDBFeature, testcases,
-)
-from django.test.runner import DiscoverRunner, dependency_ordered
+from django.core.management.base import SystemCheckError
+from django.test import TransactionTestCase, skipUnlessDBFeature, testcases
+from django.test.runner import DiscoverRunner
 from django.test.testcases import connections_support_transactions
+from django.test.utils import dependency_ordered
 
-from .models import Person
+from .models import B, Person, Through
 
 
 class DependencyOrderingTests(unittest.TestCase):
@@ -130,10 +129,11 @@ class DependencyOrderingTests(unittest.TestCase):
             dependency_ordered(raw, dependencies=dependencies)
 
 
-class MockTestRunner(object):
+class MockTestRunner:
     def __init__(self, *args, **kwargs):
         pass
 
+
 MockTestRunner.run_tests = mock.Mock(return_value=[])
 
 
@@ -146,11 +146,14 @@ class ManageCommandTests(unittest.TestCase):
 
     def test_bad_test_runner(self):
         with self.assertRaises(AttributeError):
-            call_command('test', 'sites', testrunner='test_runner.NonExistentRunner')
+            call_command('test', 'sites', testrunner='test_runner.NonexistentRunner')
 
 
-class CustomTestRunnerOptionsTests(AdminScriptTestCase):
-
+class CustomTestRunnerOptionsSettingsTests(AdminScriptTestCase):
+    """
+    Custom runners can add command line arguments. The runner is specified
+    through a settings file.
+    """
     def setUp(self):
         settings = {
             'TEST_RUNNER': '\'test_runner.runner.CustomOptionsTestRunner\'',
@@ -186,6 +189,43 @@ class CustomTestRunnerOptionsTests(AdminScriptTestCase):
         self.assertOutput(out, 'bar:foo:31337')
 
 
+class CustomTestRunnerOptionsCmdlineTests(AdminScriptTestCase):
+    """
+    Custom runners can add command line arguments when the runner is specified
+    using --testrunner.
+    """
+    def setUp(self):
+        self.write_settings('settings.py')
+
+    def tearDown(self):
+        self.remove_settings('settings.py')
+
+    def test_testrunner_option(self):
+        args = [
+            'test', '--testrunner', 'test_runner.runner.CustomOptionsTestRunner',
+            '--option_a=bar', '--option_b=foo', '--option_c=31337'
+        ]
+        out, err = self.run_django_admin(args, 'test_project.settings')
+        self.assertNoOutput(err)
+        self.assertOutput(out, 'bar:foo:31337')
+
+    def test_testrunner_equals(self):
+        args = [
+            'test', '--testrunner=test_runner.runner.CustomOptionsTestRunner',
+            '--option_a=bar', '--option_b=foo', '--option_c=31337'
+        ]
+        out, err = self.run_django_admin(args, 'test_project.settings')
+        self.assertNoOutput(err)
+        self.assertOutput(out, 'bar:foo:31337')
+
+    def test_no_testrunner(self):
+        args = ['test', '--testrunner']
+        out, err = self.run_django_admin(args, 'test_project.settings')
+        self.assertIn('usage', err)
+        self.assertNotIn('Traceback', err)
+        self.assertNoOutput(out)
+
+
 class Ticket17477RegressionTests(AdminScriptTestCase):
     def setUp(self):
         self.write_settings('settings.py')
@@ -200,14 +240,24 @@ class Ticket17477RegressionTests(AdminScriptTestCase):
         self.assertNoOutput(err)
 
 
-class Sqlite3InMemoryTestDbs(TestCase):
-
-    available_apps = []
+class SQLiteInMemoryTestDbs(TransactionTestCase):
+    available_apps = ['test_runner']
+    databases = {'default', 'other'}
 
     @unittest.skipUnless(all(db.connections[conn].vendor == 'sqlite' for conn in db.connections),
                          "This is an sqlite-specific issue")
     def test_transaction_support(self):
-        """Ticket #16329: sqlite3 in-memory test databases"""
+        # Assert connections mocking is appropriately applied by preventing
+        # any attempts at calling create_test_db on the global connection
+        # objects.
+        for connection in db.connections.all():
+            create_test_db = mock.patch.object(
+                connection.creation,
+                'create_test_db',
+                side_effect=AssertionError("Global connection object shouldn't be manipulated.")
+            )
+            create_test_db.start()
+            self.addCleanup(create_test_db.stop)
         for option_key, option_value in (
                 ('NAME', ':memory:'), ('TEST', {'NAME': ':memory:'})):
             tested_connections = db.ConnectionHandler({
@@ -220,25 +270,26 @@ class Sqlite3InMemoryTestDbs(TestCase):
                     option_key: option_value,
                 },
             })
-            with mock.patch('django.db.connections', new=tested_connections):
-                with mock.patch('django.test.testcases.connections', new=tested_connections):
-                    other = tested_connections['other']
-                    DiscoverRunner(verbosity=0).setup_databases()
-                    msg = ("DATABASES setting '%s' option set to sqlite3's ':memory:' value "
-                           "shouldn't interfere with transaction support detection." % option_key)
-                    # Transaction support should be properly initialized for the 'other' DB
-                    self.assertTrue(other.features.supports_transactions, msg)
-                    # And all the DBs should report that they support transactions
-                    self.assertTrue(connections_support_transactions(), msg)
+            with mock.patch('django.test.utils.connections', new=tested_connections):
+                other = tested_connections['other']
+                DiscoverRunner(verbosity=0).setup_databases()
+                msg = (
+                    "DATABASES setting '%s' option set to sqlite3's ':memory:' value "
+                    "shouldn't interfere with transaction support detection." % option_key
+                )
+                # Transaction support is properly initialized for the 'other' DB.
+                self.assertTrue(other.features.supports_transactions, msg)
+                # And all the DBs report that they support transactions.
+                self.assertTrue(connections_support_transactions(), msg)
 
 
 class DummyBackendTest(unittest.TestCase):
     def test_setup_databases(self):
         """
-        Test that setup_databases() doesn't fail with dummy database backend.
+        setup_databases() doesn't fail with dummy database backend.
         """
         tested_connections = db.ConnectionHandler({})
-        with mock.patch('django.test.runner.connections', new=tested_connections):
+        with mock.patch('django.test.utils.connections', new=tested_connections):
             runner_instance = DiscoverRunner(verbosity=0)
             old_config = runner_instance.setup_databases()
             runner_instance.teardown_databases(old_config)
@@ -247,7 +298,7 @@ class DummyBackendTest(unittest.TestCase):
 class AliasedDefaultTestSetupTest(unittest.TestCase):
     def test_setup_aliased_default_database(self):
         """
-        Test that setup_datebases() doesn't fail when 'default' is aliased
+        setup_datebases() doesn't fail when 'default' is aliased
         """
         tested_connections = db.ConnectionHandler({
             'default': {
@@ -257,7 +308,7 @@ class AliasedDefaultTestSetupTest(unittest.TestCase):
                 'NAME': 'dummy'
             }
         })
-        with mock.patch('django.test.runner.connections', new=tested_connections):
+        with mock.patch('django.test.utils.connections', new=tested_connections):
             runner_instance = DiscoverRunner(verbosity=0)
             old_config = runner_instance.setup_databases()
             runner_instance.teardown_databases(old_config)
@@ -280,8 +331,8 @@ class SetupDatabasesTests(unittest.TestCase):
             }
         })
 
-        with mock.patch('django.db.backends.dummy.base.DatabaseCreation') as mocked_db_creation:
-            with mock.patch('django.test.runner.connections', new=tested_connections):
+        with mock.patch('django.db.backends.dummy.base.DatabaseWrapper.creation_class') as mocked_db_creation:
+            with mock.patch('django.test.utils.connections', new=tested_connections):
                 old_config = self.runner_instance.setup_databases()
                 self.runner_instance.teardown_databases(old_config)
         mocked_db_creation.return_value.destroy_test_db.assert_called_once_with('dbname', 0, False)
@@ -305,8 +356,8 @@ class SetupDatabasesTests(unittest.TestCase):
                 'ENGINE': 'django.db.backends.dummy',
             },
         })
-        with mock.patch('django.db.backends.dummy.base.DatabaseCreation') as mocked_db_creation:
-            with mock.patch('django.test.runner.connections', new=tested_connections):
+        with mock.patch('django.db.backends.dummy.base.DatabaseWrapper.creation_class') as mocked_db_creation:
+            with mock.patch('django.test.utils.connections', new=tested_connections):
                 self.runner_instance.setup_databases()
         mocked_db_creation.return_value.create_test_db.assert_called_once_with(
             verbosity=0, autoclobber=False, serialize=True, keepdb=False
@@ -319,43 +370,107 @@ class SetupDatabasesTests(unittest.TestCase):
                 'TEST': {'SERIALIZE': False},
             },
         })
-        with mock.patch('django.db.backends.dummy.base.DatabaseCreation') as mocked_db_creation:
-            with mock.patch('django.test.runner.connections', new=tested_connections):
+        with mock.patch('django.db.backends.dummy.base.DatabaseWrapper.creation_class') as mocked_db_creation:
+            with mock.patch('django.test.utils.connections', new=tested_connections):
                 self.runner_instance.setup_databases()
         mocked_db_creation.return_value.create_test_db.assert_called_once_with(
             verbosity=0, autoclobber=False, serialize=False, keepdb=False
         )
 
 
+@skipUnlessDBFeature('supports_sequence_reset')
 class AutoIncrementResetTest(TransactionTestCase):
     """
-    Here we test creating the same model two times in different test methods,
-    and check that both times they get "1" as their PK value. That is, we test
-    that AutoField values start from 1 for each transactional test case.
+    Creating the same models in different test methods receive the same PK
+    values since the sequences are reset before each test method.
     """
 
     available_apps = ['test_runner']
 
     reset_sequences = True
 
-    @skipUnlessDBFeature('supports_sequence_reset')
-    def test_autoincrement_reset1(self):
+    def _test(self):
+        # Regular model
         p = Person.objects.create(first_name='Jack', last_name='Smith')
         self.assertEqual(p.pk, 1)
+        # Auto-created many-to-many through model
+        p.friends.add(Person.objects.create(first_name='Jacky', last_name='Smith'))
+        self.assertEqual(p.friends.through.objects.first().pk, 1)
+        # Many-to-many through model
+        b = B.objects.create()
+        t = Through.objects.create(person=p, b=b)
+        self.assertEqual(t.pk, 1)
+
+    def test_autoincrement_reset1(self):
+        self._test()
 
-    @skipUnlessDBFeature('supports_sequence_reset')
     def test_autoincrement_reset2(self):
-        p = Person.objects.create(first_name='Jack', last_name='Smith')
-        self.assertEqual(p.pk, 1)
+        self._test()
 
 
 class EmptyDefaultDatabaseTest(unittest.TestCase):
     def test_empty_default_database(self):
         """
-        Test that an empty default database in settings does not raise an ImproperlyConfigured
+        An empty default database in settings does not raise an ImproperlyConfigured
         error when running a unit test that does not use a database.
         """
         testcases.connections = db.ConnectionHandler({'default': {}})
         connection = testcases.connections[db.utils.DEFAULT_DB_ALIAS]
         self.assertEqual(connection.settings_dict['ENGINE'], 'django.db.backends.dummy')
         connections_support_transactions()
+
+
+class RunTestsExceptionHandlingTests(unittest.TestCase):
+    def test_run_checks_raises(self):
+        """
+        Teardown functions are run when run_checks() raises SystemCheckError.
+        """
+        with mock.patch('django.test.runner.DiscoverRunner.setup_test_environment'), \
+                mock.patch('django.test.runner.DiscoverRunner.setup_databases'), \
+                mock.patch('django.test.runner.DiscoverRunner.build_suite'), \
+                mock.patch('django.test.runner.DiscoverRunner.run_checks', side_effect=SystemCheckError), \
+                mock.patch('django.test.runner.DiscoverRunner.teardown_databases') as teardown_databases, \
+                mock.patch('django.test.runner.DiscoverRunner.teardown_test_environment') as teardown_test_environment:
+            runner = DiscoverRunner(verbosity=0, interactive=False)
+            with self.assertRaises(SystemCheckError):
+                runner.run_tests(['test_runner_apps.sample.tests_sample.TestDjangoTestCase'])
+            self.assertTrue(teardown_databases.called)
+            self.assertTrue(teardown_test_environment.called)
+
+    def test_run_checks_raises_and_teardown_raises(self):
+        """
+        SystemCheckError is surfaced when run_checks() raises SystemCheckError
+        and teardown databases() raises ValueError.
+        """
+        with mock.patch('django.test.runner.DiscoverRunner.setup_test_environment'), \
+                mock.patch('django.test.runner.DiscoverRunner.setup_databases'), \
+                mock.patch('django.test.runner.DiscoverRunner.build_suite'), \
+                mock.patch('django.test.runner.DiscoverRunner.run_checks', side_effect=SystemCheckError), \
+                mock.patch('django.test.runner.DiscoverRunner.teardown_databases', side_effect=ValueError) \
+                as teardown_databases, \
+                mock.patch('django.test.runner.DiscoverRunner.teardown_test_environment') as teardown_test_environment:
+            runner = DiscoverRunner(verbosity=0, interactive=False)
+            with self.assertRaises(SystemCheckError):
+                runner.run_tests(['test_runner_apps.sample.tests_sample.TestDjangoTestCase'])
+            self.assertTrue(teardown_databases.called)
+            self.assertFalse(teardown_test_environment.called)
+
+    def test_run_checks_passes_and_teardown_raises(self):
+        """
+        Exceptions on teardown are surfaced if no exceptions happen during
+        run_checks().
+        """
+        with mock.patch('django.test.runner.DiscoverRunner.setup_test_environment'), \
+                mock.patch('django.test.runner.DiscoverRunner.setup_databases'), \
+                mock.patch('django.test.runner.DiscoverRunner.build_suite'), \
+                mock.patch('django.test.runner.DiscoverRunner.run_checks'), \
+                mock.patch('django.test.runner.DiscoverRunner.teardown_databases', side_effect=ValueError) \
+                as teardown_databases, \
+                mock.patch('django.test.runner.DiscoverRunner.teardown_test_environment') as teardown_test_environment:
+            runner = DiscoverRunner(verbosity=0, interactive=False)
+            with self.assertRaises(ValueError):
+                # Suppress the output when running TestDjangoTestCase.
+                with mock.patch('sys.stderr'):
+                    runner.run_tests(['test_runner_apps.sample.tests_sample.TestDjangoTestCase'])
+            self.assertTrue(teardown_databases.called)
+            self.assertFalse(teardown_test_environment.called)
