@@ -4,27 +4,23 @@ updates.
 """
 
 from django.db import models
-from django.utils import six
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class DataPoint(models.Model):
     name = models.CharField(max_length=20)
     value = models.CharField(max_length=20)
     another_value = models.CharField(max_length=20, blank=True)
 
     def __str__(self):
-        return six.text_type(self.name)
+        return self.name
 
 
-@python_2_unicode_compatible
 class RelatedPoint(models.Model):
     name = models.CharField(max_length=20)
     data = models.ForeignKey(DataPoint, models.CASCADE)
 
     def __str__(self):
-        return six.text_type(self.name)
+        return self.name
 
 
 class A(models.Model):
@@ -50,3 +46,4 @@ class Foo(models.Model):
 
 class Bar(models.Model):
     foo = models.ForeignKey(Foo, models.CASCADE, to_field='target')
+    m2m_foo = models.ManyToManyField(Foo, related_name='m2m_foo')
