@@ -1,19 +1,16 @@
 
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
-
-GENDER_CHOICES = (
-    ('M', 'Male'),
-    ('F', 'Female'),
-)
 
 
 class Account(models.Model):
     num = models.IntegerField()
 
 
-@python_2_unicode_compatible
 class Person(models.Model):
+    GENDER_CHOICES = (
+        ('M', 'Male'),
+        ('F', 'Female'),
+    )
     name = models.CharField(max_length=20)
     gender = models.CharField(max_length=1, choices=GENDER_CHOICES)
     pid = models.IntegerField(null=True, default=None)
@@ -28,7 +25,6 @@ class Employee(Person):
     accounts = models.ManyToManyField('Account', related_name='employees', blank=True)
 
 
-@python_2_unicode_compatible
 class Profile(models.Model):
     name = models.CharField(max_length=200)
     salary = models.FloatField(default=1000.0)
