@@ -1,45 +1,19 @@
-import copy
 import datetime
 import pickle
-import unittest
+from unittest import mock
 
-from django.test import override_settings
+import pytz
+
+from django.test import SimpleTestCase, ignore_warnings, override_settings
 from django.utils import timezone
+from django.utils.deprecation import RemovedInDjango31Warning
 
-try:
-    import pytz
-except ImportError:
-    pytz = None
-
-requires_pytz = unittest.skipIf(pytz is None, "this test requires pytz")
-
-if pytz is not None:
-    CET = pytz.timezone("Europe/Paris")
+CET = pytz.timezone("Europe/Paris")
 EAT = timezone.get_fixed_timezone(180)      # Africa/Nairobi
 ICT = timezone.get_fixed_timezone(420)      # Asia/Bangkok
 
 
-class TimezoneTests(unittest.TestCase):
-
-    def test_localtime(self):
-        now = datetime.datetime.utcnow().replace(tzinfo=timezone.utc)
-        local_tz = timezone.LocalTimezone()
-        local_now = timezone.localtime(now, local_tz)
-        self.assertEqual(local_now.tzinfo, local_tz)
-
-    def test_localtime_naive(self):
-        with self.assertRaises(ValueError):
-            timezone.localtime(datetime.datetime.now())
-
-    def test_localtime_out_of_range(self):
-        local_tz = timezone.LocalTimezone()
-        long_ago = datetime.datetime(1900, 1, 1, tzinfo=timezone.utc)
-        try:
-            timezone.localtime(long_ago, local_tz)
-        except (OverflowError, ValueError) as exc:
-            self.assertIn("install pytz", exc.args[0])
-        else:
-            raise unittest.SkipTest("Failed to trigger an OverflowError or ValueError")
+class TimezoneTests(SimpleTestCase):
 
     def test_now(self):
         with override_settings(USE_TZ=True):
@@ -47,6 +21,23 @@ class TimezoneTests(unittest.TestCase):
         with override_settings(USE_TZ=False):
             self.assertTrue(timezone.is_naive(timezone.now()))
 
+    def test_localdate(self):
+        naive = datetime.datetime(2015, 1, 1, 0, 0, 1)
+        with self.assertRaisesMessage(ValueError, 'localtime() cannot be applied to a naive datetime'):
+            timezone.localdate(naive)
+        with self.assertRaisesMessage(ValueError, 'localtime() cannot be applied to a naive datetime'):
+            timezone.localdate(naive, timezone=EAT)
+
+        aware = datetime.datetime(2015, 1, 1, 0, 0, 1, tzinfo=ICT)
+        self.assertEqual(timezone.localdate(aware, timezone=EAT), datetime.date(2014, 12, 31))
+        with timezone.override(EAT):
+            self.assertEqual(timezone.localdate(aware), datetime.date(2014, 12, 31))
+
+        with mock.patch('django.utils.timezone.now', return_value=aware):
+            self.assertEqual(timezone.localdate(timezone=EAT), datetime.date(2014, 12, 31))
+            with timezone.override(EAT):
+                self.assertEqual(timezone.localdate(), datetime.date(2014, 12, 31))
+
     def test_override(self):
         default = timezone.get_default_timezone()
         try:
@@ -102,17 +93,17 @@ class TimezoneTests(unittest.TestCase):
         finally:
             timezone.deactivate()
 
-    def test_copy(self):
-        self.assertIsInstance(copy.copy(timezone.UTC()), timezone.UTC)
-        self.assertIsInstance(copy.copy(timezone.LocalTimezone()), timezone.LocalTimezone)
+    def test_override_string_tz(self):
+        with timezone.override('Asia/Bangkok'):
+            self.assertEqual(timezone.get_current_timezone_name(), 'Asia/Bangkok')
 
-    def test_deepcopy(self):
-        self.assertIsInstance(copy.deepcopy(timezone.UTC()), timezone.UTC)
-        self.assertIsInstance(copy.deepcopy(timezone.LocalTimezone()), timezone.LocalTimezone)
+    def test_override_fixed_offset(self):
+        with timezone.override(datetime.timezone(datetime.timedelta(), 'tzname')):
+            self.assertEqual(timezone.get_current_timezone_name(), 'tzname')
 
-    def test_pickling_unpickling(self):
-        self.assertIsInstance(pickle.loads(pickle.dumps(timezone.UTC())), timezone.UTC)
-        self.assertIsInstance(pickle.loads(pickle.dumps(timezone.LocalTimezone())), timezone.LocalTimezone)
+    def test_activate_invalid_timezone(self):
+        with self.assertRaisesMessage(ValueError, 'Invalid timezone: None'):
+            timezone.activate(None)
 
     def test_is_aware(self):
         self.assertTrue(timezone.is_aware(datetime.datetime(2011, 9, 1, 13, 20, 30, tzinfo=EAT)))
@@ -136,10 +127,22 @@ class TimezoneTests(unittest.TestCase):
         self.assertEqual(
             timezone.make_naive(datetime.datetime(2011, 9, 1, 17, 20, 30, tzinfo=ICT), EAT),
             datetime.datetime(2011, 9, 1, 13, 20, 30))
-        with self.assertRaises(ValueError):
+
+        with self.assertRaisesMessage(ValueError, 'make_naive() cannot be applied to a naive datetime'):
             timezone.make_naive(datetime.datetime(2011, 9, 1, 13, 20, 30), EAT)
 
-    @requires_pytz
+    def test_make_naive_no_tz(self):
+        self.assertEqual(
+            timezone.make_naive(datetime.datetime(2011, 9, 1, 13, 20, 30, tzinfo=EAT)),
+            datetime.datetime(2011, 9, 1, 5, 20, 30)
+        )
+
+    def test_make_aware_no_tz(self):
+        self.assertEqual(
+            timezone.make_aware(datetime.datetime(2011, 9, 1, 13, 20, 30)),
+            datetime.datetime(2011, 9, 1, 13, 20, 30, tzinfo=timezone.get_fixed_timezone(-300))
+        )
+
     def test_make_aware2(self):
         self.assertEqual(
             timezone.make_aware(datetime.datetime(2011, 9, 1, 12, 20, 30), CET),
@@ -147,7 +150,6 @@ class TimezoneTests(unittest.TestCase):
         with self.assertRaises(ValueError):
             timezone.make_aware(CET.localize(datetime.datetime(2011, 9, 1, 12, 20, 30)), CET)
 
-    @requires_pytz
     def test_make_aware_pytz(self):
         self.assertEqual(
             timezone.make_naive(CET.localize(datetime.datetime(2011, 9, 1, 12, 20, 30)), CET),
@@ -157,10 +159,9 @@ class TimezoneTests(unittest.TestCase):
                 pytz.timezone("Asia/Bangkok").localize(datetime.datetime(2011, 9, 1, 17, 20, 30)), CET
             ),
             datetime.datetime(2011, 9, 1, 12, 20, 30))
-        with self.assertRaises(ValueError):
+        with self.assertRaisesMessage(ValueError, 'make_naive() cannot be applied to a naive datetime'):
             timezone.make_naive(datetime.datetime(2011, 9, 1, 12, 20, 30), CET)
 
-    @requires_pytz
     def test_make_aware_pytz_ambiguous(self):
         # 2:30 happens twice, once before DST ends and once after
         ambiguous = datetime.datetime(2015, 10, 25, 2, 30)
@@ -174,7 +175,6 @@ class TimezoneTests(unittest.TestCase):
         self.assertEqual(std.tzinfo.utcoffset(std), datetime.timedelta(hours=1))
         self.assertEqual(dst.tzinfo.utcoffset(dst), datetime.timedelta(hours=2))
 
-    @requires_pytz
     def test_make_aware_pytz_non_existent(self):
         # 2:30 never happened due to DST
         non_existent = datetime.datetime(2015, 3, 29, 2, 30)
@@ -188,8 +188,38 @@ class TimezoneTests(unittest.TestCase):
         self.assertEqual(std.tzinfo.utcoffset(std), datetime.timedelta(hours=1))
         self.assertEqual(dst.tzinfo.utcoffset(dst), datetime.timedelta(hours=2))
 
-        # round trip to UTC then back to CET
-        std = timezone.localtime(timezone.localtime(std, timezone.UTC()), CET)
-        dst = timezone.localtime(timezone.localtime(dst, timezone.UTC()), CET)
-        self.assertEqual((std.hour, std.minute), (3, 30))
-        self.assertEqual((dst.hour, dst.minute), (1, 30))
+    def test_get_default_timezone(self):
+        self.assertEqual(timezone.get_default_timezone_name(), 'America/Chicago')
+
+    def test_get_default_timezone_utc(self):
+        with override_settings(USE_TZ=True, TIME_ZONE='UTC'):
+            self.assertIs(timezone.get_default_timezone(), timezone.utc)
+
+    def test_fixedoffset_timedelta(self):
+        delta = datetime.timedelta(hours=1)
+        self.assertEqual(timezone.get_fixed_timezone(delta).utcoffset(None), delta)
+
+    def test_fixedoffset_negative_timedelta(self):
+        delta = datetime.timedelta(hours=-2)
+        self.assertEqual(timezone.get_fixed_timezone(delta).utcoffset(None), delta)
+
+    @ignore_warnings(category=RemovedInDjango31Warning)
+    def test_fixedoffset_pickle(self):
+        self.assertEqual(pickle.loads(pickle.dumps(timezone.FixedOffset(0, 'tzname'))).tzname(''), 'tzname')
+
+    def test_fixedoffset_deprecation(self):
+        msg = 'FixedOffset is deprecated in favor of datetime.timezone'
+        with self.assertWarnsMessage(RemovedInDjango31Warning, msg) as cm:
+            timezone.FixedOffset()
+        self.assertEqual(cm.filename, __file__)
+
+    @ignore_warnings(category=RemovedInDjango31Warning)
+    def test_fixedoffset_utcoffset(self):
+        delta = datetime.timedelta(minutes=1)
+        self.assertEqual(timezone.FixedOffset(1).utcoffset(None), delta)
+
+    @ignore_warnings(category=RemovedInDjango31Warning)
+    def test_fixedoffset_dst(self):
+        ZERO = datetime.timedelta(minutes=0)
+        delta = datetime.timedelta(hours=0)
+        self.assertEqual(timezone.FixedOffset().dst(delta), ZERO)
