@@ -1,10 +1,7 @@
-from __future__ import unicode_literals
-
 from datetime import datetime
 
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
 def validate_answer_to_universe(value):
@@ -36,7 +33,7 @@ class ModelToValidate(models.Model):
     slug = models.SlugField(blank=True)
 
     def clean(self):
-        super(ModelToValidate, self).clean()
+        super().clean()
         if self.number == 11:
             raise ValidationError('Invalid number supplied!')
 
@@ -91,7 +88,6 @@ class Article(models.Model):
             self.pub_date = datetime.now()
 
 
-@python_2_unicode_compatible
 class Post(models.Model):
     title = models.CharField(max_length=50, unique_for_date='posted', blank=True)
     slug = models.CharField(max_length=50, unique_for_year='posted', blank=True)
@@ -134,4 +130,4 @@ try:
         auto2 = models.AutoField(primary_key=True)
 except AssertionError as exc:
     assertion_error = exc
-assert str(assertion_error) == "A model can't have more than one AutoField."
+assert str(assertion_error) == "Model validation.MultipleAutoFields can't have more than one AutoField."
