@@ -3,10 +3,8 @@ Regression tests for Django built-in views.
 """
 
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Author(models.Model):
     name = models.CharField(max_length=100)
 
@@ -17,7 +15,6 @@ class Author(models.Model):
         return '/authors/%s/' % self.id
 
 
-@python_2_unicode_compatible
 class BaseArticle(models.Model):
     """
     An abstract article Model so that we can create article models with and
