@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.core.exceptions import ImproperlyConfigured
 from django.core.servers.basehttp import get_internal_wsgi_application
 from django.core.signals import request_started
@@ -11,6 +9,7 @@ from django.test.client import RequestFactory
 
 @override_settings(ROOT_URLCONF='wsgi.urls')
 class WSGITest(SimpleTestCase):
+    request_factory = RequestFactory()
 
     def setUp(self):
         request_started.disconnect(close_old_connections)
@@ -24,7 +23,7 @@ class WSGITest(SimpleTestCase):
         """
         application = get_wsgi_application()
 
-        environ = RequestFactory()._base_environ(
+        environ = self.request_factory._base_environ(
             PATH_INFO="/",
             CONTENT_TYPE="text/html; charset=utf-8",
             REQUEST_METHOD="GET"
@@ -51,11 +50,11 @@ class WSGITest(SimpleTestCase):
         """
         FileResponse uses wsgi.file_wrapper.
         """
-        class FileWrapper(object):
+        class FileWrapper:
             def __init__(self, filelike, blksize=8192):
                 filelike.close()
         application = get_wsgi_application()
-        environ = RequestFactory()._base_environ(
+        environ = self.request_factory._base_environ(
             PATH_INFO='/file/',
             REQUEST_METHOD='GET',
             **{'wsgi.file_wrapper': FileWrapper}
